package com.glxp.udi.admin.controller.auth;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.entity.auth.*;
import com.glxp.udi.admin.req.auth.CustomerInfoFilterRequest;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.auth.AuthRoleAdminService;
import com.glxp.udi.admin.service.auth.CustomerInfoService;
import com.glxp.udi.admin.service.auth.UserRegisterService;
import com.glxp.udi.admin.service.auth.impl.CustomerContactService;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
@Slf4j
public class CustomerController {

    @Resource
    private CustomerInfoService customerInfoService;
    @Resource
    private CustomerContactService customerContactService;
    @Resource
    private UserRegisterService userRegisterService;
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private AuthRoleAdminService authRoleAdminService;


    @GetMapping("salewarehouse/customer/list")
    public BaseResponse filterList(CustomerInfoFilterRequest customerFilterRequest) {

        List<CustomerDetailEntity> customerDetailEntities = customerInfoService.filterDetailCustomer(customerFilterRequest);
        PageInfo<CustomerDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(customerDetailEntities);
        PageSimpleResponse<CustomerDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(customerDetailEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("salewarehouse/customer/detail")
    public BaseResponse customerDetail(CustomerInfoFilterRequest customerFilterRequest) {

        CustomerDetailEntity customerDetailEntity = customerInfoService.selectDetail(customerFilterRequest.getCustomerId());
        if (customerDetailEntity != null) {
            return ResultVOUtils.success(customerDetailEntity);
        }
        return ResultVOUtils.error(500, "用户信息为空");
    }


    @PostMapping("salewarehouse/customer/update")
    public BaseResponse updateCustomer(@RequestBody @Valid CustomerDetailEntity customerDetailEntity,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        CustomerInfoEntity customerInfoEntity = new CustomerInfoEntity();
        BeanUtils.copyProperties(customerDetailEntity, customerInfoEntity);
        boolean b = customerInfoService.updateCustomerInfo(customerInfoEntity);
        if (customerDetailEntity.getUserFlag() != null) {
            UpdateUserflagBean userflagBean = new UpdateUserflagBean();
            userflagBean.setCustomerId(customerDetailEntity.getCustomerId());
            userflagBean.setUserFlag(customerDetailEntity.getUserFlag());
            authAdminService.updateUserByCustomerId(userflagBean);
        }

        //更新角色
        List<Long> roles = new ArrayList<>();
        List<AuthAdmin> authAdmins = authAdminService.findByCustomerId(customerDetailEntity.getCustomerId());
        if (authAdmins != null && authAdmins.size() > 0) {
            for (AuthAdmin authAdmin : authAdmins) {
                authRoleAdminService.deleteByAdminId(authAdmin.getId());
                if (customerDetailEntity.getRoleId() != null) {
                    roles.add(Long.parseLong(customerDetailEntity.getRoleId()));
                    authRoleAdminService.insertRolesAdminIdAll(roles, authAdmin.getId());
                }
            }
        }


        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("更新成功");
    }

    @PostMapping("salewarehouse/customer/modify")
    public BaseResponse modifyCustomer(@RequestBody @Valid CustomerDetailEntity customerDetailEntity,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        CustomerInfoEntity customerInfoEntity = new CustomerInfoEntity();
        BeanUtils.copyProperties(customerDetailEntity, customerInfoEntity);
        boolean b = customerInfoService.updateCustomerInfo(customerInfoEntity);

        CustomerContactEntity customerContactEntity = new CustomerContactEntity();
        BeanUtils.copyProperties(customerDetailEntity, customerContactEntity);
        boolean c = customerContactService.updateCustomerContact(customerContactEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("更新成功");
    }

    @PostMapping("salewarehouse/customer/delete")
    public BaseResponse deleteCustomers(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String id = deleteRequest.getId();
        customerInfoService.deleteById(Long.parseLong(id));
        customerContactService.deleteById(Long.parseLong(id));
        userRegisterService.deleteByCustomerId(Long.parseLong(id));


        List<AuthAdmin> authAdmins = authAdminService.findByCustomerId(Long.parseLong(id));
        if (authAdmins != null && authAdmins.size() > 0) {
            for (int i = 0; i < authAdmins.size(); i++) {
                authRoleAdminService.deleteByAdminId(authAdmins.get(i).getId());
            }
        }
        authAdminService.deleteByCustomerId(Long.parseLong(id));
        return ResultVOUtils.success("删除成功");
    }


}
