package com.glxp.udi.admin.controller.auth;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.auth.CustomerContactEntity;
import com.glxp.udi.admin.entity.auth.CustomerInfoEntity;
import com.glxp.udi.admin.entity.auth.UserRegisterEntity;
import com.glxp.udi.admin.entity.info.CompanyEntity;
import com.glxp.udi.admin.req.auth.RegisterCheckRequest;
import com.glxp.udi.admin.req.auth.ResetPasswdRequest;
import com.glxp.udi.admin.req.auth.UserResisterFilterRequest;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.auth.CheckCodeRequest;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.auth.AuthRoleAdminService;
import com.glxp.udi.admin.service.auth.CustomerInfoService;
import com.glxp.udi.admin.service.auth.UserRegisterService;
import com.glxp.udi.admin.service.auth.impl.CustomerContactService;
import com.glxp.udi.admin.service.info.CompanyService;
import com.glxp.udi.admin.util.DateUtil;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
@Slf4j
public class RegisterController {

    private static String Url = "http://106.ihuyi.com/webservice/sms.php?method=Submit";

    @Resource
    UserRegisterService userRegisterService;
    @Resource
    private AuthRoleAdminService authRoleAdminService;
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private CustomerInfoService customerInfoService;
    @Resource
    private CustomerContactService customerContactService;

    @Resource
    private CompanyService companyService;

    @GetMapping("salewarehouse/register/list")
    public BaseResponse filterList(UserResisterFilterRequest userResisterFilterRequest) {

        List<UserRegisterEntity> userRegisterEntities = userRegisterService.filterUserRegisters(userResisterFilterRequest);
        PageInfo<UserRegisterEntity> pageInfo;
        pageInfo = new PageInfo<>(userRegisterEntities);
        PageSimpleResponse<UserRegisterEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(userRegisterEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("salewarehouse/register/getCheckcode")
    public BaseResponse getCheckcode(@RequestBody CheckCodeRequest checkCodeRequest, HttpSession httpSession) {

        if (checkCodeRequest.getUserName() != null) {
            AuthAdmin authAdmin = authAdminService.findByUserName(checkCodeRequest.getUserName());
            if (authAdmin == null) {
                return ResultVOUtils.error(500, "未找到该用户");
            }
            UserRegisterEntity userRegisterEntity = userRegisterService.selectByCustomerId(authAdmin.getCustomerId());

            if (!userRegisterEntity.getMobile().equals(checkCodeRequest.getPhoneNum())) {
                return ResultVOUtils.error(500, "手机号与注册时不一致");
            }
        }

        RestTemplate restTemplate = new RestTemplate();
        int mobile_code = (int) ((Math.random() * 9 + 1) * 100000);
//        if(mobile_code>1){
//            JSONObject json = new JSONObject();
//            json.put("memPhone", checkCodeRequest.getPhoneNum());
//            json.put("code", mobile_code);
//            json.put("createTime", System.currentTimeMillis());
//            // 将认证码存入SESSION
//            httpSession.setAttribute("code", json);
//            log.info(httpSession.getId() + "---" + httpSession);
//            return ResultVOUtils.success("发送成功" + "   " + mobile_code);
//        }

        MultiValueMap<String, Object> postParameters = new LinkedMultiValueMap<>();
        int time = 5;
        String content = new String("您正在进行手机验证，验证码是" + mobile_code + "，" + time + "分钟内有效。");
        postParameters.add("account", "C07086222");
        postParameters.add("password", "2dddbbf73636c193c5903324bdb47c5c");
        postParameters.add("mobile", checkCodeRequest.getPhoneNum());
        postParameters.add("content", content);
        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Type", "application/x-www-form-urlencoded;charset=GBK");
        HttpEntity<MultiValueMap<String, Object>> httpEntity = new HttpEntity<>(postParameters, headers);
        String response = restTemplate.postForObject(Url, httpEntity, String.class);
        if (response.contains("提交成功")) {
            JSONObject json = new JSONObject();
            json.put("memPhone", checkCodeRequest.getPhoneNum());
            json.put("code", mobile_code);
            json.put("createTime", System.currentTimeMillis());
            // 将认证码存入SESSION
            httpSession.setAttribute("code", json);
            log.info(httpSession.getId() + "---" + httpSession);
            return ResultVOUtils.success("发送成功" + "   " + mobile_code);
        } else
            return ResultVOUtils.error(500, "验证码发送失败");

    }


    @PostMapping("salewarehouse/register/deleteResiter")
    public BaseResponse deleteResiter(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String id = deleteRequest.getId();
        userRegisterService.deleteById(Long.parseLong(id));
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("salewarehouse/register/update")
    public BaseResponse updateRegister(@RequestBody @Valid UserRegisterEntity userRegisterEntity,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = userRegisterService.updateUserRegister(userRegisterEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("添加成功");
    }

    @PostMapping("salewarehouse/register/save")
    public BaseResponse saveRegister(@RequestBody @Valid UserRegisterEntity userRegisterEntity, HttpSession httpSession,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        log.info(httpSession.getId());
        JSONObject userCode = (JSONObject) httpSession.getAttribute("code");
        if (userCode == null) {
            return ResultVOUtils.error(500, "验证码已失效，请重新获取");
        }
        long lastTime = userCode.getLong("createTime");
        int checkCode = (Integer) userCode.get("code");
        String phoneNum = userCode.getString("memPhone");
        if ((System.currentTimeMillis() - lastTime) > 1000 * 60 * 5) {
            return ResultVOUtils.error(500, "验证码已过期，请重新获取");
        }
        if (checkCode != userRegisterEntity.getCheckCode()) {
            return ResultVOUtils.error(500, "验证码错误，请重新获取");
        }
        if (!phoneNum.equals(userRegisterEntity.getMobile())) {
            return ResultVOUtils.error(500, "手机号与获取验证码不一致");
        }
        boolean isExitPhone = userRegisterService.isExit(userRegisterEntity.getMobile());
        if (isExitPhone) {
            return ResultVOUtils.error(500, "该手机号已被注册");
        }
        AuthAdmin authAdmin = authAdminService.findByUserName(userRegisterEntity.getUserId());
        if (authAdmin != null) {
            return ResultVOUtils.error(500, "该用户账号已被注册");
        }
        userRegisterEntity.setRegisterTime(DateUtil.getDateTime());
        userRegisterEntity.setCheckType(0);//未审核
        boolean b = userRegisterService.insertUserRegister(userRegisterEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("添加成功");
    }

    @PostMapping("salewarehouse/reset/passwd")
    public BaseResponse resetPasswd(@RequestBody @Valid ResetPasswdRequest resetPasswdRequest, HttpSession httpSession,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        log.info(httpSession.getId());
        JSONObject userCode = (JSONObject) httpSession.getAttribute("code");
        if (userCode == null) {
            return ResultVOUtils.error(500, "验证码已失效，请重新获取");
        }
        long lastTime = userCode.getLong("createTime");
        int checkCode = (Integer) userCode.get("code");
        String phoneNum = userCode.getString("memPhone");
        if ((System.currentTimeMillis() - lastTime) > 1000 * 60 * 5) {
            return ResultVOUtils.error(500, "验证码已过期，请重新获取");
        }
        if (checkCode != resetPasswdRequest.getCheckCode()) {
            return ResultVOUtils.error(500, "验证码错误，请重新获取");
        }
        if (!phoneNum.equals(resetPasswdRequest.getMobile())) {
            return ResultVOUtils.error(500, "手机号与获取验证码不一致");
        }
        AuthAdmin authAdmin = authAdminService.findByUserName(resetPasswdRequest.getUserName());
        if (authAdmin == null) {
            return ResultVOUtils.error(500, "该用户不存在");
        }
        CustomerContactEntity customerContactEntity = customerContactService.selectById(authAdmin.getCustomerId());
        if (customerContactEntity != null && customerContactEntity.getMobile().equals(phoneNum)) {
            authAdmin.setPassWord(resetPasswdRequest.getPassword());
            authAdminService.updateAuthAdmin(authAdmin);
        } else {
            return ResultVOUtils.error(500, "手机号与该用户不匹配");
        }


//        if (!b) {
//            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
//        }
        return ResultVOUtils.success("密码修改成功");
    }

    @PostMapping("salewarehouse/register/check")
    public BaseResponse checkRegister(@RequestBody @Valid RegisterCheckRequest registerCheckRequest,
                                      BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        long userId = generateUserId();
        UserRegisterEntity userRegisterEntity = userRegisterService.selectById(registerCheckRequest.getId());
        userRegisterEntity.setCheckType(registerCheckRequest.getCheckType());
        userRegisterEntity.setCheckTime(DateUtil.getDateTime());
        userRegisterEntity.setUserId(userId + "");
        boolean b = userRegisterService.updateUserRegister(userRegisterEntity);
        if (registerCheckRequest.getCheckType() != 1) {
            return ResultVOUtils.success("修改成功");
        }
        //创建新用户
        AuthAdmin authAdmin = new AuthAdmin();
        authAdmin.setCustomerId(userId);
        authAdmin.setPassWord(userRegisterEntity.getPassword());
        authAdmin.setCreateTime(DateUtil.parseDate(userRegisterEntity.getCheckTime()));
        authAdmin.setUserName(userRegisterEntity.getMobile());  //用手机号当用户名，通过手机号登录
        authAdmin.setEmployeeName(userRegisterEntity.getNickName());
        authAdmin.setUserFlag(1);
        authAdminService.insertAuthAdmin(authAdmin);
        authAdmin = authAdminService.findByUserName(authAdmin.getUserName());

        //分配角色
//        List<Long> roles = new ArrayList<>();
//        roles.add(Long.valueOf(13));//默认角色


        //生产客户信息表
        CustomerInfoEntity customerInfoEntity = new CustomerInfoEntity();
        customerInfoEntity.setCustomerId(authAdmin.getCustomerId());
        customerInfoEntity.setArea(userRegisterEntity.getArea());
        customerInfoEntity.setDetailAddr(userRegisterEntity.getDetailAddr());
        customerInfoEntity.setBussinessStatus(userRegisterEntity.getBussinessStatus());
        customerInfoEntity.setCreditNum(userRegisterEntity.getCreditNum());
        customerInfoEntity.setCustomerName(userRegisterEntity.getNickName());
        customerInfoEntity.setUserFlag(1);
        customerInfoEntity.setUserMax(2);
        customerInfoEntity.setRoleId(registerCheckRequest.getRoles() + "");
        customerInfoEntity.setCompanyName(userRegisterEntity.getCompanyName());
        List<Long> roles = new ArrayList<>();

        if (registerCheckRequest.getRoles() != null) {
            roles.add(registerCheckRequest.getRoles());
            authRoleAdminService.insertRolesAdminIdAll(roles, authAdmin.getId());
        }
        customerInfoService.insertCustomerInfo(customerInfoEntity);

        //生产客户信息联系人表
        CustomerContactEntity customerContactEntity = new CustomerContactEntity();
        customerContactEntity.setContacts(userRegisterEntity.getRealName());
        customerContactEntity.setCustomerId(userId);
        customerContactEntity.setEmail(userRegisterEntity.getEmail());
        customerContactEntity.setMobile(userRegisterEntity.getMobile());
        customerContactEntity.setTel(userRegisterEntity.getTel());
        customerContactService.insertCustomerContact(customerContactEntity);


        //生产本企业信息
        CompanyEntity companyEntity = new CompanyEntity();
        companyEntity.setCustomerId(userId);
        companyEntity.setArea(userRegisterEntity.getArea());
        companyEntity.setAreaCode(userRegisterEntity.getAreaCode());
        companyEntity.setDetailAddr(userRegisterEntity.getDetailAddr());
        companyEntity.setContacts(customerContactEntity.getContacts());
        companyEntity.setEmail(userRegisterEntity.getEmail());
        companyEntity.setMobile(userRegisterEntity.getMobile());
        companyEntity.setTel(userRegisterEntity.getTel());
        companyEntity.setCompanyName(userRegisterEntity.getCompanyName());
        companyEntity.setClasses(userRegisterEntity.getBussinessStatus() + "");
        companyEntity.setCreditNum(userRegisterEntity.getCreditNum());

        companyService.insertCompany(companyEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("添加成功");
    }

    public long generateUserId() {
        String userId = System.currentTimeMillis() + "";
        userId = userId.substring(1, userId.length());
        int max = 100, min = 1;
        int ran2 = (int) (Math.random() * (max - min) + min);
        userId = userId + ran2;
        return Long.parseLong(userId);
    }


}
