package com.glxp.udi.admin.controller.device;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.dao.udid.DeviceMapper;
import com.glxp.udi.admin.dao.udid.DevicedownloadMapper;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.info.CompanyEntity;
import com.glxp.udi.admin.entity.udid.*;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.udid.*;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.udid.ProgressResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.info.CompanyService;
import com.glxp.udi.admin.service.inout.DeviceService;
import com.glxp.udi.admin.service.inout.DruginfoService;
import com.glxp.udi.admin.service.publish.UdiDeviceIdService;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
public class DeviceController {

    @Resource
    DeviceMapper deviceMapper;
    @Resource
    DruginfoService druginfoService;
    @Resource
    AuthAdminService authAdminService;

    @Resource
    private DeviceService deviceService;
    @Resource
    private DevicedownloadMapper devicedownloadMapper;
    @Resource
    CompanyService companyService;


    @GetMapping("warehouse/device/list")
    public BaseResponse listDevices(@Valid DeviceListRequest deviceListRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String key = deviceListRequest.getKey();
        boolean isScan = false;
        if (key != null && key.length() > 22) {
            key = key.substring(2, 16);
            deviceListRequest.setKey(key);
            isScan = true;
        }
        List<DeviceEntity> medicalResonseList = deviceService.listDevices(deviceListRequest);
        long total = 0;
        PageInfo<DeviceEntity> pageInfo = new PageInfo<>(medicalResonseList);
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        List<Devicedownload> devicedownloads = devicedownloadMapper.selectByExample(new DevicedownloadExample());
        Devicedownload devicedownload;
        if (devicedownloads != null && devicedownloads.size() > 0) {
            devicedownload = devicedownloads.get(0);
            total = devicedownload.getAlltotal() - devicedownload.getAllindex();
        }
        if (isScan) {
            total = 20;
            pageSimpleResponse.setTotal(total);
        } else
            pageSimpleResponse.setTotal(total * 10);
        pageSimpleResponse.setList(medicalResonseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("warehouse/device/search")
    public BaseResponse searchByKey(@Valid DeviceListRequest deviceListRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String key = deviceListRequest.getKey();
        if (key != null && key.length() > 22) {
            key = key.substring(2, 16);
            deviceListRequest.setKey(key);
        }
        Device device = deviceService.searchByBs(key);
        long total = 1;
        DeviceEntity deviceEntity = new DeviceEntity();
        if (device != null)
            BeanUtils.copyProperties(device, deviceEntity);
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        List<DeviceEntity> deviceEntityList = new ArrayList<>();
        deviceEntityList.add(deviceEntity);
        pageSimpleResponse.setList(deviceEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("warehouse/device/searchbydi")
    public BaseResponse searchByDi(@Valid DeviceListRequest deviceListRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String key = deviceListRequest.getKey();
        Device device = deviceService.searchByZxxscpbs(key);
        long total = 1;
        DeviceEntity deviceEntity = new DeviceEntity();
        if (device != null)
            BeanUtils.copyProperties(device, deviceEntity);
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        List<DeviceEntity> deviceEntityList = new ArrayList<>();
        deviceEntityList.add(deviceEntity);
        pageSimpleResponse.setList(deviceEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }
    @GetMapping("warehouse/device/unionSearch")
    public BaseResponse search(@Valid DeviceSearchRequest deviceSearchRequest, BindingResult bindingResult) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        deviceSearchRequest.setCustomerId(authAdmin.getCustomerId()+"");
        List<DeviceEntity> medicalResonseList = deviceService.search(deviceSearchRequest);
        long total = deviceService.seatchTotal(deviceSearchRequest);
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        pageSimpleResponse.setList(medicalResonseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Resource
    UdiDeviceIdService udiDeviceIdService;
    @GetMapping("warehouse/device/downloadAll")
    public BaseResponse downloadAll() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return ResultVOUtils.success("下载成功");
    }

    @GetMapping("warehouse/device/combineSearch")
    public BaseResponse combineSearch(@Valid DeviceSearchRequest deviceSearchRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<DeviceEntity> medicalResonseList = deviceService.search(deviceSearchRequest);

        long total = 0;
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        List<Devicedownload> devicedownloads = devicedownloadMapper.selectByExample(new DevicedownloadExample());
        Devicedownload devicedownload;
        if (devicedownloads != null && devicedownloads.size() > 0) {
            devicedownload = devicedownloads.get(0);
            total = devicedownload.getAlltotal() - devicedownload.getAllindex();
        }

        for (int i = 0; i < medicalResonseList.size(); i++) {
            String nameCode = medicalResonseList.get(i).getZxxsdycpbs();
            if (druginfoService.isExit(nameCode)) {
                medicalResonseList.get(i).setCheck(true);
            } else
                medicalResonseList.get(i).setCheck(false);
        }

        pageSimpleResponse.setTotal(total * 10);
        pageSimpleResponse.setList(medicalResonseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("")
    @PostMapping("warehouse/device/loadsingle")
    public BaseResponse downLoadDiii(@RequestBody DeviceDiRequest deviceRequest) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        CompanyEntity companyEntity = companyService.findCompany(authAdmin.getCustomerId());
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyEntity);
        if (deviceRequest.getPrimaryDeviceId() == null) {
            return ResultVOUtils.error(999, "最小销售单元标识不能为空");
        }
        new Thread(() -> {

            TokenEntity token = downloadDeviceHelper.getToken();
            DownloadDiRequest downloadDiRequest1 = new DownloadDiRequest();
            downloadDiRequest1.setAccessToken(token.getAccessToken());
            downloadDiRequest1.setPrimaryDeviceId(deviceRequest.getPrimaryDeviceId());
            int result = downloadDeviceHelper.downloadBydi(authAdmin.getCustomerId()+"",downloadDiRequest1, deviceService);
            if (result == -1) {
                return;// ResultVOUtils.error(505, "下载出错");
            }

        }).start();
        return ResultVOUtils.success("开始下载");
    }

    @PostMapping("warehouse/device/downLoadHisory")
    public BaseResponse downLoadHisory(@RequestBody DeviceDiRequest deviceRequest) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        CompanyEntity companyEntity = companyService.findCompany(authAdmin.getCustomerId());
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyEntity);
        if (deviceRequest.getPrimaryDeviceId() == null) {
            return ResultVOUtils.error(999, "最小销售单元标识不能为空");
        }
        new Thread(() -> {
            String deviceRecordKey = deviceService.selectKey(deviceRequest.getPrimaryDeviceId());
            TokenEntity token = downloadDeviceHelper.getToken();
            DownloadHistoryRequest deviceHistoryRequest = new DownloadHistoryRequest();
            deviceHistoryRequest.setAccessToken(token.getAccessToken());
            deviceHistoryRequest.setCurrentPageNumber("1");
            List<DownloadHistoryRequest.DataSetBean> dataSetBeans = new ArrayList<>();
            DownloadHistoryRequest.DataSetBean keys = new DownloadHistoryRequest.DataSetBean();
            keys.setDeviceRecordKey(deviceRecordKey);
            dataSetBeans.add(keys);
            deviceHistoryRequest.setDataSet(dataSetBeans);
            int result = downloadDeviceHelper.downloadHistory(authAdmin.getCustomerId()+"",deviceHistoryRequest, deviceService);
            if (result == -1) {
                return;// ResultVOUtils.error(505, "下载出错");
            }

        }).start();
        return ResultVOUtils.success("开始下载");
    }



    @PostMapping("warehouse/device/loadudid")
    public BaseResponse downLoadAll(DownloadDeviceRequest downloadDeviceRequest, BindingResult bindingResult) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        CompanyEntity companyEntity = companyService.findCompany(authAdmin.getCustomerId());
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyEntity);
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        new Thread(() -> {

            TokenEntity token = downloadDeviceHelper.getToken();
            int total = downloadDeviceHelper.getTotal(token.getAccessToken());
            int curpage = total;
            Devicedownload devicedownload = devicedownloadMapper.selectByPrimaryKey(authAdmin.getCustomerId() + "");
            if (devicedownload != null) {
                int offset = devicedownload.getAlltotal() - devicedownload.getAllindex();
                curpage = total - offset;
            } else {
                devicedownload = new Devicedownload();
                devicedownload.setId(authAdmin.getCustomerId() + "");
                devicedownload.setAlltotal(100);
                devicedownload.setAllindex(100);
                devicedownloadMapper.insert(devicedownload);
            }
            devicedownload.setIsdownload(1);
            long startTime = System.currentTimeMillis();
            devicedownload.setStarttime(startTime + "");
            devicedownloadMapper.updateByPrimaryKey(devicedownload);
            Devicedownload stop = devicedownloadMapper.selectByPrimaryKey(authAdmin.getCustomerId() + "");

            while (curpage > 0 && stop.getIsdownload() == 1 && stop.getStarttime().equals(startTime + "")) {

                DeviceRequest deviceRequest = new DeviceRequest();
                deviceRequest.setAccessToken(token.getAccessToken());
                deviceRequest.setRequestType(3 + "");
                deviceRequest.setCurrentPageNumber(curpage + "");
                int result = downloadDeviceHelper.downloadProduct(authAdmin.getCustomerId()+"",deviceService, deviceRequest);
                if (result == -1) {
                    return;// ResultVOUtils.error(505, "下载出错");
                }
                curpage--;
                Devicedownload devicedownload1 = new Devicedownload();
                devicedownload1.setAlltotal(total);
                devicedownload1.setAllindex(curpage);
                devicedownload1.setId(authAdmin.getCustomerId() + "");
                devicedownloadMapper.updateProgress(devicedownload1);
                stop = devicedownloadMapper.selectByPrimaryKey(authAdmin.getCustomerId() + "");
            }
        }).start();
        return ResultVOUtils.success("开始下载");
    }

    @GetMapping("warehouse/device/stop")
    public BaseResponse stopDown() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        Devicedownload devicedownload = devicedownloadMapper.selectByPrimaryKey(authAdmin.getCustomerId()+"");
        devicedownload.setIsdownload(2);
        devicedownloadMapper.updateByPrimaryKey(devicedownload);
        return ResultVOUtils.success("停止成功");
    }

    @GetMapping("warehouse/device/progress")
    public BaseResponse getProgress() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        Devicedownload devicedownload = devicedownloadMapper.selectByPrimaryKey(authAdmin.getCustomerId()+"");
        ProgressResponse response = new ProgressResponse();
        if (devicedownload != null) {
            double index = devicedownload.getAlltotal() - devicedownload.getAllindex();
            double total = devicedownload.getAlltotal();
            double progress = 0;
            if (total > 0) {
                progress = index / total;
            }
            response.setTotal(total * 10);
            response.setIndex(index * 10);
            response.setProgress(progress * 100);
        } else {
            response.setTotal(1000);
            response.setIndex(0);
            response.setProgress(0);
        }

        return ResultVOUtils.success(response);
    }


}
