package com.glxp.udi.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.info.EnterpriseCertEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.info.DeleteEnterpriseCertRequest;
import com.glxp.udi.admin.req.info.FilterEnterpriseCertRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.info.EnterpriseCertService;
import com.glxp.udi.admin.util.UuidUtils;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.List;

@RestController
public class EnterpriseCertController {

    @Resource
    private EnterpriseCertService enterpriseCertService;
    @Resource
    private AuthAdminService authAdminService;
    @AuthRuleAnnotation("")
    @PostMapping("/udims/enterprise/add")
    public BaseResponse addEnterpriseCert(@RequestBody @Valid EnterpriseCertEntity enterpriseCertEntity, BindingResult bindingResult) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));

        enterpriseCertEntity.setUDI_RegistrationOID(UuidUtils.getUUId());
        enterpriseCertEntity.setUDI_Enterprise_FK(authAdmin.getCustomerId()+"");
        boolean b = enterpriseCertService.insertEnterpriseCert(enterpriseCertEntity);
        if (!b)
            return ResultVOUtils.success("添加失败");
        return ResultVOUtils.success("添加成功");
    }

    @AuthRuleAnnotation("")
    @PostMapping("/udims/enterprise/delete")
    public BaseResponse deleteEnterpriseCert(@RequestBody @Valid DeleteEnterpriseCertRequest deleteEnterpriseCertRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = enterpriseCertService.deleteEnterpriseCert(deleteEnterpriseCertRequest);
        if (!b) {
            return ResultVOUtils.error(500, "删除失败");
        }
        return ResultVOUtils.success("删除成功");
    }


    @AuthRuleAnnotation("")
    @PostMapping("/udims/enterprise/modify")
    public BaseResponse modifyEnterpriseCert(@RequestBody @Valid EnterpriseCertEntity enterpriseCertEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = enterpriseCertService.modifyEnterpriseCert(enterpriseCertEntity);
        if (!b)
            return ResultVOUtils.success("修改失败");
        return ResultVOUtils.success("修改成功");
    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/enterprise/list")
    public BaseResponse listEnterpriseCert(@Valid FilterEnterpriseCertRequest filterEnterpriseCertRequest, BindingResult bindingResult) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        filterEnterpriseCertRequest.setCustomerId(authAdmin.getCustomerId()+"");  //暂时取消
        List<EnterpriseCertEntity> enterpriseCertEntities = enterpriseCertService.filterEnterpriseCerts(filterEnterpriseCertRequest);
        PageInfo<EnterpriseCertEntity> pageInfo = new PageInfo<>(enterpriseCertEntities);
        PageSimpleResponse<EnterpriseCertEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(enterpriseCertEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


}
