package com.glxp.udi.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.info.FacidentcodeEntity;
import com.glxp.udi.admin.entity.info.IdentifierEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.info.DeleteFacidentcodeRequest;
import com.glxp.udi.admin.req.info.FilterFacidentcodeRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.basic.AllFCodeResponse;
import com.glxp.udi.admin.res.basic.FacidentcodeResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.info.FacidentcodeService;
import com.glxp.udi.admin.service.info.IdentifierService;
import com.glxp.udi.admin.util.UuidUtils;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
public class FacidentcodeController {


    @Resource
    private FacidentcodeService facidentcodeService;
    @Resource
    private IdentifierService identifierService;

    @Resource
    private AuthAdminService authAdminService;

    @AuthRuleAnnotation("")
    @PostMapping("/udims/facidentcode/add")
    public BaseResponse addFacidentcode(@RequestBody @Valid FacidentcodeEntity facidentcodeEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();

        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        if (facidentcodeEntity != null) {
            facidentcodeEntity.setUDI_FacIdentCodeOID(UuidUtils.getUUId());
            facidentcodeEntity.setUDI_Customer_FK(authAdmin.getCustomerId() + "");
        }

        boolean b = facidentcodeService.insertFacidentcode(facidentcodeEntity);
        if (!b)
            return ResultVOUtils.success("添加失败");
        return ResultVOUtils.success("添加成功");
    }

    @AuthRuleAnnotation("")
    @PostMapping("/udims/facidentcode/delete")
    public BaseResponse deleteFacidentcode(@RequestBody @Valid DeleteFacidentcodeRequest deleteFacidentcodeRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        if(!identifierService.isUsed(deleteFacidentcodeRequest.getId())){
            boolean b = facidentcodeService.deleteFacidentcode(deleteFacidentcodeRequest);
            identifierService.deleteIdentifierByFacidentcode(deleteFacidentcodeRequest.getId());
            if (!b) {
                return ResultVOUtils.error(500, "删除失败");
            }
            return ResultVOUtils.success("删除成功");
        }else {
            return ResultVOUtils.error(500, "生成的产品项目码已被使用，不能删除");
        }

    }


    @AuthRuleAnnotation("")
    @PostMapping("/udims/facidentcode/modify")
    public BaseResponse modifyFacidentcode(@RequestBody @Valid FacidentcodeEntity facidentcodeEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = facidentcodeService.modifyFacidentcode(facidentcodeEntity);
        if (!b)
            return ResultVOUtils.success("修改失败");
        return ResultVOUtils.success("修改成功");
    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/facidentcode/list")
    public BaseResponse listFacidentcode(FilterFacidentcodeRequest filterFacidentcodeRequest) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();

        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        filterFacidentcodeRequest.setCustomerId(authAdmin.getCustomerId() + "");
        List<FacidentcodeResponse> facidentcodeEntities = facidentcodeService.selectByCustomerId(filterFacidentcodeRequest);
        PageInfo<FacidentcodeResponse> pageInfo = new PageInfo<>(facidentcodeEntities);
        PageSimpleResponse<FacidentcodeResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(facidentcodeEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    //生成项目码
    @AuthRuleAnnotation("")
    @PostMapping("/udims/facidentcode/generate")
    public BaseResponse generateIdentify(@RequestBody @Valid FacidentcodeEntity facidentcodeEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        int count = Integer.parseInt(facidentcodeEntity.getNoDigit());
        int max = (int) Math.pow(10, count);
        String serNum = null;
        List<IdentifierEntity> identifierEntities = new ArrayList<>();
        for (int i = 0; i < max; i++) {
            serNum = String.format("%0" + count + "d", i);
            IdentifierEntity identifierEntity = new IdentifierEntity();
            identifierEntity.setSeqNumber(serNum);
            identifierEntity.setUDI_IdentifierOID(UuidUtils.getUUId());
            identifierEntity.setIdentifierCode(facidentcodeEntity.getIndentCode() + serNum);
            identifierEntity.setUDI_FacIdentCode_FK(facidentcodeEntity.getUDI_FacIdentCodeOID());
            identifierEntity.setIsGTIN("0");
            identifierEntity.setIsReg("0");
            identifierEntity.setIsSPTM("0");
            identifierEntity.setIsUDI("0");
            identifierEntities.add(identifierEntity);
        }
        boolean b = identifierService.insertIdentifierList(identifierEntities);
        facidentcodeEntity.setIsGenerate(1);
        facidentcodeService.modifyFacidentcode(facidentcodeEntity);
        if (!b)
            return ResultVOUtils.success("生成失败");
        return ResultVOUtils.success("生成成功");
    }


    @AuthRuleAnnotation("")
    @GetMapping("/udims/facidentcode/all")
    public BaseResponse listFacidentcode() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();

        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        List<String> facidentcodeEntities = facidentcodeService.selectAll(authAdmin.getCustomerId() + "");
        List<AllFCodeResponse> allFCodeResponses = new ArrayList<>();
        if (facidentcodeEntities != null && facidentcodeEntities.size() > 0) {
            for (String str : facidentcodeEntities) {
                AllFCodeResponse allFCodeResponse = new AllFCodeResponse();
                allFCodeResponse.setLabel(str);
                allFCodeResponse.setValue(str);
                allFCodeResponses.add(allFCodeResponse);
            }
        }
        return ResultVOUtils.success(allFCodeResponses);
    }


}
