package com.glxp.udi.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.info.IdentifierEntity;
import com.glxp.udi.admin.req.info.DeleteIdentifierRequest;
import com.glxp.udi.admin.req.info.FilterIdentifierRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.info.IdentifierService;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

//产商识别码
@RestController
public class IdentifyCodeController {

    @Resource
    private IdentifierService identifierService;
    @Resource
    private AuthAdminService authAdminService;
    @AuthRuleAnnotation("")
    @PostMapping("/udims/identifier/add")
    public BaseResponse addIdentifier(@RequestBody @Valid IdentifierEntity identifierEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = identifierService.insertIdentifier(identifierEntity);
        if (!b)
            return ResultVOUtils.success("添加失败");
        return ResultVOUtils.success("添加成功");
    }

    @AuthRuleAnnotation("")
    @PostMapping("/udims/identifier/delete")
    public BaseResponse deleteIdentifier(@RequestBody @Valid DeleteIdentifierRequest deleteIdentifierRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = identifierService.deleteIdentifier(deleteIdentifierRequest);
        if (!b) {
            return ResultVOUtils.error(500, "删除失败");
        }
        return ResultVOUtils.success("删除成功");
    }


    @AuthRuleAnnotation("")
    @PostMapping("/udims/identifier/modify")
    public BaseResponse modifyIdentifier(@RequestBody @Valid IdentifierEntity identifierEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = identifierService.modifyIdentifier(identifierEntity);
        if (!b)
            return ResultVOUtils.success("修改失败");
        return ResultVOUtils.success("修改成功");
    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/identifier/list")
    public BaseResponse listIdentifier(@Valid FilterIdentifierRequest filterIdentifierRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<IdentifierEntity> identifierEntities = identifierService.selectByfacident(filterIdentifierRequest);
        PageInfo<IdentifierEntity> pageInfo = new PageInfo<>(identifierEntities);
        PageSimpleResponse<IdentifierEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(identifierEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    //随机获取标识
    @AuthRuleAnnotation("")
    @GetMapping("/udims/identifier/random")
    public BaseResponse randomIdentifier(@Valid FilterIdentifierRequest filterIdentifierRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String code = identifierService.randomCode(filterIdentifierRequest);
        return ResultVOUtils.success(code);
    }

}
