package com.glxp.udi.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.info.StandagencyEntity;
import com.glxp.udi.admin.req.info.DeleteStandagencyRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.info.StandagencyService;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

@RestController
public class StandagencyController {

    @Resource
    private StandagencyService standagencyService;

    @AuthRuleAnnotation("")
    @PostMapping("/udims/standagency/add")
    public BaseResponse addstandagency(@RequestBody @Valid StandagencyEntity standagencyEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = standagencyService.insertStandagency(standagencyEntity);
        if (!b)
            return ResultVOUtils.success("添加失败");
        return ResultVOUtils.success("添加成功");
    }

    @AuthRuleAnnotation("")
    @PostMapping("/udims/standagency/delete")
    public BaseResponse deletestandagency(@RequestBody @Valid DeleteStandagencyRequest deletestandagencyRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = standagencyService.deleteStandagency(deletestandagencyRequest);
        if (!b) {
            return ResultVOUtils.error(500, "删除失败");
        }
        return ResultVOUtils.success("删除成功");
    }


    @AuthRuleAnnotation("")
    @PostMapping("/udims/standagency/modify")
    public BaseResponse modifystandagency(@RequestBody @Valid StandagencyEntity standagencyEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = standagencyService.modifyStandagency(standagencyEntity);
        if (!b)
            return ResultVOUtils.success("修改失败");
        return ResultVOUtils.success("修改成功");
    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/standagency/list")
    public BaseResponse liststandagency() {
        List<StandagencyEntity> standagencyEntities = standagencyService.selectAll();
        PageInfo<StandagencyEntity> pageInfo = new PageInfo<>(standagencyEntities);
        PageSimpleResponse<StandagencyEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(standagencyEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


}
