package com.glxp.udi.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.auth.CustomerInfoEntity;
import com.glxp.udi.admin.entity.inout.BusRoleRlEntity;
import com.glxp.udi.admin.entity.inout.BussinessTypeEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.inout.BusRoleRlFilterRequest;
import com.glxp.udi.admin.req.receipt.BussinessTypeFilterRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.inout.BusRoleRlResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.auth.CustomerInfoService;
import com.glxp.udi.admin.service.inout.BusRoleRlService;
import com.glxp.udi.admin.service.inout.BussinessTypeService;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

@RestController
public class BusRoleRlController {

    @Resource
    private BusRoleRlService busRoleRlService;
    @Resource
    private CustomerInfoService customerInfoService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private AuthAdminService authAdminService;

    @GetMapping("/udiwms/busRl/filter")
    public BaseResponse filterbBusRl(BusRoleRlFilterRequest busRoleRlFilterRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BusRoleRlResponse> busRoleRlResponses;
        busRoleRlResponses = busRoleRlService.filterJoinList(busRoleRlFilterRequest);
        PageInfo<BusRoleRlResponse> pageInfo;
        pageInfo = new PageInfo<>(busRoleRlResponses);
        PageSimpleResponse<BusRoleRlResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(busRoleRlResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/busRl/filterAll")
    public BaseResponse filterbBusRlAll(BusRoleRlFilterRequest busRoleRlFilterRequest,
                                        BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BussinessTypeFilterRequest bussinessTypeFilterRequest = new BussinessTypeFilterRequest();
        BeanUtils.copyProperties(busRoleRlFilterRequest, bussinessTypeFilterRequest);
        List<BussinessTypeEntity> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterList(bussinessTypeFilterRequest);

        List<BusRoleRlEntity> busRoleRlResponses;
        busRoleRlResponses = busRoleRlService.filterList(busRoleRlFilterRequest);

        for (BussinessTypeEntity bussinessTypeEntity : bussinessTypeEntities) {
            for (BusRoleRlEntity busRoleRlResponse : busRoleRlResponses) {
                if (bussinessTypeEntity.getAction().equals(busRoleRlResponse.getBusAction())) {
                    bussinessTypeEntity.setCheck(true);
                }
            }
        }

        PageInfo<BussinessTypeEntity> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<BussinessTypeEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(bussinessTypeEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/bussinessType/filter")
    public BaseResponse filterByRole(BusRoleRlFilterRequest busRoleRlFilterRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        if (authAdmin.getCustomerId() == 110) {//系统管理员角色
            BussinessTypeFilterRequest bussinessTypeFilterRequest = new BussinessTypeFilterRequest();
            BeanUtils.copyProperties(busRoleRlFilterRequest, bussinessTypeFilterRequest);
            List<BussinessTypeEntity> bussinessTypeEntities;
            bussinessTypeEntities = bussinessTypeService.filterList(bussinessTypeFilterRequest);
            PageInfo<BussinessTypeEntity> pageInfo;
            pageInfo = new PageInfo<>(bussinessTypeEntities);
            PageSimpleResponse<BussinessTypeEntity> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(bussinessTypeEntities);
            return ResultVOUtils.success(pageSimpleResponse);

        } else {
            CustomerInfoEntity customerInfoEntity = customerInfoService.
                    selectById(authAdmin.getCustomerId());
            busRoleRlFilterRequest.setRoleId(customerInfoEntity.getRoleId() + "");
            List<BusRoleRlResponse> busRoleRlResponses;
            busRoleRlResponses = busRoleRlService.filterJoinList(busRoleRlFilterRequest);
            PageInfo<BusRoleRlResponse> pageInfo;
            pageInfo = new PageInfo<>(busRoleRlResponses);
            PageSimpleResponse<BusRoleRlResponse> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(busRoleRlResponses);
            return ResultVOUtils.success(pageSimpleResponse);
        }

    }


    @PostMapping("/udiwms/busRl/update")
    public BaseResponse postBusRl(@RequestBody BusRoleRlFilterRequest busRoleRlFilterRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        busRoleRlService.deleteById(busRoleRlFilterRequest.getRoleId() + "");
        if (busRoleRlFilterRequest.getActions() != null && busRoleRlFilterRequest.getActions().size() > 0) {
            for (String action : busRoleRlFilterRequest.getActions()) {
                BusRoleRlEntity busRoleRlEntity = new BusRoleRlEntity();
                busRoleRlEntity.setRoleId(busRoleRlFilterRequest.getRoleId());
                busRoleRlEntity.setBusAction(action);
                busRoleRlService.insertBusRoleRl(busRoleRlEntity);
            }

            return ResultVOUtils.success("关联成功");
        } else {
            return ResultVOUtils.success("未选择业务单据类型！");
        }


    }

}
