package com.glxp.udi.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.entity.inout.CodeNeedRlEntity;
import com.glxp.udi.admin.req.inout.CodeNeedRlFilterRequest;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.inout.CodeNeedRlService;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class CodeNeedRlController {

    @Resource
    CodeNeedRlService codeNeedRlService;

    @GetMapping("/udims/io/getNeedRlCodes")
    public BaseResponse getNeedRlCodes(CodeNeedRlFilterRequest codeNeedRlFilterRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<CodeNeedRlEntity> codeNeedRlEntities = codeNeedRlService.filterList(codeNeedRlFilterRequest);
        PageInfo<CodeNeedRlEntity> pageInfo;
        pageInfo = new PageInfo<>(codeNeedRlEntities);
        PageSimpleResponse<CodeNeedRlEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(codeNeedRlEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udims/io/delNeedRlCode")
    public BaseResponse delNeedRlCode(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        codeNeedRlService.deleteById(id);
        return ResultVOUtils.success("删除成功");
    }


}
