package com.glxp.udi.admin.controller.inout;


import com.glxp.udi.admin.constant.Constant;
import com.glxp.udi.admin.constant.TypeConstant;
import com.glxp.udi.admin.entity.info.SetupEntity;
import com.glxp.udi.admin.entity.inout.*;
import com.glxp.udi.admin.entity.udid.UdiEntity;
import com.glxp.udi.admin.mongo.service.OrderDetailMdService;
import com.glxp.udi.admin.mongo.service.WarehouseMdService;
import com.glxp.udi.admin.req.inout.FilterWarehouseRequest;
import com.glxp.udi.admin.service.info.SetupService;
import com.glxp.udi.admin.service.inout.UdiInfoService;
import com.glxp.udi.admin.service.inout.UdiRelevanceService;
import com.glxp.udi.admin.service.receipts.ProductInfoService;
import com.glxp.udi.admin.util.FilterUdiUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class InoutUtils {
    String udiUrl = "";


    @Resource
    UdiInfoService udiInfoService;

    public InoutUtils() {

    }

    public InoutUtils(String udiUrl) {
        this.udiUrl = udiUrl;
    }

    public String filterCode(WarehouseMdService warehouseService, WarehousetempEntity warehousetemp, SetupService setupService) {
        SetupEntity setupEntity = setupService.selectSetup();
        FilterWarehouseRequest filterWarehouseRequest = new FilterWarehouseRequest();
        filterWarehouseRequest.setCode(warehousetemp.getCode());
        filterWarehouseRequest.setCustomerId(warehousetemp.getCustomerId());
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByCode(filterWarehouseRequest);

        WarehouseEntity warehouseData = new WarehouseEntity();
        BeanUtils.copyProperties(warehousetemp, warehouseData);
        if (warehouseEntityList == null || warehouseEntityList.size() == 0) { //数据库为null,可以添加
            return nullVail(warehousetemp, setupEntity);
        }
        List<WarehouseEntity> inList = new ArrayList<>();
        List<WarehouseEntity> outList = new ArrayList<>();
        if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
            for (int i = 0; i < warehouseEntityList.size(); i++) {
                WarehouseEntity temp = warehouseEntityList.get(i);
                if (temp.getAction().equals(TypeConstant.TYPE_CODE_DESTORY)) {   //过滤已销毁的
                    return getErrMsg(temp);
                }
                if (temp.getAction().equals(TypeConstant.TYPE_STOCK_CHECK)) {   //过滤已盘点
                    if (warehouseData.getAction().equals(TypeConstant.TYPE_STOCK_CHECK)) {
                        return warehouseData.getCode() + "该码已盘点";
                    }
                }
                if (temp.getMainAction().equals(TypeConstant.TYPE_OUT)) {
                    outList.add(temp);
                } else if (temp.getMainAction().equals(TypeConstant.TYPE_PUT)) {
                    inList.add(temp);
                }
            }
            //如果序列号未null则不判断是否重码
            UdiEntity udiEntity = FilterUdiUtils.getUdi(warehouseData.getCode());
            if (udiEntity.getSerialNo() == null) {
                String serialMsg = serialVail(warehouseData, inList, outList);
                if (serialMsg != null) {
                    return serialMsg;
                }
                return null;
            }

            //如果数据库存在1个出库，则可以再录入1个入库;（同理：存在4个出库，3个入库，则可以再录入1个入库）
            if (outList.size() - inList.size() == 1) {
                if (warehouseData.getMainAction().equals(TypeConstant.TYPE_PUT)) {
                    return null;
                }
            } else if (inList.size() - outList.size() == 1) {
                if (warehouseData.getMainAction().equals(TypeConstant.TYPE_OUT)) {
                    WarehouseEntity in = inList.get(inList.size() - 1);
                    return null;
                }
            } else if (inList.size() - outList.size() == 0) {
                return null;
            } else {
                return warehousetemp.getCode() + "条码重复";
            }
            return warehousetemp.getCode() + "条码重复";
        }
        return null;
    }

    public String serialVail(WarehouseEntity warehouseData, List<WarehouseEntity> inList, List<WarehouseEntity> outList) {
        for (int i = 0; i < inList.size(); i++) {
            if (warehouseData.getCorpOrderId().equals(inList.get(i).getCorpOrderId())) {
                return warehouseData.getCode() + "条码重复";
            }
        }
        for (int i = 0; i < outList.size(); i++) {
            if (warehouseData.getCorpOrderId().equals(outList.get(i).getCorpOrderId())) {
                return warehouseData.getCode() + "条码重复";
            }
        }
        return null;
    }

    public String nullVail(WarehousetempEntity warehousetemp, SetupEntity setupEntity) {
        if (warehousetemp.getAction().equals(TypeConstant.TYPE_PUT_RETURN)) {
            if (setupEntity.isReturnWareHouseIn()) {
                return null;
            } else {
                return warehousetemp.getCode() + "不允许未销售出库可以采购入库";
            }
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_RETURN)) {
            if (setupEntity.isReturnWareHouseOut()) {
                return null;
            } else {
                return warehousetemp.getCode() + "不允许零库存退货出库";
            }
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_SALE)) {
            if (setupEntity.isReturnWareHouseOut()) {
                return null;
            } else {
                return warehousetemp.getCode() + "不允许零库存销售出库";
            }
        }
        return null;
    }

    public String getErrMsg(WarehouseEntity warehousetemp) {
        if (warehousetemp.getAction().equals(TypeConstant.TYPE_PUT_PRODUCT)) {
            return warehousetemp.getCode() + "已生产入库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_PUT_PURCHASE)) {
            return warehousetemp.getCode() + "已采购入库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_PUT_RETURN)) {
            return warehousetemp.getCode() + "已退货入库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_PUT_ALLOT)) {
            return warehousetemp.getCode() + "已调拨入库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_SALE)) {
            return warehousetemp.getCode() + "已销售出库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_RETURN)) {
            return warehousetemp.getCode() + "已退货出库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_ALLOT)) {
            return warehousetemp.getCode() + "已调拨出库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_DESTORY)) {
            return warehousetemp.getCode() + "已销毁出库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_STRAIGHT)) {
            return warehousetemp.getCode() + "已直调出库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_REWORK)) {
            return warehousetemp.getCode() + "已返工出库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_OUT_CHECK)) {
            return warehousetemp.getCode() + "已抽检出库";
        } else if (warehousetemp.getAction().equals(TypeConstant.TYPE_CODE_DESTORY)) {
            return warehousetemp.getCode() + "已销毁";
        }
        return warehousetemp.getCode() + "出错";
    }


    public void generateOrderDetail(ProductInfoService productInfoService, List<WarehouseEntity> warehouseEntityList) {
        //把所有药监码分类
        Map<String, List<WarehouseEntity>> sortMap = new HashMap<>();
        for (WarehouseEntity warehouseEntity : warehouseEntityList) {
            String code = warehouseEntity.getCode().substring(0, 7);
            List<WarehouseEntity> warehouseEntities = sortMap.get(code);
            if (warehouseEntities == null) {
                warehouseEntities = new ArrayList<>();
            }
            warehouseEntities.add(warehouseEntity);
            sortMap.put(code, warehouseEntities);
        }

        //获取所有产品编码
        List<String> nameCodes = new ArrayList<>();
        for (String key : sortMap.keySet()) {
            nameCodes.add(key);
        }

        //获取药品信息并计算数量
        List<OrderDetailEntity> orderDetailEntities = productInfoService.findByCodes(nameCodes);
        for (int i = 0; i < orderDetailEntities.size(); i++) {
            List<WarehouseEntity> warehouseEntities = sortMap.get(orderDetailEntities.get(i).getNameCode());
            if (warehouseEntities != null) {
                int odCount = orderDetailEntities.get(i).getBhzxxsbzsl();
                if (odCount > 0) {
                    orderDetailEntities.get(i).setCount(warehouseEntities.size() * odCount);
                } else {
                    orderDetailEntities.get(i).setCount(warehouseEntities.size());
                }

            }
        }
        List<OrderDetailEntity> returnOrderDetail = new ArrayList<>();
        Map<String, OrderDetailEntity> detailEntityMap = new HashMap<>();
        for (OrderDetailEntity orderDetailEntity : orderDetailEntities) {
            String key = orderDetailEntity.getCoName() + orderDetailEntity.getFormSpec() + orderDetailEntity.getPackSpec() + orderDetailEntity.getPackRatio();
            OrderDetailEntity temp = detailEntityMap.get(key);
            if (temp != null) {
                orderDetailEntity.setCount(temp.getCount() + orderDetailEntity.getCount());
            }
            detailEntityMap.put(key, orderDetailEntity);
        }
        for (Map.Entry<String, OrderDetailEntity> entry : detailEntityMap.entrySet()) {
            System.out.println("key = " + entry.getKey() + ", value = " + entry.getValue());
            returnOrderDetail.add(entry.getValue());
        }
    }


    public ErpDetailCodeEntity getDetailCode(ProductInfoService productInfoService, String code) {
        ErpDetailCodeEntity detailCodeEntity = productInfoService.getDetailCode(code);
        if (detailCodeEntity == null) {
            detailCodeEntity = productInfoService.getDetailCodeNoBh(code);
        }
        return detailCodeEntity;
    }

    public UdiInfoEntity getUdiInfo(UdiInfoService udiInfoService, UdiRelevanceService udiRelevanceService, String code) {
        String prefix;
        prefix = FilterUdiUtils.getDiStr(code);
        UdiInfoEntity udiInfoEntity = udiInfoService.findByNameCode(prefix);
        if (udiInfoEntity == null) {
//            udiInfoEntity = new UdiInfoEntity();
            List<UdiInfoEntity> froms = new UdiDlHttpClient(udiUrl).getUdiByDi(prefix);
            if (froms != null && !froms.isEmpty()) {
//                udiInfoService.insertUdiInfos(froms);
                udiInfoEntity = froms.get(0);
                List<UdiInfoEntity> udiInfoEntities = new UdiDlHttpClient(udiUrl).getUdiByUuid(udiInfoEntity.getUuid());
                if (udiInfoEntities != null) {
                    for (UdiInfoEntity temp : udiInfoEntities) {
                        udiInfoEntity.setCustomerId(temp.getCustomerId() + "");
                    }
                    udiInfoService.insertUdiInfos(udiInfoEntities);
                }
//                udiInfoService.findByNameCode(prefix);
//                UdiRelevanceEntity udiRelevanceEntity = new UdiRelevanceEntity();
//                udiRelevanceEntity.setUuid(udiInfoEntity.getUuid());
//                udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
//                udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
            } else {
                udiInfoEntity = new UdiInfoEntity();
            }
        }
        udiInfoEntity.setCode(code);
        udiInfoEntity = FilterUdiUtils.transUdi(udiInfoEntity);
        return udiInfoEntity;
    }


    public String vailIsUseDy(WarehousetempEntity warehouseEntity) {
        UdiInfoEntity udiInfoEntity = getUdiInfo(udiInfoService, null, warehouseEntity.getCode());
        if (udiInfoEntity.getDiType() == Constant.DITYPE_SYDY) {
            if (udiInfoEntity.getIsUseDy() != Constant.USEDY_TRUE) {
                return "该产品未勾选以使用单元入库";
            }
        }
        return null;
    }


    public void generateConstrastDetail(UdiInfoService udiInfoService, UdiRelevanceService udiRelevanceService,
                                        OrderDetailMdService orderDetailMdService,
                                        List<WarehouseEntity> warehouseEntityList) {
        if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
            List<UdiInfoEntity> udiInfoEntities = new ArrayList<>();
            for (WarehouseEntity warehouseEntity : warehouseEntityList) {
                UdiInfoEntity udiInfoEntity = getUdiInfo(udiInfoService, udiRelevanceService, warehouseEntity.getCode());
                if (udiInfoEntity != null) {
                    if (warehouseEntity.getCount() != null)
                        udiInfoEntity.setCount(warehouseEntity.getCount());
                    else {
                        udiInfoEntity.setCount(getActCount(udiInfoEntity));
                    }
                    udiInfoEntities.add(udiInfoEntity);
                }
            }

            List<UdiInfoEntity> returnOrderDetail = new ArrayList<>();
            Map<String, UdiInfoEntity> sortMaps = new HashMap<>();
            for (UdiInfoEntity detailCodeEntity : udiInfoEntities) {
                String key = detailCodeEntity.getBatchNo() + detailCodeEntity.getCpmctymc();
                UdiInfoEntity temp = sortMaps.get(key);
                if (temp != null) {
                    detailCodeEntity.setCount(temp.getCount() + detailCodeEntity.getCount());
                }


                sortMaps.put(key, detailCodeEntity);
            }

            for (Map.Entry<String, UdiInfoEntity> entry : sortMaps.entrySet()) {
                System.out.println("key = " + entry.getKey() + ", value = " + entry.getValue());
                returnOrderDetail.add(entry.getValue());
            }

            List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
            for (UdiInfoEntity erpDetailCodeEntity : returnOrderDetail) {
                ErpOrderEntity erpOrderEntity = new ErpOrderEntity();
                erpOrderEntity.setNameCode(erpDetailCodeEntity.getNameCode());
                erpOrderEntity.setPackRatio(erpDetailCodeEntity.getPackRatio());
                erpOrderEntity.setCoName(erpDetailCodeEntity.getCpmctymc());
                erpOrderEntity.setLotno(erpDetailCodeEntity.getBatchNo());
                erpOrderEntity.setProduceDate(erpDetailCodeEntity.getProduceDate());
                erpOrderEntity.setExpireDate(erpDetailCodeEntity.getExpireDate());
                erpOrderEntity.setCount(erpDetailCodeEntity.getCount());
                erpOrderEntity.setOrderIdFk(warehouseEntityList.get(0).getOrderId());
                erpOrderEntity.setProductCompany(erpDetailCodeEntity.getYlqxzcrbarmc());
                erpOrderEntity.setAuthCode(erpDetailCodeEntity.getZczbhhzbapzbh());
                erpOrderEntity.setUuidFk(erpDetailCodeEntity.getUuid());
                erpOrderEntity.setPackSpec(erpDetailCodeEntity.getGgxh());
                erpOrderEntities.add(erpOrderEntity);
            }
            if (!erpOrderEntities.isEmpty()) {
                orderDetailMdService.insertOrderDetail(erpOrderEntities);
            }
        }
    }

    public int getActCount(UdiInfoEntity udiInfoBean) {
        if (udiInfoBean != null) {
            if (udiInfoBean.getZxxsbzbhsydysl() > 1) {
                return udiInfoBean.getZxxsbzbhsydysl() * udiInfoBean.getBhzxxsbzsl();
            } else {
                return udiInfoBean.getBhzxxsbzsl();
            }
        }

        return 1;
    }
}
