package com.glxp.udi.admin.controller.inout;

import com.glxp.udi.admin.entity.inout.InvProductDetailEntity;
import com.glxp.udi.admin.entity.inout.InvProductEntity;
import com.glxp.udi.admin.entity.inout.UdiInfoEntity;
import com.glxp.udi.admin.mongo.service.InvProductDetailMdService;
import com.glxp.udi.admin.mongo.service.InvProductMdService;
import com.glxp.udi.admin.mongo.utils.PageResult;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.FilterInvProductDetailRequest;
import com.glxp.udi.admin.req.inout.FilterInvProductRequest;
import com.glxp.udi.admin.req.inout.FilterUdiInfoRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.inout.UdiInfoService;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RestController
public class InvProductsController {

    @Resource
    InvProductMdService invProductService;
    @Resource
    InvProductDetailMdService invProductDetailService;
    @Resource
    UdiInfoService udiInfoService;

    @GetMapping("udims/inv/products/filter")
    public BaseResponse filterInvProduct(FilterInvProductRequest filterInvProductRequest) {

        if ((filterInvProductRequest.getProductsName() != null && !filterInvProductRequest.getProductsName().isEmpty())
                || (filterInvProductRequest.getNameCode() != null && !filterInvProductRequest.getNameCode().isEmpty())) {
            FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
            filterUdiInfoRequest.setNameCode(filterInvProductRequest.getNameCode());
            filterUdiInfoRequest.setCpmctymc(filterInvProductRequest.getProductsName());
            List<UdiInfoEntity> udiInfoEntities = udiInfoService.filterAllUdiInfo(filterUdiInfoRequest);
            List<String> uuids = new ArrayList<>();
            for (UdiInfoEntity udiInfoEntity : udiInfoEntities) {
                uuids.add(udiInfoEntity.getUuid());
            }
            filterInvProductRequest.setUuids(uuids);
            if (uuids.size() <= 0)
                return ResultVOUtils.success(new PageSimpleResponse<>());
        }
        PageResult<InvProductEntity> invProductEntityPageResult = invProductService.filterInvProduct(filterInvProductRequest);

        List<InvProductEntity> invProductEntities = invProductEntityPageResult.getList();
        for (InvProductEntity invProductEntity : invProductEntities) {
            UdiInfoEntity udiInfoEntity = udiInfoService.findByUuid(invProductEntity.getRelIdFk());
            invProductEntity.setNameCode(udiInfoEntity.getNameCode());
            invProductEntity.setProductsName(udiInfoEntity.getCpmctymc());
            invProductEntity.setGgxh(udiInfoEntity.getGgxh());
        }

        PageSimpleResponse<InvProductEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(invProductEntityPageResult.getTotal());
        pageSimpleResponse.setList(invProductEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("udims/inv/products/filterDetail")
    public BaseResponse filterInvProductDetail(FilterInvProductDetailRequest filterInvProductRequest) {
        PageResult<InvProductDetailEntity> pageResult = invProductDetailService.filterInvProduct(filterInvProductRequest);
        PageSimpleResponse<InvProductDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageResult.getTotal());
        pageSimpleResponse.setList(pageResult.getList());
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("udims/inv/products/delete")
    public BaseResponse deleteInvProduct(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        InvProductEntity invProductEntity = invProductService.selectById(id);
        if (invProductEntity != null) {
            invProductService.deleteById(id);
            invProductDetailService.deleteByProductId(invProductEntity.getId() + "");
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败！");
        }


    }
}
