package com.glxp.udi.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.entity.inout.ErpOrderEntity;
import com.glxp.udi.admin.mongo.service.OrderDetailMdService;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.FilterErpOrderRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.inout.MyErpOrderService;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class MyErpOrderController {
    @Value("${UCLOD_SERVER_URL}")
    private String ucloudUrl;
    @Resource
    MyErpOrderService myErpOrderService;
    @Resource
    private OrderDetailMdService orderDetailMdService;

    @GetMapping(value = "/warehouse/inout/order/detail")
    public BaseResponse orderDetail(FilterErpOrderRequest filterErpOrderRequest,
                                    BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getOrderId() == null || filterErpOrderRequest.getOrderId().equals("")) {
            filterErpOrderRequest.setOrderId("1");
        }
        filterErpOrderRequest.setPage(1);
        filterErpOrderRequest.setLimit(100);
        List<ErpOrderEntity> erpOrderEntities = orderDetailMdService.filterOrderDetails(filterErpOrderRequest);
        return ResultVOUtils.success(erpOrderEntities);
    }

    /**
     * ------------------------------------------------(
     **/


    @GetMapping("/warehouse/getMyErpOrders")
    public BaseResponse getMyErpOrders(FilterErpOrderRequest filterErpOrderRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getDocid() == null || filterErpOrderRequest.getDocid().equals("")) {
            filterErpOrderRequest.setDocid("null");
        }
        List<ErpOrderEntity> erpInfoEntityList = myErpOrderService.filterMyErpDetailOrder(filterErpOrderRequest);
        PageInfo<ErpOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(erpInfoEntityList);
        PageSimpleResponse<ErpOrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/warehouse/deleteErpOrder")
    public BaseResponse deleteErpOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = myErpOrderService.deleteById(deleteRequest.getId());
        if (b) {
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }


}
