package com.glxp.udi.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.constant.Constant;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.inout.ErpOrderEntity;
import com.glxp.udi.admin.entity.inout.OrderEntity;
import com.glxp.udi.admin.entity.inout.OrderMdEntity;
import com.glxp.udi.admin.entity.inout.WarehouseEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.mongo.service.OrderDetailMdService;
import com.glxp.udi.admin.mongo.service.OrderMdService;
import com.glxp.udi.admin.mongo.service.WarehouseMdService;
import com.glxp.udi.admin.mongo.utils.PageResult;
import com.glxp.udi.admin.req.inout.*;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.info.SetupService;
import com.glxp.udi.admin.service.inout.*;
import com.glxp.udi.admin.service.receipts.MipsBarcodeService;
import com.glxp.udi.admin.util.GetAuthAdminUtil;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
public class OrderController {

    @Resource
    private OrderService orderService;
    @Resource
    private WarehouseService warehouseService;
    @Resource
    private MyErpOrderService myErpOrderService;
    @Resource
    private MipsBarcodeService mipsBarcodeService;
    @Resource
    private UdiInfoService udiInfoService;
    @Resource
    private UdiRelevanceService udiRelevanceService;
    @Resource
    private SetupService setupService;
    @Resource
    AuthAdminService authAdminService;
    @Resource
    private OrderDetailMdService orderDetailMdService;
    @Resource
    private OrderMdService orderMdService;
    @Resource
    private WarehouseMdService warehouseMdService;

    @AuthRuleAnnotation("")
    @GetMapping("/warehouse/inout/order/filter")
    public BaseResponse filterOrder(OrderFilterRequest orderFilterRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        orderFilterRequest.setCustomerId(authAdmin.getCustomerId());
//        orderFilterRequest.setStatus(Constant.ORDER_STATUS_SUCCESS);
//        if (orderFilterRequest.getCode() != null && !orderFilterRequest.getCode().equals("")) {
//            FilterWarehouseRequest filterWarehouseRequest = new FilterWarehouseRequest();
//            filterWarehouseRequest.setCustomerId(authAdmin.getCustomerId());
//            filterWarehouseRequest.setCode(orderFilterRequest.getCode());
//            List<WarehouseEntity> warehouseEntityList = warehouseService.findByCode(filterWarehouseRequest);
//            orderEntityList = orderService.filterListByCode(warehouseEntityList);
//        } else {
//
//        }

        PageResult<OrderMdEntity> pageResult =
                orderMdService.filterList(orderFilterRequest);
        PageSimpleResponse<OrderMdEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageResult.getTotal());
        pageSimpleResponse.setList(pageResult.getList());
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("")
    @GetMapping("/warehouse/inout/order/filterCode")
    public BaseResponse filterOrderByCode(OrderFilterByCodeRequest orderFilterByCodeRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        FilterWarehouseRequest filterWarehouseRequest = new FilterWarehouseRequest();
        filterWarehouseRequest.setCustomerId(authAdmin.getCustomerId());
        filterWarehouseRequest.setCode(orderFilterByCodeRequest.getCode());
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByCode(filterWarehouseRequest);
        List<OrderEntity> orderEntityList = orderService.filterListByCode(warehouseEntityList);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("warehouse/inout/order/deleteOrder")
    public BaseResponse deleteOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        orderMdService.deleteOrder(deleteRequest);
        orderDetailMdService.deleteOrderDetail(deleteRequest);
        warehouseMdService.deleteCode(deleteRequest);
        return ResultVOUtils.success("删除成功");
    }

    @AuthRuleAnnotation("")
    @GetMapping("admin/warehouse/inout/order/listerror")
    public BaseResponse listOrderError(OrderFilterRequest orderFilterRequest) {

        //        orderFilterRequest.setIsError("success");

        orderFilterRequest.setStatus(Constant.ORDER_STATUS_ERROR);
        PageResult<OrderMdEntity> pageResult =
                orderMdService.filterList(orderFilterRequest);
        PageSimpleResponse<OrderMdEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageResult.getTotal());
        pageSimpleResponse.setList(pageResult.getList());
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("")
    @PostMapping("/warehouse/inout/order/mergeOrder")
    public BaseResponse mergeOrder(@RequestBody OrderMergeRequest orderMergeRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<String> orderList = orderMergeRequest.getOrderList();

        if (orderList != null && orderList.size() > 1) {
            for (String orderId : orderList) {
                DeleteRequest deleteRequest = new DeleteRequest();
                deleteRequest.setOrderId(orderId);
                orderDetailMdService.deleteOrderDetail(deleteRequest);
            }
            String mainOrder = orderList.get(0);
            for (int i = 1; i < orderList.size(); i++) {
                warehouseMdService.updateOrderId(orderList.get(i), mainOrder);
                DeleteRequest deleteRequest = new DeleteRequest();
                deleteRequest.setOrderId(orderList.get(i));
                orderMdService.deleteOrder(deleteRequest);
            }
            new Thread(() -> {
                WarehouseQueryRequest warehouseQueryRequest = new WarehouseQueryRequest();
                warehouseQueryRequest.setOrderId(mainOrder);
                List<WarehouseEntity> transData = warehouseMdService.filterWarehouse(warehouseQueryRequest);
                new InoutUtils().generateConstrastDetail(udiInfoService, udiRelevanceService, orderDetailMdService, transData);
            }).start();
        }


        return ResultVOUtils.success("合并成功,后台正在数据转换，请稍后刷新重试！");
    }

    @PostMapping("/warehouse/inout/order/unit")
    public BaseResponse updateUnit(@RequestBody @Valid OrderMdEntity orderEntity,
                                   BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        orderMdService.updateOrder(orderEntity);
        warehouseMdService.updateUnit(orderEntity.getOrderId(), orderEntity.getFromCorpId(), orderEntity.getFromCorp());

        return ResultVOUtils.success("添加成功");
    }


    /**
     * -----------------------暂不确定----------------------
     */


    @AuthRuleAnnotation("")
    @GetMapping("admin/warehouse/inout/order/list")
    public BaseResponse list(OrderQueryRequest orderQueryRequest) {
        List<OrderEntity> orderEntityList = orderService.listOrders(orderQueryRequest);
        PageInfo<OrderEntity> pageInfo;
        String data;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @AuthRuleAnnotation("")
    @GetMapping("admin/warehouse/inout/order/listRepeat")
    public BaseResponse listOrderRepeat(OrderQueryRequest orderQueryRequest) {
        List<OrderEntity> orderEntityList = orderService.listOrderRepeat(orderQueryRequest);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("")
    @PostMapping("admin/warehouse/inout/order/save")
    public BaseResponse save(@RequestBody @Valid OrderEntity orderSaveRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        OrderEntity orderEntity = new OrderEntity();
        BeanUtils.copyProperties(orderSaveRequest, orderEntity);

        boolean b = orderService.insertOrder(orderEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("添加成功");
    }


    public List<String> getDocids(List<ErpOrderEntity> mergeList) {
        List<String> docids = new ArrayList<>();
        Map<String, List<ErpOrderEntity>> mapErps = mergeList.stream().collect(Collectors.groupingBy(ErpOrderEntity::getDocid));
        for (String key : mapErps.keySet()) {
            docids.add(key);
        }
        return docids;
    }


    public boolean isSameUnit() {
        return false;
    }


    @PostMapping("/warehouse/inout/order/updateExportStatus")
    public BaseResponse updateExportStatus(@RequestBody UpdateExportStatusRequest updateExportStatusRequest,
                                           BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = orderService.updateExportStatus(updateExportStatusRequest.getOrderId(), updateExportStatusRequest.getExportStatus());
        List<String> codes = orderService.findCodesById(updateExportStatusRequest.getOrderId());
        mipsBarcodeService.updateDtFlag(codes);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("更新成功");
    }

}
