package com.glxp.udi.admin.controller.inout;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.inout.UdiCompanyEntity;
import com.glxp.udi.admin.entity.inout.UdiInfoEntity;
import com.glxp.udi.admin.entity.udid.UdiEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.inout.FilterUdiInfoRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.basic.ProductInfoResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.inout.UdiInfoService;
import com.glxp.udi.admin.util.FilterUdiUtils;
import com.glxp.udi.admin.util.GetAuthAdminUtil;
import com.glxp.udi.admin.util.HttpClient;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
public class UdiInfoController {

    @Resource
    UdiInfoService udiInfoService;
    @Resource
    AuthAdminService authAdminService;
    @Value("${UDI_SERVER_URL}")
    private String udiUrl;

    @GetMapping("udi/udiinfo/filterUdi")
    public BaseResponse filterUdi(FilterUdiInfoRequest productInfoFilterRequest) {


        String nameCode = null;
        if (productInfoFilterRequest.getNameCode() != null) {
            UdiEntity udiEntity = FilterUdiUtils.getUdi(productInfoFilterRequest.getNameCode());
            if (udiEntity != null) {
                nameCode = udiEntity.getUdi();
            } else {
                nameCode = productInfoFilterRequest.getNameCode();
            }
        }
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("page", productInfoFilterRequest.getPage());
        paramMap.put("limit", productInfoFilterRequest.getLimit());
        paramMap.put("ylqxzcrbarmc", productInfoFilterRequest.getYlqxzcrbarmc());
        paramMap.put("cpmctymc", productInfoFilterRequest.getCpmctymc());
        paramMap.put("nameCode", nameCode);
        paramMap.put("ggxh", productInfoFilterRequest.getGgxh());
        paramMap.put("uuid", productInfoFilterRequest.getUuid());

        String response = HttpClient.mipsGet(udiUrl + "/udidl/device/filterUdi", paramMap);
        try {

            String customerId = getCustomerId();
            BaseResponse<PageSimpleResponse<UdiInfoEntity>> udiDlDeviceResponse =
                    JSONObject.parseObject(response, new TypeReference<BaseResponse<PageSimpleResponse<UdiInfoEntity>>>() {
                    });

            if (productInfoFilterRequest.getIsCheck() != null && productInfoFilterRequest.getIsCheck()) {
                List<UdiInfoEntity> udiInfoEntities = udiDlDeviceResponse.getData().getList();

                if (udiInfoEntities != null && udiInfoEntities.size() > 0) {
                    for (UdiInfoEntity udiInfoEntity : udiInfoEntities) {
                        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                        filterUdiInfoRequest.setCustomerId(customerId);
                        filterUdiInfoRequest.setUuid(udiInfoEntity.getUuid());
                        List<UdiInfoEntity> udiRelevanceEntities = udiInfoService.filterUdiInfo(filterUdiInfoRequest);
                        if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 0) {
                            udiInfoEntity.setCheck(true);
                        } else {
                            udiInfoEntity.setCheck(false);
                        }
                    }
                }
            }
            return udiDlDeviceResponse;
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }


    //获取同步库医疗器械注册人信息
    @GetMapping("udiwms/udiinfo/filterCompany")
    public BaseResponse filterCompany(FilterUdiInfoRequest productInfoFilterRequest) {

        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("page", productInfoFilterRequest.getPage());
        paramMap.put("limit", productInfoFilterRequest.getLimit());
        paramMap.put("ylqxzcrbarmc", productInfoFilterRequest.getYlqxzcrbarmc());

        String response = HttpClient.mipsGet(udiUrl + "/udi/company/filter", paramMap);
        try {
            BaseResponse<PageSimpleResponse<UdiCompanyEntity>> udiDlDeviceResponse = (BaseResponse<PageSimpleResponse<UdiCompanyEntity>>) JSONObject.parseObject(response, BaseResponse.class);
            return udiDlDeviceResponse;
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }

    @GetMapping("udi/udiinfo/filterByUuid")
    public BaseResponse filterByUuid(FilterUdiInfoRequest productInfoFilterRequest) {

        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("page", productInfoFilterRequest.getPage());
        paramMap.put("limit", productInfoFilterRequest.getLimit());
        String response = HttpClient.mipsGet(udiUrl + "/udidl/device/filterUdi", paramMap);
        try {
            BaseResponse<List<UdiInfoEntity>> udiDlDeviceResponse = (BaseResponse<List<UdiInfoEntity>>) JSONObject.parseObject(response, BaseResponse.class);
            return udiDlDeviceResponse;
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }

    @AuthRuleAnnotation("")
    @GetMapping("udi/udiinfo/local/filterByUuid")
    public BaseResponse filterLocalByUuid(FilterUdiInfoRequest productInfoFilterRequest) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        productInfoFilterRequest.setCustomerId(authAdmin.getCustomerId() + "");
        List<UdiInfoEntity> udiInfoEntities = udiInfoService.filterAllUdiInfo(productInfoFilterRequest);
        return ResultVOUtils.success(udiInfoEntities);
    }

    @AuthRuleAnnotation("")
    @PostMapping("/udi/udirel/update")
    public BaseResponse updateRelevanceEntity(@RequestBody UdiInfoEntity udiInfoEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        udiInfoService.updateUdiInfoByUuid(udiInfoEntity);
        return ResultVOUtils.success("更新成功");
    }


    @GetMapping("/udidl/wms/findBydi")
    public BaseResponse findByDi(FilterUdiInfoRequest filterUdiInfoRequest) {
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("nameCode", filterUdiInfoRequest.getNameCode());
        String response = HttpClient.mipsGet(udiUrl + "/udidl/wms/findBydi", paramMap);
        try {
            BaseResponse<ProductInfoResponse> udiDlDeviceResponse = (BaseResponse<ProductInfoResponse>) JSONObject.parseObject(response, BaseResponse.class);
            return udiDlDeviceResponse;
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }


    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
