package com.glxp.udi.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.inout.UdiInfoEntity;
import com.glxp.udi.admin.entity.inout.WarehouseEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.FilterUdiInfoRequest;
import com.glxp.udi.admin.req.receipt.CombineRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.inout.UdiRelevanceResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.inout.UdiInfoService;
import com.glxp.udi.admin.service.inout.WarehouseService;
import com.glxp.udi.admin.thread.BasicUpdateProductService;
import com.glxp.udi.admin.util.GetAuthAdminUtil;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class UdiRelevanceController {

    @Value("${UDI_SERVER_URL}")
    private String udiUrl;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    WarehouseService warehouseService;
    @Resource
    AuthAdminService authAdminService;
    @Resource
    BasicUpdateProductService basicUpdateProductService;

    //客户端器械信息下载
    @GetMapping("/udi/udirel/download")
    public BaseResponse downloadProducts(FilterUdiInfoRequest filterUdiInfoRequest,
                                         BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        filterUdiInfoRequest.setCustomerId(authAdmin.getCustomerId() + "");

        List<UdiInfoEntity> udiInfoEntities = udiInfoService.filterUdiInfo(filterUdiInfoRequest);
        List<UdiRelevanceResponse> udiRelevanceResponses = udiInfoEntities.stream().map(e -> {
            UdiRelevanceResponse authAdminRoleResponse = new UdiRelevanceResponse();
            BeanUtils.copyProperties(e, authAdminRoleResponse);
            return authAdminRoleResponse;
        }).collect(Collectors.toList());

        PageInfo<UdiRelevanceResponse> pageInfo;
        pageInfo = new PageInfo<>(udiRelevanceResponses);
        PageSimpleResponse<UdiRelevanceResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiRelevanceResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udi/udirel/filterUdi")
    public BaseResponse filterUdiGp(FilterUdiInfoRequest filterUdiInfoRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        filterUdiInfoRequest.setCustomerId(authAdmin.getCustomerId() + "");

        List<UdiInfoEntity> udiRelevanceResponses = null;
        filterUdiInfoRequest.setDiType(1);
        udiRelevanceResponses = udiInfoService.filterUdiInfoGp(filterUdiInfoRequest);
//        if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
//            for (UdiInfoEntity udiRelevanceResponse : udiRelevanceResponses) {
//                if (udiRelevanceResponse.getDiType() == 2 && udiRelevanceResponse.getIsUseDy() != 1) {
//                    udiRelevanceResponse.setNameCode(udiRelevanceResponse.getSjcpbm());
//                }
//            }
//        }
        PageInfo<UdiInfoEntity> pageInfo;
        pageInfo = new PageInfo<>(udiRelevanceResponses);
        PageSimpleResponse<UdiInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiRelevanceResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udi/udirel/deleteById")
    public BaseResponse deleteById(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String ids = deleteRequest.getId();
        List<UdiInfoEntity> udiInfoEntities = udiInfoService.findByUuids(ids);
        if (udiInfoEntities != null && udiInfoEntities.size() > 0) {
            for (int i = 0; i < udiInfoEntities.size(); i++) {
                List<WarehouseEntity> warehouseEntityList = warehouseService.searchCode(udiInfoEntities.get(i).getNameCode(), authAdmin.getCustomerId());
                if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
                    return ResultVOUtils.error(500, "改产品已被" + warehouseEntityList.get(0).getOrderId() + "订单关联不能删除");
                }
            }
            udiInfoService.deleteByUuid(ids);
        }
        return ResultVOUtils.success("删除成功");
    }

    @AuthRuleAnnotation("")
    @PostMapping("/udi/udirel/revleance")
    public BaseResponse revleanceUdi(@RequestBody CombineRequest combineRequest) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);

        List<String> keys = combineRequest.getKeys();
        if (keys != null && keys.size() > 0) {
            for (String key : keys) {
                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setUuid(key);
                filterUdiInfoRequest.setCustomerId(authAdmin.getCustomerId() + "");
                List<UdiInfoEntity> udiInfoEntities = udiInfoService.filterAllUdiInfo(filterUdiInfoRequest);
                if (udiInfoEntities == null || udiInfoEntities.isEmpty()) {
                    udiInfoEntities = new UdiDlHttpClient(udiUrl).getUdiByUuid(key);
                    if (udiInfoEntities != null) {
                        for (UdiInfoEntity udiInfoEntity : udiInfoEntities) {
                            udiInfoEntity.setCustomerId(authAdmin.getCustomerId() + "");
                        }
                        udiInfoService.insertUdiInfos(udiInfoEntities);
                    } else {
                        return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
                    }
                }
            }
        }
        return ResultVOUtils.success("合并成功");
    }

    //选入查询结果的所有DI
    @AuthRuleAnnotation("")
    @GetMapping("/udi/udirel/combineAll")
    public BaseResponse combineAll(FilterUdiInfoRequest productInfoFilterRequest) {
        productInfoFilterRequest.setCustomerId(getCustomerId());
        basicUpdateProductService.relAllUdi(productInfoFilterRequest);
        return ResultVOUtils.success("后台已开始导入，请稍后刷新查看");
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
