package com.glxp.udi.admin.controller.inout;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.inout.OrderMdEntity;
import com.glxp.udi.admin.entity.inout.TraceUploadEntity;
import com.glxp.udi.admin.entity.inout.UdiInfoEntity;
import com.glxp.udi.admin.entity.inout.WarehouseEntity;
import com.glxp.udi.admin.entity.udid.UdiEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.mongo.service.OrderMdService;
import com.glxp.udi.admin.mongo.service.WarehouseMdService;
import com.glxp.udi.admin.req.inout.FilterUdiInfoRequest;
import com.glxp.udi.admin.req.inout.OrderFilterRequest;
import com.glxp.udi.admin.req.inout.WarehouseQueryRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.inout.UdiRelevanceResponse;
import com.glxp.udi.admin.res.inout.UdiTraceResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.inout.UdiInfoService;
import com.glxp.udi.admin.service.inout.UdiRelevanceService;
import com.glxp.udi.admin.service.inout.WarehouseService;
import com.glxp.udi.admin.util.DateUtil;
import com.glxp.udi.admin.util.FilterUdiUtils;
import com.glxp.udi.admin.util.GetAuthAdminUtil;
import com.glxp.udi.admin.util.HttpClient;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@RestController
public class UdiTraceController {

    @Resource
    private WarehouseService warehouseService;
    @Resource
    private WarehouseMdService warehouseMdService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    UdiInfoService udiInfoService;
    @Value("${COOR_UPLOAD_ORDER}")
    private String coorUploadUrl;
    private static final Logger logger = LoggerFactory.getLogger(UdiTraceController.class);
    @Resource
    AuthAdminService authAdminService;
    @Resource
    private OrderMdService orderMdService;

    @GetMapping("/udiwarehouse/trace/filter")
    public BaseResponse list(WarehouseQueryRequest warehouseQueryRequest) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        if (authAdmin.getCustomerId() != 110)
            warehouseQueryRequest.setCustomerId(authAdmin.getCustomerId());
//        List<WarehouseEntity> warehouseEntityList = warehouseService.findByOrderId(warehouseQueryRequest);


        List<WarehouseEntity> warehouseEntityList = warehouseMdService.filterWarehouse(warehouseQueryRequest);
        if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
            WarehouseEntity warehouseEntity = warehouseEntityList.get(0);
            UdiEntity udiEntity = FilterUdiUtils.getUdi(warehouseEntity.getCode());
            String udi = udiEntity.getUdi();
            FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
            filterUdiInfoRequest.setNameCode(udi);
            UdiInfoEntity udiInfoEntity = udiInfoService.findByNameCode(udi);
//            List<UdiTraceResponse> udiTraceResponseList = warehouseEntityList.stream().map(item ->
//            {
//                UdiTraceResponse udiTraceResponse = new UdiTraceResponse();
//                BeanUtils.copyProperties(item, udiTraceResponse);
//                if (udiInfoEntity != null) {
//                    udiTraceResponse.setYlqxzcrbarmc(udiInfoEntity.getYlqxzcrbarmc());
//                    udiTraceResponse.setCpmctymc(udiInfoEntity.getCpmctymc());
//                    udiTraceResponse.setGgxh(udiInfoEntity.getGgxh());
//                }
//                return udiTraceResponse;
//            }).collect(Collectors.toList());

            //查找父级或者子级
            List<UdiInfoEntity> udiInfoEntities = udiInfoService.findByUuids(udiInfoEntity.getUuid());
            List<UdiTraceResponse> udiTraceResponseList = new ArrayList<>();
            if (udiInfoEntities != null && udiInfoEntities.size() > 0) {

                for (UdiInfoEntity temp : udiInfoEntities) {
                    WarehouseQueryRequest queryRequest = new WarehouseQueryRequest();
                    queryRequest.setBatchNo(temp.getBatchNo());
                    queryRequest.setUdi(temp.getNameCode());
                    if (authAdmin.getCustomerId() != 110)
                        queryRequest.setCustomerId(authAdmin.getCustomerId());
                    List<WarehouseEntity> warehouseEntities = warehouseMdService.filterWarehouse(queryRequest);

                    List<UdiTraceResponse> udiTraceResponses = warehouseEntities.stream().map(item ->
                    {
                        UdiTraceResponse udiTraceResponse = new UdiTraceResponse();
                        BeanUtils.copyProperties(item, udiTraceResponse);
                        if (udiInfoEntity != null) {
                            udiTraceResponse.setYlqxzcrbarmc(udiInfoEntity.getYlqxzcrbarmc());
                            udiTraceResponse.setCpmctymc(udiInfoEntity.getCpmctymc());
                            udiTraceResponse.setGgxh(udiInfoEntity.getGgxh());
                        }
                        return udiTraceResponse;
                    }).collect(Collectors.toList());

                    Map<String, UdiTraceResponse> filterMap = new HashMap<>();
                    if (udiTraceResponses != null && udiTraceResponses.size() > 0) {
                        for (UdiTraceResponse udiTraceResponse : udiTraceResponses) {
                            filterMap.put(udiTraceResponse.getOrderId(), udiTraceResponse);
                        }
                        for (UdiTraceResponse udiTraceResponse : filterMap.values()) {
                            udiTraceResponseList.add(udiTraceResponse);
                        }

                    }


                }
            }


            PageInfo<UdiTraceResponse> pageInfo;
            pageInfo = new PageInfo<>(udiTraceResponseList);
            PageSimpleResponse<UdiTraceResponse> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(udiTraceResponseList);
            return ResultVOUtils.success(pageSimpleResponse);
        } else {
            return ResultVOUtils.error(500, "未找到该条码！");
        }
    }


    @GetMapping("/udims/trace/filter")
    public BaseResponse filterTrace(WarehouseQueryRequest warehouseQueryRequest) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        List<OrderMdEntity> orderMdEntities = new ArrayList<>();
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        if (authAdmin.getCustomerId() != 110)
            warehouseQueryRequest.setCustomerId(authAdmin.getCustomerId());
        if (warehouseQueryRequest.getCode() != null && !"".equals(warehouseQueryRequest.getCode())) {
            if (warehouseQueryRequest.getFilterType() == null || warehouseQueryRequest.getFilterType() == 1) {
                List<OrderMdEntity> temps = filterUdiTrace(warehouseQueryRequest);
                orderMdEntities.addAll(temps);
            } else {
                UdiEntity udiEntity = FilterUdiUtils.getUdi(warehouseQueryRequest.getCode());
                if (udiEntity == null)
                    return ResultVOUtils.error(500, "UDI码格式错误！");
                UdiInfoEntity udiInfoEntity = udiInfoService.findByNameCode(udiEntity.getUdi());
                List<UdiInfoEntity> udiInfoEntities = udiInfoService.findByUuids(udiInfoEntity.getUuid());
                if (udiInfoEntities != null && udiInfoEntities.size() > 0) {
                    for (UdiInfoEntity temp : udiInfoEntities) {
                        WarehouseQueryRequest queryRequest = new WarehouseQueryRequest();
                        queryRequest.setBatchNo(udiEntity.getBatchNo());
                        queryRequest.setUdi(temp.getNameCode());
                        if (authAdmin.getCustomerId() != 110)
                            queryRequest.setCustomerId(authAdmin.getCustomerId());

                        List<OrderMdEntity> temps = filterUdiTrace(queryRequest);
                        orderMdEntities.addAll(temps);
                    }
                }

            }
        }

        PageInfo<OrderMdEntity> pageInfo;
        pageInfo = new PageInfo<>(orderMdEntities);
        PageSimpleResponse<OrderMdEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderMdEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    public List<OrderMdEntity> filterUdiTrace(WarehouseQueryRequest warehouseQueryRequest) {
        List<OrderMdEntity> orderMdEntities = new ArrayList<>();
        List<WarehouseEntity> warehouseEntityList = warehouseMdService.filterWarehouse(warehouseQueryRequest);
        List<String> orderS = new ArrayList<>();
        warehouseEntityList.stream().filter(
                v -> {
                    boolean flag = !orderS.contains(v.getOrderId());
                    if (flag)
                        orderS.add(v.getOrderId());
                    return flag;
                }
        ).collect(Collectors.toList());


        for (String orderId : orderS) {
            OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
            orderFilterRequest.setId(orderId);
            List<OrderMdEntity> temps = orderMdService.findAll(orderFilterRequest);
            orderMdEntities.addAll(temps);
        }
        Collections.sort(orderMdEntities, Comparator.comparing(OrderMdEntity::getId));
        return orderMdEntities;
    }

    @GetMapping("/coor/trace/findCode")
    public BaseResponse findCode(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByOrderId(warehouseQueryRequest);
        if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
            WarehouseEntity warehouseEntity = warehouseEntityList.get(0);
            String udi = warehouseEntity.getCode().substring(2, 16);
            FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
            filterUdiInfoRequest.setNameCode(udi);
            List<UdiRelevanceResponse> udiRelevanceResponses = udiRelevanceService.filterUdiGp(filterUdiInfoRequest);
            UdiRelevanceResponse udiRelevance = null;
            if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
                udiRelevance = udiRelevanceResponses.get(0);
            }

            UdiRelevanceResponse finalUdiRelevance = udiRelevance;
            List<TraceUploadEntity> udiTraceResponseList = warehouseEntityList.stream().map(item ->
            {
                TraceUploadEntity udiTraceResponse = new TraceUploadEntity();
                udiTraceResponse.setUdiCode(item.getCode());
                udiTraceResponse.setBatchNo(item.getBatchNo());
                udiTraceResponse.setCorpName(item.getFromCorp());
                udiTraceResponse.setCount(item.getCount());
                udiTraceResponse.setCreditNum("9357002");
                try {
                    udiTraceResponse.setDocumentsDate(DateUtil.timeToStamp(item.getActDate()) + "");
                } catch (Exception e) {
                    e.printStackTrace();
                }
                udiTraceResponse.setDocumentsType(item.getAction());
                udiTraceResponse.setProductionDate(item.getProduceDate());
                udiTraceResponse.setExpireDate(item.getExpireDate());
                BeanUtils.copyProperties(item, udiTraceResponse);
                if (finalUdiRelevance != null) {
                    udiTraceResponse.setProductionCorp(finalUdiRelevance.getYlqxzcrbarmc());
                    udiTraceResponse.setProductName(finalUdiRelevance.getCpmctymc());
                    udiTraceResponse.setSpecifications(finalUdiRelevance.getGgxh());
                }
                return udiTraceResponse;
            }).collect(Collectors.toList());
            if (udiTraceResponseList != null && udiTraceResponseList.size() > 0) {
                for (TraceUploadEntity traceUploadEntity : udiTraceResponseList) {
                    String response = HttpClient.uCloudPost(coorUploadUrl, traceUploadEntity);
                    logger.info(response);
                    BaseResponse baseResponse =
                            JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
                            });
                }
            }
            return ResultVOUtils.success("查询成功，等待上传");
        } else {
            return ResultVOUtils.error(500, "未找到该条码！");
        }
    }
}
