package com.glxp.udi.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.inout.UnitMaintainEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.UnitMaintainFilterRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.inout.UnitMaintainService;
import com.glxp.udi.admin.util.GetAuthAdminUtil;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import java.util.List;

/**
 * 往来单位信息维护
 */
@RestController
public class UnitMaintainController {

    @Resource
    private UnitMaintainService unitMaintainService;
    @Resource
    AuthAdminService authAdminService;

    @GetMapping("/warehouse/unitMaintain/filter")
    public BaseResponse filterUnitMaintain(UnitMaintainFilterRequest unitMaintainFilterRequest,
                                           BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        unitMaintainFilterRequest.setCustomerId(authAdmin.getCustomerId() + "");

        List<UnitMaintainEntity> unitMaintainEntities;
        unitMaintainEntities = unitMaintainService.filterList(unitMaintainFilterRequest);
        PageInfo<UnitMaintainEntity> pageInfo;
        pageInfo = new PageInfo<>(unitMaintainEntities);
        PageSimpleResponse<UnitMaintainEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(unitMaintainEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @AuthRuleAnnotation("")
    @GetMapping("/warehouse/unit/download")
    public BaseResponse downloadUnit(UnitMaintainFilterRequest unitMaintainFilterRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        unitMaintainFilterRequest.setCustomerId(authAdmin.getCustomerId() + "");

        List<UnitMaintainEntity> unitMaintainEntities;
        unitMaintainEntities = unitMaintainService.filterList(unitMaintainFilterRequest);

        PageInfo<UnitMaintainEntity> pageInfo;
        pageInfo = new PageInfo<>(unitMaintainEntities);
        PageSimpleResponse<UnitMaintainEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(unitMaintainEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("")
    @GetMapping("/udiwms/unit/download")
    public BaseResponse downloadUnitB(UnitMaintainFilterRequest unitMaintainFilterRequest,
                                      BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        unitMaintainFilterRequest.setCustomerId(authAdmin.getCustomerId() + "");

        List<UnitMaintainEntity> unitMaintainEntities;
        unitMaintainEntities = unitMaintainService.filterList(unitMaintainFilterRequest);
        for (UnitMaintainEntity unitMaintainEntity : unitMaintainEntities) {
            unitMaintainEntity.setErpId(unitMaintainEntity.getUnitId());
        }
        PageInfo<UnitMaintainEntity> pageInfo;
        pageInfo = new PageInfo<>(unitMaintainEntities);
        PageSimpleResponse<UnitMaintainEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(unitMaintainEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/warehouse/unitMaintain/add")
    public BaseResponse addUnitMaintain(@RequestBody UnitMaintainEntity unitMaintainEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        unitMaintainEntity.setCustomerId(authAdmin.getCustomerId() + "");
        unitMaintainService.insertUnitMaintain(unitMaintainEntity);
        return ResultVOUtils.success("添加成功");
    }


    @PostMapping("/warehouse/unitMaintain/modify")
    public BaseResponse modifyUnitMaintain(@RequestBody UnitMaintainEntity unitMaintainEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
//
//        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
//        if (attributes == null) {
//            throw new JsonException(ResultEnum.NOT_NETWORK);
//        }
//        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
//        unitMaintainEntity.setCustomerId(authAdmin.getCustomerId() + "");
        unitMaintainService.updateUnit(unitMaintainEntity);
        return ResultVOUtils.success("添加成功");
    }

    @PostMapping("/warehouse/unitMaintain/delete")
    public BaseResponse deleteUnitMaintain(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();

        unitMaintainService.deleteById(id);

        return ResultVOUtils.success("删除成功");
    }
}
