package com.glxp.udi.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.inout.WarehouseEntity;
import com.glxp.udi.admin.entity.inout.WarehousetempEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.mongo.service.WarehouseMdService;
import com.glxp.udi.admin.mongo.service.WarehouseTempMdService;
import com.glxp.udi.admin.mongo.utils.PageResult;
import com.glxp.udi.admin.req.inout.*;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.inout.WarehouseResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.inout.WarehouseService;
import com.glxp.udi.admin.thread.TransInoutService;
import com.glxp.udi.admin.util.GetAuthAdminUtil;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class WareHouseController {
    @Resource
    private WarehouseService warehouseService;
    @Resource
    private WarehouseMdService warehouseMdService;
    @Resource
    private WarehouseTempMdService warehouseTempMdService;
    @Resource
    private TransInoutService transInoutService;

    @Resource
    AuthAdminService authAdminService;

    @AuthRuleAnnotation("")
    @PostMapping("warehouse/inout/postOrders")
    public BaseResponse postOrders(@RequestBody PostOrderRequest postOrderRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        AuthAdmin authAdmin = new GetAuthAdminUtil(authAdminService).getAuthAdmin(attributes);
        try {
            transInoutService.transInout(postOrderRequest, authAdmin.getCustomerId());
        } catch (Exception e) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success("导入成功");
    }

    @GetMapping("admin/warehouse/inout/findByOrderId")
    public BaseResponse findByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        PageResult<WarehouseEntity> pageResult = warehouseMdService.filterWarehouseByPage(warehouseQueryRequest);
        PageSimpleResponse<WarehouseEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageResult.getTotal());
        pageSimpleResponse.setList(pageResult.getList());
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("admin/warehouse/inout/findErrorByOrderId")
    public BaseResponse findErrorByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        PageResult<WarehousetempEntity> pageResult = warehouseTempMdService.filterWarehouseByPage(warehouseQueryRequest);
        PageSimpleResponse<WarehousetempEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageResult.getTotal());
        pageSimpleResponse.setList(pageResult.getList());
        return ResultVOUtils.success(pageSimpleResponse);
    }

    /**
     * -----------------------暂不确定----------------------
     */


    @AuthRuleAnnotation("")
    @GetMapping("admin/warehouse/inout/list")
    public BaseResponse list(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = warehouseService.listWarehouse(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/warehouse/inout/code/deleteCodes")
    public BaseResponse deleteCodes(@RequestBody DeleteCodeRequest deleteCodeRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Long> codes = deleteCodeRequest.getCodes();
        if (codes != null && codes.size() > 0) {
            warehouseService.deleteTempWarehouses(codes);
        } else {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR);
        }
        return ResultVOUtils.success("删除成功");
    }


    @GetMapping("admin/warehouse/inout/findByNo")
    public BaseResponse findByNo(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByNo(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("admin/warehouse/inout/findAllByOrderId")
    public BaseResponse findAllByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = warehouseService.findAllByOrderId(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/admin/warehouse/inout/save")
    public BaseResponse save(@RequestBody @Valid WarehouseSaveRequest warehouseSaveRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        WarehouseEntity warehouseEntity = new WarehouseEntity();
        BeanUtils.copyProperties(warehouseSaveRequest, warehouseEntity);

        boolean b = warehouseService.insertWarehouse(warehouseEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success("添加成功");
    }

    @AuthRuleAnnotation("")
    @PostMapping("/warehouse/inout/commit")
    public BaseResponse commitOrder(@RequestBody CommitRequest commitRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        transInoutService.repeatCommit(commitRequest);
        return ResultVOUtils.success("提交成功");
    }

}
