package com.glxp.udi.admin.controller.publish;

import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.controller.device.DownloadDeviceHelper;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.info.CompanyEntity;
import com.glxp.udi.admin.entity.publish.*;
import com.glxp.udi.admin.entity.udid.TokenEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.publish.PostIdentifyKeyRequest;
import com.glxp.udi.admin.req.publish.PostIdentifyRequest;
import com.glxp.udi.admin.res.publish.PostSaveResponse;
import com.glxp.udi.admin.res.publish.PostSubmitResponse;
import com.glxp.udi.admin.res.publish.UdiDeviceIdResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.info.CompanyService;
import com.glxp.udi.admin.service.publish.*;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
public class PostIdentifyController {

    @Resource
    CompanyService companyService;
    @Resource
    AuthAdminService authAdminService;
    @Resource
    UdiDeviceIdService udiDeviceIdService;
    @Resource
    UdiDeviceCliService udiDeviceCliService;
    @Resource
    UdiDevicepkService udiDevicepkService;
    @Resource
    UdiDeviceSrgService udiDeviceSrgService;
    @Resource
    UdiProductService udiProductService;
    @Resource
    UdiSubProductService udiSubProductService;

    //草稿上传
    @AuthRuleAnnotation("")
    @PostMapping("/udims/postIdentify/add")
    public BaseResponse add(@RequestBody @Valid PostIdentifyRequest postIdentifyRequest, BindingResult bindingResult) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        CompanyEntity companyEntity = companyService.findCompany(authAdmin.getCustomerId());
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyEntity);
        TokenEntity tokenEntity = downloadDeviceHelper.getToken();
        PostIdentifyEntity postIdentifyEntity = buildPost(postIdentifyRequest, tokenEntity.getAccessToken(), companyEntity);
        if (postIdentifyEntity == null) {
            return ResultVOUtils.error(500, errMsg);
        }
        PostSaveResponse postSaveResponse = downloadDeviceHelper.postIdentify(postIdentifyEntity);
        if (postSaveResponse.getReturnCode() == 1) {
            List<PostSuccessBean> successListBeans = postSaveResponse.getSuccessList();
            for (PostSuccessBean successListBean : successListBeans) {
                String status = "2";
                if (postIdentifyRequest.getUploadType().equals("change")) {
                    status = "3";
                }
                //标识状态更新
                successListBean.setStatus(status);
                udiDeviceIdService.postUpdate(successListBean);
                UdiDeviceIdEntity udiDeviceIdEntity = udiDeviceIdService.selectByPriId(successListBean.getZXXSDYCPBS());
                //更新子类信息状态
                UdiSubProductEntity udiSubProductEntity = new UdiSubProductEntity();
                udiSubProductEntity.setUDI_SubProductOID(udiDeviceIdEntity.getUDI_SubProduct_FK());
                udiSubProductEntity.setStatus(status);
                udiSubProductService.modifyUdiSubProduct(udiSubProductEntity);
                //更新企业产品状态
                UdiProductEntity udiProductEntity = new UdiProductEntity();
                udiProductEntity.setUDI_ProductOID(udiDeviceIdEntity.getUDI_Product_FK());
                udiProductEntity.setStatus(status);
                udiProductService.modifyUdiProduct(udiProductEntity);
                return ResultVOUtils.success("上传成功");
            }
        } else {
            List<PostSaveResponse.ErrorListBean> errorListBean = postSaveResponse.getErrorList();
            if (errorListBean != null && errorListBean.size() > 0) {
                List<PostSaveResponse.ErrorListBean.ErrorDetailListBean> errorDetailListBeans = errorListBean.get(0).getErrorDetailList();
                if (errorDetailListBeans != null && errorDetailListBeans.size() > 0) {
                    return ResultVOUtils.error(500, errorDetailListBeans.get(0).getErrorMsg());
                }
            }
        }

        return ResultVOUtils.error(500, "提交草稿上传失败");

    }

    String errMsg;

    public PostIdentifyEntity buildPost(PostIdentifyRequest postIdentifyRequest, String token, CompanyEntity companyEntity) {
        PostIdentifyEntity postIdentifyEntity = new PostIdentifyEntity();
        postIdentifyEntity.setAccessToken(token);
        List<PostDataBean> dataSetBeans = new ArrayList<>();
        List<String> ids = postIdentifyRequest.getIds();
        List<UdiDeviceIdResponse> udiDeviceIdResponses = udiDeviceIdService.selectByByIds(ids);

        for (int i = 0; i < udiDeviceIdResponses.size(); i++) {

            UdiDeviceIdResponse udiDeviceIdResponse = udiDeviceIdResponses.get(i);
            PostDataBean dataSetBean = new PostDataBean();
            UdiProductEntity udiProductEntity = udiProductService.selectById(udiDeviceIdResponse.getUDI_Product_FK());
            if (!postIdentifyRequest.getUploadType().equals("add") && udiDeviceIdResponse.getDeviceRecordKey() == null) {
                errMsg = "提交草稿上传失败,标识未草稿上传";
                return null;
            }
            if (postIdentifyRequest.getUploadType().equals("change")) {
                if (!udiProductEntity.getStatus().equals("3")) {
                    errMsg = "变更上传失败,标识未提交发布";
                    return null;
                }
                if (postIdentifyRequest.getBgsm() == null || postIdentifyRequest.getBgsm().equals("")) {
                    errMsg = "变更说明不能为空";
                    return null;
                }
                dataSetBean.setBGSM(postIdentifyRequest.getBgsm());
            }

            if (postIdentifyRequest.getUploadType().equals("modify")) {
                if (udiProductEntity.getStatus().equals("3")) {
                    errMsg = "修改上传失败,标识已提交发布";
                    return null;
                }
                if (udiProductEntity.getStatus().equals("1")) {
                    errMsg = "修改上传失败,标识未草稿上传";
                    return null;
                }
            }
            dataSetBean.setUploadType(postIdentifyRequest.getUploadType());
            dataSetBean.setCPMCTYMC(udiProductEntity.getProductName());
            dataSetBean.setTYSHXYDM(companyEntity.getCreditNum());
            dataSetBean.setDeviceRecordKey(udiDeviceIdResponse.getDeviceRecordKey());
            dataSetBean.setZXXSDYCPBS(udiDeviceIdResponse.getPrimaryDeviceId());
            dataSetBean.setZXXSDYZSYDYDSL(udiDeviceIdResponse.getPkgQuantity());
            if (udiDeviceIdResponse.getUseUnitDeviceId() == null)
                dataSetBean.setSYDYCPBS("");
            else
                dataSetBean.setSYDYCPBS(udiDeviceIdResponse.getUseUnitDeviceId());
            dataSetBean.setCPBSBMTXMC(udiDeviceIdResponse.getIssuingAgency());
            if (udiDeviceIdResponse.getIsDirectMark().equals("2")) {
                dataSetBean.setSFYBTZJBS("0");
            } else dataSetBean.setSFYBTZJBS("1");

            dataSetBean.setBTCPBSYZXXSDYCPBSSFYZ(udiDeviceIdResponse.getIsSameDeviceId());
            dataSetBean.setBTCPBS(udiDeviceIdResponse.getDirectMarkDI());
            if (udiDeviceIdResponse.getDeviceCarrier() != null) {
                String data = udiDeviceIdResponse.getDeviceCarrier().replace(";", ",");
                dataSetBean.setBSZT(data);//
            }

            dataSetBean.setSFYZCBAYZ(udiDeviceIdResponse.getIsSameRegDI());
            dataSetBean.setZCBACPBS(udiDeviceIdResponse.getRegistrationDI());
            dataSetBean.setCPBSFBRQ(udiDeviceIdResponse.getPublishDate());
//            dataSetBean.setCPMCTYMC(udiDeviceIdResponse.getProductName());
            dataSetBean.setSPMC(udiDeviceIdResponse.getBrandName());
            dataSetBean.setGGXH(udiDeviceIdResponse.getSpecification());
            if (udiDeviceIdResponse.getDeviceKit() != null && udiDeviceIdResponse.getDeviceKit().equals("2")) {
                dataSetBean.setSFWBLZTLCP("0");
            } else
                dataSetBean.setSFWBLZTLCP("1");

            dataSetBean.setCPMS(udiDeviceIdResponse.getDeviceDescription());
            dataSetBean.setCPHHHBH(udiDeviceIdResponse.getCatalogNumber());
            if (udiDeviceIdResponse.getDeviceType() != null && udiDeviceIdResponse.getDeviceType().equals("体外诊断试剂"))
                dataSetBean.setCPLX("2");
            else
                dataSetBean.setCPLX("1");
            dataSetBean.setFLBM(udiDeviceIdResponse.getProductCode());
            dataSetBean.setYFLBM(udiDeviceIdResponse.getProductOldCode());
            dataSetBean.setYLQXZCRBARMC(udiDeviceIdResponse.getCompanyName());
            dataSetBean.setYLQXZCRBARYWMC(udiDeviceIdResponse.getCompanyGBName());
//            dataSetBean.setTYSHXYDM(udiDeviceIdResponse.getBusinessLicense());
            dataSetBean.setZCZBHHZBAPZBH(udiDeviceIdResponse.getRegcardCodes());
            if (udiDeviceIdResponse.getProductType() != null && udiDeviceIdResponse.getProductType().equals("耗材"))
                dataSetBean.setHCHZSB("0");
            else dataSetBean.setHCHZSB("1");
            if (udiDeviceIdResponse.getSingleUse() != null && udiDeviceIdResponse.getSingleUse().equals("1"))
                dataSetBean.setSFBJWYCXSY("1");
            else
                dataSetBean.setSFBJWYCXSY("0");
            dataSetBean.setZDCFSYCS(udiDeviceIdResponse.getReuseTimes());

            if (udiDeviceIdResponse.getIsDeviceSterile() != null && udiDeviceIdResponse.getIsDeviceSterile().equals("1"))
                dataSetBean.setSFWWJBZ("1");
            else dataSetBean.setSFWWJBZ("0");
            if (udiDeviceIdResponse.getIsSterilizationUse() != null && udiDeviceIdResponse.getIsSterilizationUse().equals("1"))
                dataSetBean.setSYQSFXYJXMJ("1");
            else dataSetBean.setSYQSFXYJXMJ("0");
//            dataSetBean.setSYQSFXYJXMJ(udiDeviceIdResponse.getIsSterilizationUse());
            dataSetBean.setMJFS(udiDeviceIdResponse.getSterilizationMethod());
            dataSetBean.setYBBM(udiDeviceIdResponse.getYBBM());
            if (udiDeviceIdResponse.getMRISafetyInfo() == null)
                dataSetBean.setCGZMRAQXGXX("3");
            else {
                int value = Integer.parseInt(udiDeviceIdResponse.getMRISafetyInfo()) - 1;
                dataSetBean.setCGZMRAQXGXX(value + "");
            }

            dataSetBean.setTSCCHCZTJ(udiDeviceIdResponse.getSpecialText());
            dataSetBean.setTSCCSM(udiDeviceIdResponse.getClinicalSizeText());
//            dataSetBean.setSCBSSFBHPH(udiDeviceIdResponse.getIsLotBatch());

            if (udiDeviceIdResponse.getIsLotBatch() != null && udiDeviceIdResponse.getIsLotBatch().equals("2"))
                dataSetBean.setSCBSSFBHPH("0");
            else dataSetBean.setSCBSSFBHPH("1");

            if (udiDeviceIdResponse.getIsSerialNumber() != null && udiDeviceIdResponse.getIsSerialNumber().equals("2"))
                dataSetBean.setSCBSSFBHXLH("0");
            else dataSetBean.setSCBSSFBHXLH("1");
//            dataSetBean.setSCBSSFBHXLH(udiDeviceIdResponse.getIsSerialNumber());
            dataSetBean.setSCBSSFBHSCRQ(udiDeviceIdResponse.getIsManufacturingDate());
            if (udiDeviceIdResponse.getIsExpirationDate() != null && udiDeviceIdResponse.getIsExpirationDate().equals("2"))
                dataSetBean.setSCBSSFBHSXRQ("0");
            else dataSetBean.setSCBSSFBHSXRQ("1");

            dataSetBean.setQTXXDWZLJ(udiDeviceIdResponse.getOtherInfoLink());

            List<PostDeviceClinicalBean> deviceClinicalBeans = new ArrayList<>();
            List<UdiDeviceCliEntity> udiDeviceCliEntities = udiDeviceCliService.selectBySubProductId(udiDeviceIdResponse.getUDI_SubProduct_FK());
            if (udiDeviceCliEntities != null && udiDeviceCliEntities.size() > 0) {
                for (UdiDeviceCliEntity udiDeviceCliEntity : udiDeviceCliEntities) {
                    PostDeviceClinicalBean deviceClinicalBean = new PostDeviceClinicalBean();
                    deviceClinicalBean.setCCDW(udiDeviceCliEntity.getCCDW());
                    deviceClinicalBean.setLCSYCCLX(udiDeviceCliEntity.getLCSYCCLX());
                    deviceClinicalBean.setCCZ(udiDeviceCliEntity.getCCZ());
                    deviceClinicalBeans.add(deviceClinicalBean);

                }
            }
            List<PostDevicePackageBean> devicePackageBeans = new ArrayList<>();
            List<UdiDevicepkEntity> udiDevicepkEntities = udiDevicepkService.selectByDeviceId(udiDeviceIdResponse.getUDI_DeviceIdOID());
            if (udiDevicepkEntities != null && udiDevicepkEntities.size() > 0) {
                for (UdiDevicepkEntity udiDevicepkEntity : udiDevicepkEntities) {
                    PostDevicePackageBean devicePackageBean = new PostDevicePackageBean();
                    devicePackageBean.setBZCPBS(udiDevicepkEntity.getPkgDeviceId());
                    devicePackageBean.setBZNHXYJBZCPBS(udiDevicepkEntity.getContainsDIPackage());
                    devicePackageBean.setBZNHXYJCPBSSL(udiDevicepkEntity.getPkgQuantity());
                    devicePackageBean.setCPBZJB(udiDevicepkEntity.getPkgType());
                    devicePackageBeans.add(devicePackageBean);
                }
            }
            List<PostDeviceStorageBean> deviceStorageBeans = new ArrayList<>();
            List<UdiDeviceSrgEntity> udiDeviceSrgEntities = udiDeviceSrgService.selectBySubProduct(udiDeviceIdResponse.getUDI_SubProduct_FK());
            if (udiDeviceSrgEntities != null && udiDeviceSrgEntities.size() > 0) {
                for (UdiDeviceSrgEntity udiDeviceSrgEntity : udiDeviceSrgEntities) {
                    PostDeviceStorageBean deviceStorageBean = new PostDeviceStorageBean();
                    deviceStorageBean.setCCHCZTJ(udiDeviceSrgEntity.getStorageType());
                    deviceStorageBean.setZDZ(udiDeviceSrgEntity.getLowValue());
                    deviceStorageBean.setZGZ(udiDeviceSrgEntity.getHighValue());
                    deviceStorageBean.setJLDW(udiDeviceSrgEntity.getUnit());
                    deviceStorageBeans.add(deviceStorageBean);
                }
            }
            dataSetBean.setTSRQ(udiDeviceIdResponse.getDeviceEndDate());
//            dataSetBean.setBGSM(udiDeviceIdResponse);
            dataSetBean.setDevicePackage(devicePackageBeans);
            dataSetBean.setDeviceStorage(deviceStorageBeans);
            dataSetBean.setDeviceClinical(deviceClinicalBeans);
            dataSetBeans.add(dataSetBean);
            postIdentifyEntity.setDataSet(dataSetBeans);
        }

        return postIdentifyEntity;
    }


    //提交标识
    @AuthRuleAnnotation("")
    @PostMapping("/udims/postIdentify/upload")
    public BaseResponse postUpload(@RequestBody @Valid PostIdentifyRequest postIdentifyRequest, BindingResult bindingResult) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        CompanyEntity companyEntity = companyService.findCompany(authAdmin.getCustomerId());
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyEntity);
        TokenEntity tokenEntity = downloadDeviceHelper.getToken();
        List<UdiDeviceIdResponse> udiDeviceIdResponses = udiDeviceIdService.selectKeysByIds(postIdentifyRequest.getIds());

        if (udiDeviceIdResponses != null && udiDeviceIdResponses.size() > 0) {
            PostIdentifyKeyRequest postIdentifyKeyRequest = new PostIdentifyKeyRequest();
            List<PostIdentifyKeyRequest.DataSetBean> dataSetBeans = new ArrayList<>();
            for (UdiDeviceIdResponse udiDeviceIdResponse : udiDeviceIdResponses) {

                PostIdentifyKeyRequest.DataSetBean dataSetBean = new PostIdentifyKeyRequest.DataSetBean();
                if (udiDeviceIdResponse.getDeviceRecordKey() == null) {
                    return ResultVOUtils.error(500, "提交的标识包含未上传草稿，不能提交");
                } else
                    dataSetBean.setDeviceRecordKey(udiDeviceIdResponse.getDeviceRecordKey());
                dataSetBean.setZXXSDYCPBS(udiDeviceIdResponse.getPrimaryDeviceId());
                dataSetBeans.add(dataSetBean);
            }
            postIdentifyKeyRequest.setDataSet(dataSetBeans);
            postIdentifyKeyRequest.setAccessToken(tokenEntity.getAccessToken());
            PostSubmitResponse postSaveResponse = downloadDeviceHelper.postIdentifyKey(postIdentifyKeyRequest);
            if (postSaveResponse.getReturnCode() == 1) {
                for (PostIdentifyKeyRequest.DataSetBean dataSetBean : dataSetBeans) {
                    //更新产品标识状态
                    PostSuccessBean postSuccessBean = new PostSuccessBean();
                    postSuccessBean.setDeviceRecordKey(dataSetBean.getDeviceRecordKey());
                    postSuccessBean.setZXXSDYCPBS(dataSetBean.getZXXSDYCPBS());
                    postSuccessBean.setStatus("3");
                    udiDeviceIdService.postUpdate(postSuccessBean);

                    UdiDeviceIdEntity udiDeviceIdEntity = udiDeviceIdService.selectByPriId(postSuccessBean.getZXXSDYCPBS());
                    //更新子类信息状态
                    UdiSubProductEntity udiSubProductEntity = new UdiSubProductEntity();
                    udiSubProductEntity.setUDI_SubProductOID(udiDeviceIdEntity.getUDI_SubProduct_FK());
                    udiSubProductEntity.setStatus("3");
                    udiSubProductService.modifyUdiSubProduct(udiSubProductEntity);
                    //更新企业产品状态
                    UdiProductEntity udiProductEntity = new UdiProductEntity();
                    udiProductEntity.setUDI_ProductOID(udiDeviceIdEntity.getUDI_Product_FK());
                    udiProductEntity.setStatus("3");
                    udiProductService.modifyUdiProduct(udiProductEntity);
                }

            } else {
                String errMsg = "提交失败";
                if (postSaveResponse.getErrorList() != null && postSaveResponse.getErrorList().size() > 0) {
                    List<PostSubmitResponse.ErrorListBean.ErrorDetailListBean> errorDetailListBeans = postSaveResponse.getErrorList().get(0).getErrorDetailList();
                    if (errorDetailListBeans != null && errorDetailListBeans.size() > 0)
                        errMsg = errorDetailListBeans.get(0).getErrorMsg();

                }

                return ResultVOUtils.error(500, errMsg);
            }

        } else {
            return ResultVOUtils.error(500, "上传数据为空");
        }
        return ResultVOUtils.success("添加成功");
    }


    //提交标识
    @AuthRuleAnnotation("")
    @PostMapping("/udims/postIdentify/uploadNow")
    public BaseResponse postUploadNow(@RequestBody @Valid PostIdentifyRequest postIdentifyRequest, BindingResult bindingResult) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        CompanyEntity companyEntity = companyService.findCompany(authAdmin.getCustomerId());
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyEntity);
        TokenEntity tokenEntity = downloadDeviceHelper.getToken();
        List<UdiDeviceIdResponse> udiDeviceIdResponses = udiDeviceIdService.selectKeysByIds(postIdentifyRequest.getIds());

        if (udiDeviceIdResponses != null && udiDeviceIdResponses.size() > 0) {
            PostIdentifyKeyRequest postIdentifyKeyRequest = new PostIdentifyKeyRequest();
            List<PostIdentifyKeyRequest.DataSetBean> dataSetBeans = new ArrayList<>();
            for (UdiDeviceIdResponse udiDeviceIdResponse : udiDeviceIdResponses) {

                PostIdentifyKeyRequest.DataSetBean dataSetBean = new PostIdentifyKeyRequest.DataSetBean();
                dataSetBean.setDeviceRecordKey(udiDeviceIdResponse.getDeviceRecordKey());
                dataSetBean.setZXXSDYCPBS(udiDeviceIdResponse.getPrimaryDeviceId());
                dataSetBeans.add(dataSetBean);
            }
            postIdentifyKeyRequest.setDataSet(dataSetBeans);
            postIdentifyKeyRequest.setAccessToken(tokenEntity.getAccessToken());
            PostSubmitResponse postSaveResponse = downloadDeviceHelper.postIdentifyKey(postIdentifyKeyRequest);
            if (postSaveResponse.getReturnCode() == 1) {
                for (PostIdentifyKeyRequest.DataSetBean dataSetBean : dataSetBeans) {
                    //更新产品标识状态
                    PostSuccessBean postSuccessBean = new PostSuccessBean();
                    postSuccessBean.setDeviceRecordKey(dataSetBean.getDeviceRecordKey());
                    postSuccessBean.setZXXSDYCPBS(dataSetBean.getZXXSDYCPBS());
                    postSuccessBean.setStatus("3");
                    udiDeviceIdService.postUpdate(postSuccessBean);

                    UdiDeviceIdEntity udiDeviceIdEntity = udiDeviceIdService.selectByPriId(postSuccessBean.getZXXSDYCPBS());
                    //更新子类信息状态
                    UdiSubProductEntity udiSubProductEntity = new UdiSubProductEntity();
                    udiSubProductEntity.setUDI_SubProductOID(udiDeviceIdEntity.getUDI_SubProduct_FK());
                    udiSubProductEntity.setStatus("3");
                    udiSubProductService.modifyUdiSubProduct(udiSubProductEntity);
                    //更新企业产品状态
                    UdiProductEntity udiProductEntity = new UdiProductEntity();
                    udiProductEntity.setUDI_ProductOID(udiDeviceIdEntity.getUDI_Product_FK());
                    udiProductEntity.setStatus("3");
                    udiProductService.modifyUdiProduct(udiProductEntity);
                }

            } else {
                return ResultVOUtils.error(500, "提交失败");
            }
        } else {
            return ResultVOUtils.error(500, "上传数据为空");
        }
        return ResultVOUtils.success("添加成功");
    }

}
