package com.glxp.udi.admin.controller.publish;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.info.IdentifierEntity;
import com.glxp.udi.admin.entity.publish.UdiDeviceCliEntity;
import com.glxp.udi.admin.entity.publish.UdiDeviceIdEntity;
import com.glxp.udi.admin.entity.publish.UdiDeviceSrgEntity;
import com.glxp.udi.admin.entity.publish.UdiSubProductEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.publish.*;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.info.IdentifierService;
import com.glxp.udi.admin.service.publish.UdiDeviceCliService;
import com.glxp.udi.admin.service.publish.UdiDeviceIdService;
import com.glxp.udi.admin.service.publish.UdiDeviceSrgService;
import com.glxp.udi.admin.service.publish.UdiSubProductService;
import com.glxp.udi.admin.util.DateUtil;
import com.glxp.udi.admin.util.UuidUtils;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
public class SubUdiProductController {

    @Resource
    private UdiSubProductService udiSubProductService;
    @Resource
    private UdiDeviceCliService udiDeviceCliService;
    @Resource
    private UdiDeviceSrgService udiDeviceSrgService;
    @Resource
    private IdentifierService identifierService;
    @Resource
    private UdiDeviceIdService udiDeviceIdService;
    @Resource
    AuthAdminService authAdminService;

    @AuthRuleAnnotation("")
    @PostMapping("/udims/subProduct/add")
    public BaseResponse add(@RequestBody @Valid AddSubProductRequest addSubProductRequest, BindingResult bindingResult) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        UdiSubProductEntity udiSubProductEntity = addSubProductRequest.getUdiSubProductEntity();
        udiSubProductEntity.setCustomerId(authAdmin.getCustomerId() + "");
        String productId = UuidUtils.getUUId();

        if (udiSubProductEntity != null) {
            if (udiSubProductEntity.getSPTM() != null && udiSubProductEntity.getSPTM().length() == 13) {
                if (identifierService.isSptmUsed(udiSubProductEntity.getSPTM().
                        substring(0, udiSubProductEntity.getSPTM().length() - 1))) {
                    return ResultVOUtils.error(500, "商品条码已被使用");
                }
            }
            udiSubProductEntity.setSYS_Created(DateUtil.getDateTime());
            udiSubProductEntity.setUDI_SubProductOID(productId);
            udiSubProductEntity.setStatus("1");
            udiSubProductService.insertUdiSubProduct(udiSubProductEntity);
        } else return ResultVOUtils.error(500, "添加失败");
        List<UdiDeviceCliEntity> udiDeviceCliEntities = addSubProductRequest.getUdiDeviceCliEntities();
        List<UdiDeviceCliEntity> insertClis = new ArrayList<>();

        if (udiDeviceCliEntities != null && udiDeviceCliEntities.size() > 0) {
            for (UdiDeviceCliEntity udiDeviceCliEntity : udiDeviceCliEntities) {
                udiDeviceCliEntity.setUDI_DeviceCliOID(UuidUtils.getUUId());
                udiDeviceCliEntity.setUDI_SubProduct_FK(productId);
                insertClis.add(udiDeviceCliEntity);
            }
            udiDeviceCliService.insertUdiDeviceCliList(insertClis);
        }
        List<UdiDeviceSrgEntity> udiDeviceSrgEntities = addSubProductRequest.getUdiDeviceSrgEntities();
        List<UdiDeviceSrgEntity> insertSrgs = new ArrayList<>();
        if (udiDeviceSrgEntities != null && udiDeviceSrgEntities.size() > 0) {
            for (UdiDeviceSrgEntity udiDeviceSrgEntity : udiDeviceSrgEntities) {
                udiDeviceSrgEntity.setUDI_DeviceSrgOID(UuidUtils.getUUId());
                udiDeviceSrgEntity.setUDI_SubProduct_FK(productId);
                insertSrgs.add(udiDeviceSrgEntity);
            }
            udiDeviceSrgService.insertUdiDeviceSrgList(insertSrgs);
        }
        if (udiSubProductEntity.getSPTM() != null && udiSubProductEntity.getSPTM().length() == 13) {
            String ideCode = udiSubProductEntity.getSPTM().
                    substring(0, udiSubProductEntity.getSPTM().length() - 1);
            identifierService.updateIsUsed(new IdentifierEntity(null, "1", ideCode, null));

        }
        return ResultVOUtils.success("添加成功");
    }


    @AuthRuleAnnotation("")
    @PostMapping("/udims/subProduct/modify")
    public BaseResponse modify(@RequestBody @Valid AddSubProductRequest addSubProductRequest, BindingResult bindingResult) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        UdiSubProductEntity udiSubProductEntity = addSubProductRequest.getUdiSubProductEntity();
        udiSubProductEntity.setCustomerId(authAdmin.getCustomerId() + "");
        UdiSubProductEntity temp = udiSubProductService.selectById(udiSubProductEntity.getUDI_SubProductOID());
        udiSubProductEntity.setSYS_Created(DateUtil.getDateTime());
        if (udiSubProductEntity != null) {

            if (udiSubProductEntity.getSPTM() != null && udiSubProductEntity.getSPTM().length() == 13 && !temp.getSPTM().equals(udiSubProductEntity.getSPTM())) {
                if (identifierService.isSptmUsed(udiSubProductEntity.getSPTM().
                        substring(0, udiSubProductEntity.getSPTM().length() - 1))) {
                    return ResultVOUtils.error(500, "商品条码已被使用");
                }
            }
            udiSubProductService.modifyUdiSubProduct(udiSubProductEntity);
        } else return ResultVOUtils.success("修改失败");
        List<UdiDeviceCliEntity> udiDeviceCliEntities = addSubProductRequest.getUdiDeviceCliEntities();
        List<UdiDeviceCliEntity> insertClis = new ArrayList<>();

        if (udiDeviceCliEntities != null && udiDeviceCliEntities.size() > 0) {
            for (UdiDeviceCliEntity udiDeviceCliEntity : udiDeviceCliEntities) {
                if (udiDeviceCliEntity.getUDI_DeviceCliOID() == null || udiDeviceCliEntity.getUDI_DeviceCliOID().equals("")) {
                    udiDeviceCliEntity.setUDI_DeviceCliOID(UuidUtils.getUUId());
                    udiDeviceCliEntity.setUDI_SubProduct_FK(udiSubProductEntity.getUDI_SubProductOID());
                }
                if (udiDeviceCliEntity.getIsDelete() != 1)
                    insertClis.add(udiDeviceCliEntity);
                else {
                    DeleteUdiDeviceCliRequest deleteUdiDeviceCliRequest = new DeleteUdiDeviceCliRequest();
                    deleteUdiDeviceCliRequest.setId(udiDeviceCliEntity.getUDI_DeviceCliOID());
                    udiDeviceCliService.deleteUdiDeviceCli(deleteUdiDeviceCliRequest);
                }
            }
            udiDeviceCliService.insertUdiDeviceCliList(insertClis);
        }
        List<UdiDeviceSrgEntity> udiDeviceSrgEntities = addSubProductRequest.getUdiDeviceSrgEntities();
        List<UdiDeviceSrgEntity> insertSrgs = new ArrayList<>();
        if (udiDeviceSrgEntities != null && udiDeviceSrgEntities.size() > 0) {
            for (UdiDeviceSrgEntity udiDeviceSrgEntity : udiDeviceSrgEntities) {
                if (udiDeviceSrgEntity.getUDI_DeviceSrgOID() == null || udiDeviceSrgEntity.getUDI_DeviceSrgOID().equals("")) {
                    udiDeviceSrgEntity.setUDI_DeviceSrgOID(UuidUtils.getUUId());
                    udiDeviceSrgEntity.setUDI_SubProduct_FK(udiSubProductEntity.getUDI_SubProductOID());
                }
                if (udiDeviceSrgEntity.getIsDelete() != 1)
                    insertSrgs.add(udiDeviceSrgEntity);
                else {
                    DeleteUdiDeviceSrgRequest deleteUdiDeviceSrgRequest = new DeleteUdiDeviceSrgRequest();
                    deleteUdiDeviceSrgRequest.setId(udiDeviceSrgEntity.getUDI_DeviceSrgOID());
                    udiDeviceSrgService.deleteUdiDeviceSrg(deleteUdiDeviceSrgRequest);
                }

            }
            udiDeviceSrgService.insertUdiDeviceSrgList(insertSrgs);
        }
        if (udiSubProductEntity.getSPTM() != null && udiSubProductEntity.getSPTM().length() == 13) {
            String ideCode = udiSubProductEntity.getSPTM().
                    substring(0, udiSubProductEntity.getSPTM().length() - 1);
            identifierService.updateIsUsed(new IdentifierEntity(null, "1", ideCode, null));

        }
        return ResultVOUtils.success("修改成功");
    }


    @AuthRuleAnnotation("")
    @PostMapping("/udims/subProduct/delete")
    public BaseResponse deleteSubProduct(@RequestBody @Valid DeleteUdiSubProductRequest
                                                 deleteUdiProductRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<UdiDeviceIdEntity> udiDeviceIdEntities = udiDeviceIdService.selectBySubProductId(deleteUdiProductRequest.getId());
        if(udiDeviceIdEntities!=null && udiDeviceIdEntities.size()>0){
            return ResultVOUtils.error(500, "删除失败,请先删除该产品标识信息！");
        }
        UdiSubProductEntity udiSubProductEntity = udiSubProductService.selectById(deleteUdiProductRequest.getId());
        boolean b = udiSubProductService.deleteUdiSubProduct(deleteUdiProductRequest);
        if (!b) {
            return ResultVOUtils.error(500, "删除失败");
        }

        if (udiSubProductEntity!=null && udiSubProductEntity.getSPTM() != null && udiSubProductEntity.getSPTM().length() == 13) {
            String ideCode = udiSubProductEntity.getSPTM().
                    substring(0, udiSubProductEntity.getSPTM().length() - 1);
            identifierService.updateIsUsed(new IdentifierEntity(null, "0", ideCode, null));

        }

        return ResultVOUtils.success("删除成功");
    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/subProduct/list")
    public BaseResponse listSubProduct(@Valid FilterUdiSubProductRequest filterUdiSubProductRequest, BindingResult
            bindingResult) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        filterUdiSubProductRequest.setCustomerId(authAdmin.getCustomerId() + "");
        List<UdiSubProductEntity> udiProductEntities = udiSubProductService.filterAll(filterUdiSubProductRequest);
        PageInfo<UdiSubProductEntity> pageInfo = new PageInfo<>(udiProductEntities);
        PageSimpleResponse<UdiSubProductEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiProductEntities);
        return ResultVOUtils.success(pageSimpleResponse);

    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/subProduct/cliList")
    public BaseResponse listCli(@Valid FilterCliRequest filterCliRequest, BindingResult
            bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiDeviceCliEntity> udiDeviceCliEntities = udiDeviceCliService.selectBySubProductId(filterCliRequest.getSubProductId());
        return ResultVOUtils.success(udiDeviceCliEntities);

    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/subProduct/srgList")
    public BaseResponse listSrg(@Valid FilterCliRequest filterCliRequest, BindingResult
            bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiDeviceSrgEntity> udiDeviceSrgEntities = udiDeviceSrgService.selectBySubProduct(filterCliRequest.getSubProductId());
        return ResultVOUtils.success(udiDeviceSrgEntities);

    }


}
