package com.glxp.udi.admin.controller.publish;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.info.IdentifierEntity;
import com.glxp.udi.admin.entity.publish.UdiDeviceIdEntity;
import com.glxp.udi.admin.entity.publish.UdiDevicepkEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.publish.*;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.publish.UdiDeviceIdResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.info.IdentifierService;
import com.glxp.udi.admin.service.publish.UdiDeviceIdService;
import com.glxp.udi.admin.service.publish.UdiDevicepkService;
import com.glxp.udi.admin.service.publish.UdiSubProductService;
import com.glxp.udi.admin.util.DateUtil;
import com.glxp.udi.admin.util.UuidUtils;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

//产品标识

@RestController
public class UdiDeviceIdController {

    @Resource
    private UdiSubProductService udiSubProductService;
    @Resource
    private UdiDeviceIdService udiDeviceIdService;
    @Resource
    private UdiDevicepkService udiDevicepkService;
    @Resource
    AuthAdminService authAdminService;

    @Resource
    private IdentifierService identifierService;

    @AuthRuleAnnotation("")
    @PostMapping("/udims/deviceId/add")
    public BaseResponse add(@RequestBody @Valid AddUdiDeviceIdRequest addUdiDeviceIdRequest, BindingResult bindingResult) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        UdiDeviceIdEntity udiDeviceIdEntity = addUdiDeviceIdRequest.getUdiDeviceIdEntity();
        udiDeviceIdEntity.setSYS_CreatedBy(DateUtil.getDateTime());
        udiDeviceIdEntity.setCustomerId(authAdmin.getCustomerId()+"");
        List<UdiDevicepkEntity> udiDevicepkEntities = addUdiDeviceIdRequest.getUdiDevicepkEntities();
        String productId = UuidUtils.getUUId();

        if (udiDeviceIdEntity != null) {
            if (udiDeviceIdEntity.getPrimaryDeviceId() != null && udiDeviceIdEntity.getPrimaryDeviceId().length() > 10) {
                if (identifierService.isUdiUsed(udiDeviceIdEntity.getPrimaryDeviceId().
                        substring(1, udiDeviceIdEntity.getPrimaryDeviceId().length() - 1))) {
                    return ResultVOUtils.error(500, "最小销售单元产品标识条码已被使用");
                }

            }
            if (udiDeviceIdEntity.getDirectMarkDI() != null && udiDeviceIdEntity.getDirectMarkDI().length() > 10) {
                if (identifierService.isUdiUsed(udiDeviceIdEntity.getDirectMarkDI().
                        substring(1, udiDeviceIdEntity.getDirectMarkDI().length() - 1))) {
                    return ResultVOUtils.error(500, "本体产品标识条码已被使用");
                }
            }
            if (udiDeviceIdEntity.getUseUnitDeviceId() != null && udiDeviceIdEntity.getUseUnitDeviceId().length() > 10) {
                if (identifierService.isUdiUsed(udiDeviceIdEntity.getUseUnitDeviceId().
                        substring(1, udiDeviceIdEntity.getUseUnitDeviceId().length() - 1))) {
                    return ResultVOUtils.error(500, "使用单元产品标识条码已被使用");
                }
            }
            if (udiDevicepkEntities != null && udiDevicepkEntities.size() > 0) {
                for (UdiDevicepkEntity udiDevicepkEntity : udiDevicepkEntities) {
                    if (identifierService.isUdiUsed(udiDevicepkEntity.getPkgDeviceId().
                            substring(1, udiDevicepkEntity.getPkgDeviceId().length() - 1))) {
                        return ResultVOUtils.error(500, "包装产品标识条码已被使用");
                    }
                }
            }
            udiDeviceIdEntity.setUDI_DeviceIdOID(productId);
            udiDeviceIdEntity.setStatus("1");
            udiDeviceIdService.insertUdiDeviceId(udiDeviceIdEntity);
        } else return ResultVOUtils.success("添加失败");

        List<UdiDevicepkEntity> insertPkgs = new ArrayList<>();

        if (udiDevicepkEntities != null && udiDevicepkEntities.size() > 0) {
            for (UdiDevicepkEntity udiDevicepkEntity : udiDevicepkEntities) {
                udiDevicepkEntity.setUDI_DevicePkgOID(UuidUtils.getUUId());
                udiDevicepkEntity.setUDI_DeviceId_FK(productId);
                insertPkgs.add(udiDevicepkEntity);
            }
            udiDevicepkService.insertUdiDevicepkList(insertPkgs);
        }
        updateIdentifyStatus(udiDeviceIdEntity,udiDevicepkEntities);
        return ResultVOUtils.success("添加成功");

    }

    public void updateIdentifyStatus(UdiDeviceIdEntity udiDeviceIdEntity, List<UdiDevicepkEntity> udiDevicepkEntities) {
        if (udiDeviceIdEntity.getPrimaryDeviceId() != null && udiDeviceIdEntity.getPrimaryDeviceId().length() > 10) {
            String ideCode = udiDeviceIdEntity.getPrimaryDeviceId().
                    substring(1, udiDeviceIdEntity.getPrimaryDeviceId().length() - 1);
                identifierService.updateIsUsed(new IdentifierEntity(null, null, ideCode, "1"));

        }
        if (udiDeviceIdEntity.getDirectMarkDI() != null && udiDeviceIdEntity.getDirectMarkDI().length() > 10) {
            String ideCode = udiDeviceIdEntity.getDirectMarkDI().
                    substring(1, udiDeviceIdEntity.getDirectMarkDI().length() - 1);
                identifierService.updateIsUsed(new IdentifierEntity(null, null, ideCode, "1"));
        }
        if (udiDeviceIdEntity.getUseUnitDeviceId() != null && udiDeviceIdEntity.getUseUnitDeviceId().length() > 10) {
            String ideCode = udiDeviceIdEntity.getUseUnitDeviceId().
                    substring(1, udiDeviceIdEntity.getUseUnitDeviceId().length() - 1);
                identifierService.updateIsUsed(new IdentifierEntity(null, null, ideCode, "1"));
        }
        if (udiDevicepkEntities != null && udiDevicepkEntities.size() > 0) {
            for (UdiDevicepkEntity udiDevicepkEntity : udiDevicepkEntities) {
                String ideCode = udiDevicepkEntity.getPkgDeviceId().
                        substring(1, udiDevicepkEntity.getPkgDeviceId().length() - 1);
                    identifierService.updateIsUsed(new IdentifierEntity(null, null, ideCode, "1"));
            }
        }
    }

    @AuthRuleAnnotation("")
    @PostMapping("/udims/deviceId/delete")
    public BaseResponse deleteDeviceId(@RequestBody @Valid DeleteUdiDeviceIdRequest deleteUdiProductRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        UdiDeviceIdEntity udiDeviceIdEntity = udiDeviceIdService.selectById(deleteUdiProductRequest.getId());
        boolean b = udiDeviceIdService.deleteUdiDeviceId(deleteUdiProductRequest);
        //删除关联数据
        //toDo
        if (udiDeviceIdEntity.getPrimaryDeviceId() != null && udiDeviceIdEntity.getPrimaryDeviceId().length() > 10) {
            String ideCode = udiDeviceIdEntity.getPrimaryDeviceId().
                    substring(1, udiDeviceIdEntity.getPrimaryDeviceId().length() - 1);
            identifierService.updateIsUsed(new IdentifierEntity(null, null, ideCode, "0"));

        }
        if (udiDeviceIdEntity.getDirectMarkDI() != null && udiDeviceIdEntity.getDirectMarkDI().length() > 10) {
            String ideCode = udiDeviceIdEntity.getDirectMarkDI().
                    substring(1, udiDeviceIdEntity.getDirectMarkDI().length() - 1);
            identifierService.updateIsUsed(new IdentifierEntity(null, null, ideCode, "0"));
        }
        if (udiDeviceIdEntity.getUseUnitDeviceId() != null && udiDeviceIdEntity.getUseUnitDeviceId().length() > 10) {
            String ideCode = udiDeviceIdEntity.getUseUnitDeviceId().
                    substring(1, udiDeviceIdEntity.getUseUnitDeviceId().length() - 1);
            identifierService.updateIsUsed(new IdentifierEntity(null, null, ideCode, "0"));
        }
       List<UdiDevicepkEntity>  udiDevicepkEntities =  udiDevicepkService.selectByDeviceId(deleteUdiProductRequest.getId());
        if (udiDevicepkEntities != null && udiDevicepkEntities.size() > 0) {
            for (UdiDevicepkEntity udiDevicepkEntity : udiDevicepkEntities) {
                String ideCode = udiDevicepkEntity.getPkgDeviceId().
                        substring(1, udiDevicepkEntity.getPkgDeviceId().length() - 1);
                identifierService.updateIsUsed(new IdentifierEntity(null, null, ideCode, "0"));
            }
        }

        if (!b) {
            return ResultVOUtils.error(500, "删除失败");
        }
        return ResultVOUtils.success("删除成功");
    }


    @AuthRuleAnnotation("")
    @PostMapping("/udims/deviceId/modify")
    public BaseResponse modify(@RequestBody @Valid AddUdiDeviceIdRequest addUdiDeviceIdRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        UdiDeviceIdEntity udiDeviceIdEntity = addUdiDeviceIdRequest.getUdiDeviceIdEntity();
        udiDeviceIdEntity.setSYS_CreatedBy(DateUtil.getDateTime());
        UdiDeviceIdEntity temp =  udiDeviceIdService.selectById(udiDeviceIdEntity.getUDI_DeviceIdOID());
        if (udiDeviceIdEntity.getPrimaryDeviceId() != null && udiDeviceIdEntity.getPrimaryDeviceId().length() > 13 && !udiDeviceIdEntity.getPrimaryDeviceId().equals(temp.getPrimaryDeviceId())) {
            if (identifierService.isUdiUsed(udiDeviceIdEntity.getPrimaryDeviceId().
                    substring(1, udiDeviceIdEntity.getPrimaryDeviceId().length() - 1))) {
                return ResultVOUtils.error(500, "最小销售单元产品标识条码已被使用");
            }

        }
        if (udiDeviceIdEntity.getDirectMarkDI() != null && udiDeviceIdEntity.getDirectMarkDI().length() > 13 && !udiDeviceIdEntity.getPrimaryDeviceId().equals(temp.getPrimaryDeviceId())) {
            if (identifierService.isUdiUsed(udiDeviceIdEntity.getDirectMarkDI().
                    substring(1, udiDeviceIdEntity.getDirectMarkDI().length() - 1))) {
                return ResultVOUtils.error(500, "本体产品标识条码已被使用");
            }
        }
        if (udiDeviceIdEntity.getUseUnitDeviceId() != null && udiDeviceIdEntity.getUseUnitDeviceId().length() > 13 && !udiDeviceIdEntity.getPrimaryDeviceId().equals(temp.getPrimaryDeviceId())) {
            if (identifierService.isUdiUsed(udiDeviceIdEntity.getUseUnitDeviceId().
                    substring(1, udiDeviceIdEntity.getUseUnitDeviceId().length() - 1))) {
                return ResultVOUtils.error(500, "使用单元产品标识条码已被使用");
            }
        }


        List<UdiDevicepkEntity> udiDevicepkEntities = addUdiDeviceIdRequest.getUdiDevicepkEntities();
        if (udiDeviceIdEntity != null) {
            udiDeviceIdService.modifyUdiDeviceId(udiDeviceIdEntity);
        } else return ResultVOUtils.error(500, "修改失败");

        List<UdiDevicepkEntity> insertPkgs = new ArrayList<>();

        if (udiDevicepkEntities != null && udiDevicepkEntities.size() > 0) {
            for (UdiDevicepkEntity udiDevicepkEntity : udiDevicepkEntities) {
                if (udiDevicepkEntity.getUDI_DevicePkgOID() == null || udiDevicepkEntity.getUDI_DevicePkgOID().equals("")) {
                    udiDevicepkEntity.setUDI_DevicePkgOID(UuidUtils.getUUId());
                    udiDevicepkEntity.setUDI_DeviceId_FK(udiDeviceIdEntity.getUDI_DeviceIdOID());
                }
                if (udiDevicepkEntity.getIsDelete() != 1)
                    insertPkgs.add(udiDevicepkEntity);
                else {
                    DeleteUdiDevicepkRequest deleteUdiDevicepkRequest = new DeleteUdiDevicepkRequest();
                    deleteUdiDevicepkRequest.setId(udiDevicepkEntity.getUDI_DevicePkgOID());
                    udiDevicepkService.deleteUdiDevicepk(deleteUdiDevicepkRequest);
                }
            }
            udiDevicepkService.insertUdiDevicepkList(insertPkgs);
        }
        updateIdentifyStatus(udiDeviceIdEntity,udiDevicepkEntities);

        return ResultVOUtils.success("修改成功");
    }


    @AuthRuleAnnotation("")
    @GetMapping("/udims/deviceId/list")
    public BaseResponse listDeviceId(@Valid FilterDeviceIdRequest filterDeviceIdRequest, BindingResult bindingResult) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        filterDeviceIdRequest.setCustomerId(authAdmin.getCustomerId() + "");
        List<UdiDeviceIdResponse> udiDeviceIdEntities = udiDeviceIdService.selectByProductId(filterDeviceIdRequest);

        PageInfo<UdiDeviceIdResponse> pageInfo = new PageInfo<>(udiDeviceIdEntities);
        PageSimpleResponse<UdiDeviceIdResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiDeviceIdEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/deviceId/pkgList")
    public BaseResponse listPkg(@Valid FilterPkgRequest filterPkgRequest, BindingResult
            bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiDevicepkEntity> udiDevicepkEntities = udiDevicepkService.selectByDeviceId(filterPkgRequest.getDeviceId());
        return ResultVOUtils.success(udiDevicepkEntities);

    }


}
