package com.glxp.udi.admin.controller.publish;

import com.github.pagehelper.PageInfo;
import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.publish.UdiProductEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.publish.DeleteUdiProductRequest;
import com.glxp.udi.admin.req.publish.FilterUdiProductRequest;
import com.glxp.udi.admin.res.PageSimpleResponse;
import com.glxp.udi.admin.res.publish.UdiProductResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.publish.UdiProductService;
import com.glxp.udi.admin.service.publish.UdiSubProductService;
import com.glxp.udi.admin.util.UuidUtils;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
public class UdiProductController {
    @Resource
    private UdiProductService udiProductService;
    @Resource
    private UdiSubProductService udiSubProductService;
    @Resource
    private AuthAdminService authAdminService;

    @AuthRuleAnnotation("")
    @PostMapping("/udims/product/add")
    public BaseResponse add(@RequestBody @Valid UdiProductEntity udiProductEntity, BindingResult bindingResult) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        udiProductEntity.setUDI_Customer_FK(authAdmin.getCustomerId() + "");
        udiProductEntity.setUDI_ProductOID(UuidUtils.getUUId());
        udiProductEntity.setStatus("1");
        boolean b = udiProductService.insertUdiProduct(udiProductEntity);
        if (!b)
            return ResultVOUtils.success("添加失败");
        return ResultVOUtils.success("添加成功");
    }

    @AuthRuleAnnotation("")
    @PostMapping("/udims/product/delete")
    public BaseResponse deleteCustomerContrast(@RequestBody @Valid DeleteUdiProductRequest deleteUdiProductRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        int count = udiSubProductService.selectCount(deleteUdiProductRequest.getId());
        if (count > 0) {
            return ResultVOUtils.error(500, "请先删除子类信息");
        }
        boolean b = udiProductService.deleteUdiProduct(deleteUdiProductRequest);

        if (!b) {
            return ResultVOUtils.error(500, "删除失败");
        }
        return ResultVOUtils.success("删除成功");
    }


    @AuthRuleAnnotation("")
    @PostMapping("/udims/product/modify")
    public BaseResponse modify(@RequestBody @Valid UdiProductEntity udiProductEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = udiProductService.modifyUdiProduct(udiProductEntity);
        if (!b)
            return ResultVOUtils.success("修改失败");
        return ResultVOUtils.success("修改成功");
    }

    @AuthRuleAnnotation("")
    @GetMapping("/udims/product/list")
    public BaseResponse listUdiProduct(@Valid FilterUdiProductRequest filterUdiProductRequest, BindingResult bindingResult) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        filterUdiProductRequest.setCustomerId(authAdmin.getCustomerId() + "");
        List<UdiProductEntity> udiProductEntities = udiProductService.filterAll(filterUdiProductRequest);

        PageInfo<UdiProductEntity> pageInfo = new PageInfo<>(udiProductEntities);
        PageSimpleResponse<UdiProductEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiProductEntities);
        return ResultVOUtils.success(pageSimpleResponse);

    }


    @AuthRuleAnnotation("")
    @GetMapping("/udims/product/all")
    public BaseResponse selectAll() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        List<UdiProductEntity> udiProductEntities = udiProductService.selectByCustomerId(authAdmin.getCustomerId());

        List<UdiProductResponse> udiProductResponses = new ArrayList<>();
        if (udiProductEntities != null && udiProductEntities.size() > 0) {
            for (UdiProductEntity udiProductEntity : udiProductEntities) {
                UdiProductResponse udiProductResponse = new UdiProductResponse();
                udiProductResponse.setLabel(udiProductEntity.getProductName());
                udiProductResponse.setValue(udiProductEntity.getUDI_ProductOID());
                udiProductResponses.add(udiProductResponse);
            }
        }
        return ResultVOUtils.success(udiProductResponses);

    }

}
