package com.glxp.udi.admin.mongo.service;

import com.glxp.udi.admin.entity.inout.InvProductDetailEntity;
import com.glxp.udi.admin.mongo.MongoDBName;
import com.glxp.udi.admin.mongo.utils.MongoPageHelper;
import com.glxp.udi.admin.mongo.utils.PageResult;
import com.glxp.udi.admin.req.inout.FilterInvProductDetailRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Service
public class InvProductDetailMdService {

    @Autowired
    private MongoTemplate mongoTemplate;
    @Resource
    MongoPageHelper mongoPageHelper;

    public PageResult<InvProductDetailEntity> filterInvProduct(FilterInvProductDetailRequest filterInvProductDetailRequest) {
        Query query = new Query();
        if (filterInvProductDetailRequest.getCode() != null && !filterInvProductDetailRequest.getCode().isEmpty()) {
            query.addCriteria(Criteria.where("code").is(filterInvProductDetailRequest.getCode()));
        }
        if (filterInvProductDetailRequest.getProductIdFk() != null && !filterInvProductDetailRequest.getProductIdFk().isEmpty()) {
            query.addCriteria(Criteria.where("productIdFk").is(filterInvProductDetailRequest.getProductIdFk()));
        }
        if (filterInvProductDetailRequest.getMainAction() != null && !filterInvProductDetailRequest.getMainAction().isEmpty()) {
            query.addCriteria(Criteria.where("mainAction").is(filterInvProductDetailRequest.getMainAction()));
        }
        if (filterInvProductDetailRequest.getCustomerId() != null && !filterInvProductDetailRequest.getCustomerId().isEmpty()) {
            query.addCriteria(Criteria.where("customerId").is(filterInvProductDetailRequest.getCustomerId()));
        }
        return mongoPageHelper.pageQuery(query, InvProductDetailEntity.class,
                filterInvProductDetailRequest.getLimit(), filterInvProductDetailRequest.getPage(),
                filterInvProductDetailRequest.getLastId(), MongoDBName.NAME_INV_PRODUCT_DETAIL);

    }

    public void insertInvProduct(InvProductDetailEntity invProductDetailEntity) {
        mongoTemplate.insert(invProductDetailEntity, MongoDBName.NAME_INV_PRODUCT_DETAIL);
    }

    public void insertInvProducts(List<InvProductDetailEntity> invProductDetailEntitys) {
        if (invProductDetailEntitys != null) {
            Collection<InvProductDetailEntity> invProductDetailEntityCollection = new ArrayList<>();
            invProductDetailEntityCollection.addAll(invProductDetailEntitys);
            mongoTemplate.insert(invProductDetailEntityCollection, MongoDBName.NAME_INV_PRODUCT_DETAIL);
        }

    }

    public void updateInvProduct(InvProductDetailEntity invProductDetailEntity) {
        Query query = new Query();
        query.addCriteria(new Criteria().and("id").is(invProductDetailEntity.getId()));
        Update update = new Update();

        if (invProductDetailEntity.getCount() != null) {
            update.set("count", invProductDetailEntity.getCount());
        }
        if (invProductDetailEntity.getCode() != null) {
            update.set("code", invProductDetailEntity.getCode());
        }
        if (invProductDetailEntity.getProductIdFk() != null) {
            update.set("productIdFk", invProductDetailEntity.getProductIdFk());
        }
        if (invProductDetailEntity.getOrderIdFk() != null) {
            update.set("orderIdFk", invProductDetailEntity.getOrderIdFk());
        }
        if (invProductDetailEntity.getCustomerId() != null) {
            update.set("customerId", invProductDetailEntity.getCustomerId());
        }
        if (invProductDetailEntity.getMainAction() != null) {
            update.set("mainAction", invProductDetailEntity.getMainAction());
        }
        if (invProductDetailEntity.getAction() != null) {
            update.set("action", invProductDetailEntity.getAction());
        }
        mongoTemplate.updateMulti(query, update, MongoDBName.NAME_INV_PRODUCT_DETAIL);
    }

    public void deleteById(String id) {
        Query query = new Query();
        if (id != null) {
            query.addCriteria(Criteria.where("id").is(id));
        }
        mongoTemplate.remove(query, InvProductDetailEntity.class, MongoDBName.NAME_INV_PRODUCT_DETAIL);
    }

    public void deleteByProductId(String productIdFk) {
        Query query = new Query();
        if (productIdFk != null) {
            query.addCriteria(Criteria.where("productIdFk").is(productIdFk));
        }
        mongoTemplate.remove(query, InvProductDetailEntity.class, MongoDBName.NAME_INV_PRODUCT_DETAIL);
    }
}
