package com.glxp.udi.admin.mongo.service;

import com.glxp.udi.admin.entity.inout.InvProductEntity;
import com.glxp.udi.admin.mongo.MongoDBName;
import com.glxp.udi.admin.mongo.utils.MongoPageHelper;
import com.glxp.udi.admin.mongo.utils.PageResult;
import com.glxp.udi.admin.req.inout.FilterInvProductRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class InvProductMdService {
    @Autowired
    private MongoTemplate mongoTemplate;
    @Resource
    MongoPageHelper mongoPageHelper;

    public PageResult<InvProductEntity> filterInvProduct(FilterInvProductRequest filterInvProductRequest) {
        Query query = new Query();
        if (filterInvProductRequest.getUuids() != null && filterInvProductRequest.getUuids().size() > 0) {
            query.addCriteria(Criteria.where("relIdFk").in(filterInvProductRequest.getUuids()));
        }
        if (filterInvProductRequest.getBatchNo() != null && !filterInvProductRequest.getBatchNo().isEmpty()) {
            query.addCriteria(Criteria.where("batchNo").is(filterInvProductRequest.getBatchNo()));
        }
        if (filterInvProductRequest.getCustomerId() != null && !filterInvProductRequest.getCustomerId().isEmpty()) {
            query.addCriteria(Criteria.where("customerId").is(filterInvProductRequest.getCustomerId()));
        }
        return mongoPageHelper.pageQuery(query, InvProductEntity.class,
                filterInvProductRequest.getLimit(), filterInvProductRequest.getPage(), filterInvProductRequest.getLastId(), MongoDBName.NAME_INV_PRODUCT);
    }


    public InvProductEntity selectByUuid(String relIdFk, String batchNo) {
        Query query = new Query();
        if (relIdFk != null && !relIdFk.isEmpty()) {
            query.addCriteria(Criteria.where("relIdFk").is(relIdFk));
        }
        if (batchNo != null && !batchNo.isEmpty()) {
            query.addCriteria(Criteria.where("batchNo").is(batchNo));
        }
        List<InvProductEntity> invProductEntities = mongoTemplate.find(query, InvProductEntity.class, MongoDBName.NAME_INV_PRODUCT);
        if (invProductEntities != null && invProductEntities.size() > 0) {
            return invProductEntities.get(0);
        }
        return null;
    }

    public InvProductEntity selectById(String id) {
        Query query = new Query();
        if (id != null && !id.isEmpty()) {
            query.addCriteria(Criteria.where("id").is(id));
        }
        List<InvProductEntity> invProductEntities = mongoTemplate.find(query, InvProductEntity.class, MongoDBName.NAME_INV_PRODUCT);
        if (invProductEntities != null && invProductEntities.size() > 0) {
            return invProductEntities.get(0);
        }
        return null;
    }


    public void insertInvProduct(InvProductEntity invProductEntity) {
        mongoTemplate.insert(invProductEntity, MongoDBName.NAME_INV_PRODUCT);
    }

    public void updateInvProduct(InvProductEntity invProductEntity) {
        Query query = new Query();
        query.addCriteria(new Criteria().and("id").is(invProductEntity.getId()));
        Update update = new Update();

        if (invProductEntity.getInCount() != null) {
            update.set("inCount", invProductEntity.getInCount());
        }
        if (invProductEntity.getOutCount() != null) {
            update.set("outCount", invProductEntity.getOutCount());
        }
        if (invProductEntity.getProductsName() != null) {
            update.set("productsName", invProductEntity.getProductsName());
        }
        if (invProductEntity.getNameCode() != null) {
            update.set("nameCode", invProductEntity.getNameCode());
        }
        if (invProductEntity.getRelIdFk() != null) {
            update.set("relIdFk", invProductEntity.getRelIdFk());
        }
        if (invProductEntity.getGgxh() != null) {
            update.set("ggxh", invProductEntity.getGgxh());
        }
        if (invProductEntity.getBatchNo() != null) {
            update.set("batchNo", invProductEntity.getBatchNo());
        }
        if (invProductEntity.getProductionDate() != null) {
            update.set("productionDate", invProductEntity.getProductionDate());
        }
        if (invProductEntity.getExpireDate() != null) {
            update.set("expireDate", invProductEntity.getExpireDate());
        }
        if (invProductEntity.getYlqxzcrbarmc() != null) {
            update.set("ylqxzcrbarmc", invProductEntity.getYlqxzcrbarmc());
        }
        if (invProductEntity.getZczbhhzbapzbh() != null) {
            update.set("zczbhhzbapzbh", invProductEntity.getZczbhhzbapzbh());
        }
        if (invProductEntity.getReCount() != null) {
            update.set("reCount", invProductEntity.getReCount());
        }
        if (invProductEntity.getCustomerId() != null) {
            update.set("customerId", invProductEntity.getCustomerId());
        }
        mongoTemplate.updateMulti(query, update, MongoDBName.NAME_INV_PRODUCT);
    }


    public void deleteById(String id) {
        Query query = new Query();
        if (id != null) {
            query.addCriteria(Criteria.where("id").is(id));
        }
        mongoTemplate.remove(query, InvProductEntity.class, MongoDBName.NAME_INV_PRODUCT);
    }

}
