package com.glxp.udi.admin.mongo.service;

import com.glxp.udi.admin.entity.inout.ErpOrderEntity;
import com.glxp.udi.admin.mongo.MongoDBName;
import com.glxp.udi.admin.mongo.utils.MongoPageHelper;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.FilterErpOrderRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Service
public class OrderDetailMdService {
    @Autowired
    private MongoTemplate mongoTemplate;
    @Resource
    MongoPageHelper mongoPageHelper;

    public List<ErpOrderEntity> filterOrderDetails(FilterErpOrderRequest filterErpOrderRequest) {

        Query query = new Query();
        if (filterErpOrderRequest.getOrderId() != null && !filterErpOrderRequest.getOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("orderIdFk").is(filterErpOrderRequest.getOrderId()));
        }
        if (filterErpOrderRequest.getCustomerId() != null) {
            query.addCriteria(Criteria.where("customerId").is(filterErpOrderRequest.getCustomerId()));
        }
        return mongoTemplate.find(query, ErpOrderEntity.class, MongoDBName.NAME_ORDER_DETAIL);
    }

    public void insertOrderDetail(List<ErpOrderEntity> erpOrderEntities) {
        if (erpOrderEntities != null) {
            Collection<ErpOrderEntity> warehouseEntityCollection = new ArrayList<>();
            warehouseEntityCollection.addAll(erpOrderEntities);
            mongoTemplate.insert(warehouseEntityCollection, MongoDBName.NAME_ORDER_DETAIL);
        }
    }

    public void deleteOrderDetail(DeleteRequest deleteRequest) {
        Query query = new Query();
        if (deleteRequest.getId() != null) {
            query.addCriteria(Criteria.where("id").is(deleteRequest.getId()));
        }
        if (deleteRequest.getOrderId() != null) {
            query.addCriteria(Criteria.where("orderIdFk").is(deleteRequest.getOrderId()));
        }
        mongoTemplate.remove(query, ErpOrderEntity.class, MongoDBName.NAME_ORDER_DETAIL);
    }

    public void updateOrderDetail(ErpOrderEntity erpOrderEntity) {
        Query query = new Query();
        query.addCriteria(new Criteria().and("id").is(erpOrderEntity.getId()));
        Update update = new Update();

        if (erpOrderEntity.getCount() != null) {
            update.set("count", erpOrderEntity.getCount());
        }
        if (erpOrderEntity.getReCount() != null) {
            update.set("reCount", erpOrderEntity.getReCount());
        }
        mongoTemplate.updateMulti(query, update, MongoDBName.NAME_ORDER_DETAIL);
    }
}
