package com.glxp.udi.admin.mongo.service;

import com.glxp.udi.admin.entity.inout.OrderMdEntity;
import com.glxp.udi.admin.mongo.MongoDBName;
import com.glxp.udi.admin.mongo.utils.MongoPageHelper;
import com.glxp.udi.admin.mongo.utils.PageResult;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.OrderFilterRequest;
import com.glxp.udi.admin.util.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

@Service
public class OrderMdService {

    @Autowired
    private MongoTemplate mongoTemplate;
    @Resource
    MongoPageHelper mongoPageHelper;

    private static final Logger log = LoggerFactory.getLogger(OrderMdService.class);

    public PageResult<OrderMdEntity> filterList(OrderFilterRequest orderFilterRequest) {
        Query query = new Query();
        if (orderFilterRequest.getId() != null && !orderFilterRequest.getId().isEmpty()) {
            query.addCriteria(Criteria.where("id").is(orderFilterRequest.getId()));
        }
        if (orderFilterRequest.getMainAction() != null && !orderFilterRequest.getMainAction().isEmpty()) {
            query.addCriteria(Criteria.where("mainAction").is(orderFilterRequest.getMainAction()));
        }
        if (orderFilterRequest.getAction() != null && !orderFilterRequest.getAction().isEmpty()) {
            query.addCriteria(Criteria.where("action").is(orderFilterRequest.getAction()));
        }
        if (orderFilterRequest.getAction() == null || orderFilterRequest.getAction().isEmpty()) {
            query.addCriteria(Criteria.where("action").ne("StockCheck"));
        }
        if (orderFilterRequest.getStatus() != null && !orderFilterRequest.getStatus().isEmpty()) {
            query.addCriteria(Criteria.where("status").is(orderFilterRequest.getStatus()));
        }
        if (orderFilterRequest.getExportStatus() != null && !orderFilterRequest.getExportStatus().isEmpty()) {
            query.addCriteria(Criteria.where("exportStatus").is(orderFilterRequest.getExportStatus()));
        }
        if (orderFilterRequest.getCustomerId() != null) {
            if (orderFilterRequest.getStartTime() != null && !orderFilterRequest.getStartTime().isEmpty()
                    && orderFilterRequest.getEndTime() != null && !orderFilterRequest.getEndTime().isEmpty()) {
                Date sdate, edate = null;
                if (orderFilterRequest.getStartTime().contains("T")) {
                    sdate = DateUtil.convertTimeFormat(orderFilterRequest.getStartTime());
                } else {
                    sdate = DateUtil.parseDate(orderFilterRequest.getStartTime());
                }
                if (orderFilterRequest.getEndTime().contains("T")) {
                    edate = DateUtil.convertTimeFormat(orderFilterRequest.getEndTime());
                } else {
                    edate = DateUtil.parseDate(orderFilterRequest.getEndTime());
                }
                query.addCriteria(Criteria.where("customerId").is(orderFilterRequest.getCustomerId()).andOperator(Criteria.where("actDate").gte(DateUtil.dateToISODate(sdate)
                ), Criteria.where("actDate").lt(DateUtil.dateToISODate(edate))));
            } else
                query.addCriteria(Criteria.where("customerId").is(orderFilterRequest.getCustomerId()));
        }


        if (orderFilterRequest.getIsError() != null) {
            query.addCriteria(Criteria.where("status").ne(orderFilterRequest.getIsError()));
        }

        return mongoPageHelper.pageQuery(query, OrderMdEntity.class,
                orderFilterRequest.getLimit(), orderFilterRequest.getPage(), orderFilterRequest.getLastId(), MongoDBName.NAME_ORDER);

    }

    public List<OrderMdEntity> findAll(OrderFilterRequest orderFilterRequest) {
        Query query = new Query();
        if (orderFilterRequest.getId() != null && !orderFilterRequest.getId().isEmpty()) {
            query.addCriteria(Criteria.where("orderId").is(orderFilterRequest.getId()));
        }
        if (orderFilterRequest.getMainAction() != null && !orderFilterRequest.getMainAction().isEmpty()) {
            query.addCriteria(Criteria.where("mainAction").is(orderFilterRequest.getMainAction()));
        }
        if (orderFilterRequest.getAction() != null && !orderFilterRequest.getAction().isEmpty()) {
            query.addCriteria(Criteria.where("action").is(orderFilterRequest.getAction()));
        }
        if (orderFilterRequest.getAction() == null || orderFilterRequest.getAction().isEmpty()) {
            query.addCriteria(Criteria.where("action").ne("StockCheck"));
        }
        if (orderFilterRequest.getStatus() != null && !orderFilterRequest.getStatus().isEmpty()) {
            query.addCriteria(Criteria.where("status").is(orderFilterRequest.getStatus()));
        }
        if (orderFilterRequest.getExportStatus() != null && !orderFilterRequest.getExportStatus().isEmpty()) {
            query.addCriteria(Criteria.where("exportStatus").is(orderFilterRequest.getExportStatus()));
        }
        if (orderFilterRequest.getCustomerId() != null) {
            if (orderFilterRequest.getStartTime() != null && !orderFilterRequest.getStartTime().isEmpty()
                    && orderFilterRequest.getEndTime() != null && !orderFilterRequest.getEndTime().isEmpty()) {
                Date sdate, edate = null;
                if (orderFilterRequest.getStartTime().contains("T")) {
                    sdate = DateUtil.convertTimeFormat(orderFilterRequest.getStartTime());
                } else {
                    sdate = DateUtil.parseDate(orderFilterRequest.getStartTime());
                }
                if (orderFilterRequest.getEndTime().contains("T")) {
                    edate = DateUtil.convertTimeFormat(orderFilterRequest.getEndTime());
                } else {
                    edate = DateUtil.parseDate(orderFilterRequest.getEndTime());
                }
                query.addCriteria(Criteria.where("customerId").is(orderFilterRequest.getCustomerId()).andOperator(Criteria.where("actDate").gte(DateUtil.dateToISODate(sdate)
                ), Criteria.where("actDate").lt(DateUtil.dateToISODate(edate))));
            } else
                query.addCriteria(Criteria.where("customerId").is(orderFilterRequest.getCustomerId()));
        }

        if (orderFilterRequest.getIsError() != null) {
            query.addCriteria(Criteria.where("status").ne(orderFilterRequest.getIsError()));
        }
        return mongoTemplate.find(query, OrderMdEntity.class, MongoDBName.NAME_ORDER);
    }

    public void updateOrder(OrderMdEntity orderMdEntity) {
        Query query = new Query();
        if (orderMdEntity.getOrderId() != null && !orderMdEntity.getOrderId().isEmpty()) {
            query.addCriteria(new Criteria().and("orderId").is(orderMdEntity.getOrderId()));
        } else if (orderMdEntity.getId() != null && !orderMdEntity.getId().isEmpty()) {
            query.addCriteria(new Criteria().and("id").is(orderMdEntity.getId()));
        } else {
            log.error("更新失败，未指定orderId");
            return;
        }

        Update update = new Update();
        if (orderMdEntity.getId() != null) {
            update.set("fromCorp", orderMdEntity.getFromCorp());
        }
        if (orderMdEntity.getFromCorp() != null) {
            update.set("fromCorp", orderMdEntity.getFromCorp());
        }
        if (orderMdEntity.getFromCorpId() != null) {
            update.set("fromCorpId", orderMdEntity.getFromCorpId());
        }
        if (orderMdEntity.getActDate() != null) {
            update.set("actDate", orderMdEntity.getActDate());
        }
        if (orderMdEntity.getStatus() != null) {
            update.set("status", orderMdEntity.getStatus());
        }
        if (orderMdEntity.getRemark() != null) {
            update.set("remark", orderMdEntity.getRemark());
        }
        if (orderMdEntity.getExportStatus() != null) {
            update.set("exportStatus", orderMdEntity.getExportStatus());
        }
        if (orderMdEntity.getContrastStatus() != null) {
            update.set("contrastStatus", orderMdEntity.getContrastStatus());
        }
        if (orderMdEntity.getErpFk() != null) {
            update.set("erpFk", orderMdEntity.getErpFk());
        }
        mongoTemplate.updateMulti(query, update, MongoDBName.NAME_ORDER);
    }

    public void deleteOrder(DeleteRequest deleteRequest) {
        Query query = new Query();
        if (deleteRequest.getId() != null) {
            query.addCriteria(Criteria.where("id").is(deleteRequest.getId()));
        }
        if (deleteRequest.getOrderId() != null) {
            query.addCriteria(Criteria.where("orderId").is(deleteRequest.getOrderId()));
        }
        mongoTemplate.remove(query, OrderMdEntity.class, MongoDBName.NAME_ORDER);
    }


    public void insertOrder(OrderMdEntity orderMdEntity) {
        if (orderMdEntity != null) {
            Collection<OrderMdEntity> orderMdEntities = new ArrayList<>();
            orderMdEntities.add(orderMdEntity);
            mongoTemplate.insert(orderMdEntities, MongoDBName.NAME_ORDER);
        }
    }
}
