package com.glxp.udi.admin.mongo.service;

import com.glxp.udi.admin.entity.inout.WarehouseEntity;
import com.glxp.udi.admin.mongo.MongoDBName;
import com.glxp.udi.admin.mongo.utils.MongoPageHelper;
import com.glxp.udi.admin.mongo.utils.PageResult;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.FilterWarehouseRequest;
import com.glxp.udi.admin.req.inout.WarehouseQueryRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Service
public class WarehouseMdService {

    @Autowired
    private MongoTemplate mongoTemplate;
    @Resource
    MongoPageHelper mongoPageHelper;


    public PageResult<WarehouseEntity> filterWarehouseByPage(WarehouseQueryRequest warehouseQueryRequest) {

        Query query = new Query();
        if (warehouseQueryRequest.getCode() != null && !warehouseQueryRequest.getCode().isEmpty()) {
            query.addCriteria(Criteria.where("code").is(warehouseQueryRequest.getCode()));
        }
        if (warehouseQueryRequest.getCorpOrderId() != null && !warehouseQueryRequest.getCorpOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("corpOrderId").is(warehouseQueryRequest.getCorpOrderId()));
        }
        if (warehouseQueryRequest.getOrderId() != null && !warehouseQueryRequest.getOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("orderId").is(warehouseQueryRequest.getOrderId()));
        }
//        if (warehouseQueryRequest.getCustomerId() != null) {
//            query.addCriteria(Criteria.where("customerId").is(warehouseQueryRequest.getCustomerId()));
//        }
        return mongoPageHelper.pageQuery(query, WarehouseEntity.class,
                warehouseQueryRequest.getLimit(), warehouseQueryRequest.getPage(), warehouseQueryRequest.getLastId(), MongoDBName.NAME_CODE);
    }

    public List<WarehouseEntity> filterWarehouse(WarehouseQueryRequest warehouseQueryRequest) {

        Query query = new Query();
        if (warehouseQueryRequest.getCode() != null && !warehouseQueryRequest.getCode().isEmpty()) {
            query.addCriteria(Criteria.where("code").is(warehouseQueryRequest.getCode()));
        }
        if (warehouseQueryRequest.getCorpOrderId() != null && !warehouseQueryRequest.getCorpOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("corpOrderId").is(warehouseQueryRequest.getCorpOrderId()));
        }
        if (warehouseQueryRequest.getOrderId() != null && !warehouseQueryRequest.getOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("orderId").is(warehouseQueryRequest.getOrderId()));
        }
        if (warehouseQueryRequest.getCustomerId() != null) {
            query.addCriteria(Criteria.where("customerId").is(warehouseQueryRequest.getCustomerId()));
        }
        if (warehouseQueryRequest.getBatchNo() != null) {
            query.addCriteria(Criteria.where("batchNo").is(warehouseQueryRequest.getBatchNo()));
        }
        if (warehouseQueryRequest.getUdi() != null) {
            query.addCriteria(Criteria.where("udi").is(warehouseQueryRequest.getUdi()));
        }


        return mongoTemplate.find(query, WarehouseEntity.class, MongoDBName.NAME_CODE);
    }


    public void insertWarehouses(List<WarehouseEntity> warehouseEntities) {
        if (warehouseEntities != null) {
            Collection<WarehouseEntity> warehouseEntityCollection = new ArrayList<>();
            warehouseEntityCollection.addAll(warehouseEntities);
            mongoTemplate.insert(warehouseEntityCollection, MongoDBName.NAME_CODE);
        }
    }


    public List<WarehouseEntity> findByCode(FilterWarehouseRequest filterWarehouseRequest) {
        Query query = new Query();
        query.addCriteria(
                Criteria.where("customerId").is(filterWarehouseRequest.getCustomerId())
                        .and("code").is(filterWarehouseRequest.getCode())
        );
        return mongoTemplate.find(query, WarehouseEntity.class, MongoDBName.NAME_CODE);
    }


    public List<WarehouseEntity> searchCode(String code, Long customerId) {
        Query query = new Query();
        query.addCriteria(
                Criteria.where("customerId").is(customerId)
                        .and("code").is(code)
                        .and("action").ne("StockCheck")
        );
        return mongoTemplate.find(query, WarehouseEntity.class, MongoDBName.NAME_CODE);
    }

    public void updateUnit(String orderId, String fromCorpId, String fromCorp) {

        Query query = new Query();
        query.addCriteria(new Criteria().and("orderId").is(orderId));
        Update update = new Update();
        update.set("fromCorp", fromCorp);
        update.set("fromCorpId", fromCorpId);
        mongoTemplate.updateMulti(query, update, MongoDBName.NAME_CODE);
    }

    public void updateOrderId(String orderId, String newOrderId) {

        Query query = new Query();
        query.addCriteria(new Criteria().and("orderId").is(orderId));
        Update update = new Update();
        update.set("orderId", newOrderId);
        mongoTemplate.updateMulti(query, update, MongoDBName.NAME_CODE);
    }

    public void deleteCode(DeleteRequest deleteRequest) {
        Query query = new Query();
        if (deleteRequest.getId() != null) {
            query.addCriteria(Criteria.where("id").is(deleteRequest.getId()));
        }
        if (deleteRequest.getOrderId() != null) {
            query.addCriteria(Criteria.where("orderId").is(deleteRequest.getOrderId()));
        }
        mongoTemplate.remove(query, WarehouseEntity.class, MongoDBName.NAME_CODE);
    }

}
