package com.glxp.udi.admin.mongo.service;

import com.glxp.udi.admin.entity.inout.WarehouseEntity;
import com.glxp.udi.admin.entity.inout.WarehousetempEntity;
import com.glxp.udi.admin.mongo.MongoDBName;
import com.glxp.udi.admin.mongo.utils.MongoPageHelper;
import com.glxp.udi.admin.mongo.utils.PageResult;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.WarehouseQueryRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Service
public class WarehouseTempMdService {
    @Autowired
    private MongoTemplate mongoTemplate;
    @Resource
    MongoPageHelper mongoPageHelper;


    public PageResult<WarehousetempEntity> filterWarehouseByPage(WarehouseQueryRequest warehouseQueryRequest) {

        Query query = new Query();
        if (warehouseQueryRequest.getCode() != null && !warehouseQueryRequest.getCode().isEmpty()) {
            query.addCriteria(Criteria.where("code").is(warehouseQueryRequest.getCode()));
        }
        if (warehouseQueryRequest.getCorpOrderId() != null && !warehouseQueryRequest.getCorpOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("corpOrderId").is(warehouseQueryRequest.getCorpOrderId()));
        }
        if (warehouseQueryRequest.getOrderId() != null && !warehouseQueryRequest.getOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("orderId").is(warehouseQueryRequest.getOrderId()));
        }
        if (warehouseQueryRequest.getCustomerId() != null) {
            query.addCriteria(Criteria.where("customerId").is(warehouseQueryRequest.getCustomerId()));
        }
        return mongoPageHelper.pageQuery(query, WarehousetempEntity.class,
                warehouseQueryRequest.getLimit(), warehouseQueryRequest.getPage(), warehouseQueryRequest.getLastId(), MongoDBName.NAME_CODE_ERROR);
    }

    public List<WarehousetempEntity> filterWarehouse(WarehouseQueryRequest warehouseQueryRequest) {

        Query query = new Query();
        if (warehouseQueryRequest.getCode() != null && !warehouseQueryRequest.getCode().isEmpty()) {
            query.addCriteria(Criteria.where("code").is(warehouseQueryRequest.getCode()));
        }
        if (warehouseQueryRequest.getCorpOrderId() != null && !warehouseQueryRequest.getCorpOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("corpOrderId").is(warehouseQueryRequest.getCorpOrderId()));
        }
        if (warehouseQueryRequest.getOrderId() != null && !warehouseQueryRequest.getOrderId().isEmpty()) {
            query.addCriteria(Criteria.where("orderId").is(warehouseQueryRequest.getOrderId()));
        }
        if (warehouseQueryRequest.getCustomerId() != null) {
            query.addCriteria(Criteria.where("customerId").is(warehouseQueryRequest.getCustomerId()));
        }
        return mongoTemplate.find(query, WarehousetempEntity.class, MongoDBName.NAME_CODE_ERROR);
    }

    public List<WarehousetempEntity> findTempByOrderId(String orderId) {
        Query query = new Query(Criteria.where("orderId").is(orderId));
        List<WarehousetempEntity> result = mongoTemplate.find(query, WarehousetempEntity.class, MongoDBName.NAME_CODE_ERROR);
        return result;
    }

    public void insertTempWarehouses(List<WarehousetempEntity> warehouseEntities) {
        if (warehouseEntities != null) {
            Collection<WarehousetempEntity> warehousetempEntityCollection = new ArrayList<>();
            warehousetempEntityCollection.addAll(warehouseEntities);
            mongoTemplate.insert(warehousetempEntityCollection, MongoDBName.NAME_CODE_ERROR);
        }
    }

    public void deleteTempWarehouses(List<String> ids) {
        if (ids != null) {
            for (String id : ids) {
                Query query = new Query();
                query.addCriteria(Criteria.where("id").is(id));
                mongoTemplate.remove(query, MongoDBName.NAME_CODE_ERROR);
            }
        }
    }


    public void deleteCode(DeleteRequest deleteRequest) {
        Query query = new Query();
        if (deleteRequest.getId() != null) {
            query.addCriteria(Criteria.where("id").is(deleteRequest.getId()));
        }
        if (deleteRequest.getOrderId() != null) {
            query.addCriteria(Criteria.where("orderId").is(deleteRequest.getOrderId()));
        }
        mongoTemplate.remove(query, WarehouseEntity.class, MongoDBName.NAME_CODE_ERROR);
    }


}
