package com.glxp.udi.admin.service.auth.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.udi.admin.dao.auth.AuthAdminDao;
import com.glxp.udi.admin.dao.auth.CustomerContacDao;
import com.glxp.udi.admin.dao.auth.CustomerInfoDao;
import com.glxp.udi.admin.entity.auth.AuthRole;
import com.glxp.udi.admin.entity.auth.CustomerDetailEntity;
import com.glxp.udi.admin.entity.auth.CustomerInfoEntity;
import com.glxp.udi.admin.req.auth.CustomerInfoFilterRequest;
import com.glxp.udi.admin.service.auth.AuthRoleService;
import com.glxp.udi.admin.service.auth.CustomerInfoService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@Service
public class CustomerInfoServiceImpl implements CustomerInfoService {


    @Resource
    private CustomerInfoDao customerInfoDao;

    @Resource
    private CustomerContacDao customerContacDao;

    @Resource
    private AuthAdminDao authAdminDao;

    @Resource
    private AuthRoleService authRoleService;

    @Override
    public List<CustomerInfoEntity> filterCustomerInfo(CustomerInfoFilterRequest customerInfoFilterRequest) {
        if (customerInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        int offset = (customerInfoFilterRequest.getPage() - 1) * customerInfoFilterRequest.getLimit();
        PageHelper.offsetPage(offset, customerInfoFilterRequest.getLimit());
        return customerInfoDao.filterCustomerInfo(customerInfoFilterRequest);
    }

    @Override
    public List<CustomerDetailEntity> filterDetailCustomer(CustomerInfoFilterRequest customerInfoFilterRequest) {
        if (customerInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        int offset = (customerInfoFilterRequest.getPage() - 1) * customerInfoFilterRequest.getLimit();
        PageHelper.offsetPage(offset, customerInfoFilterRequest.getLimit());
        List<CustomerDetailEntity> customerDetailEntities = customerInfoDao.filterDetailCustomerInfo(customerInfoFilterRequest);
        for (int i = 0; i < customerDetailEntities.size(); i++) {
            CustomerDetailEntity customerDetailEntity = customerDetailEntities.get(i);
            int count = authAdminDao.findCountByCustomerId(customerDetailEntity.getCustomerId());
            customerDetailEntities.get(i).setCount(count + "");
            if(!customerDetailEntity.getRoleId().equals("null") && !customerDetailEntity.getRoleId().equals("") && customerDetailEntity.getRoleId()!=null ){
                AuthRole authRole = authRoleService.findById(Long.parseLong(customerDetailEntity.getRoleId() + ""));
                if (authRole != null)
                    customerDetailEntities.get(i).setRole(authRole.getName());
            }

        }
        return customerDetailEntities;
    }

    @Override
    public boolean insertCustomerInfo(CustomerInfoEntity customerInfoEntity) {
        return customerInfoDao.insertCustomerInfo(customerInfoEntity);
    }

    @Override
    public boolean updateCustomerInfo(CustomerInfoEntity customerInfoEntity) {
        return customerInfoDao.updateCustomerInfo(customerInfoEntity);
    }

    @Override
    public boolean modifyCustomerInfo(CustomerDetailEntity customerDetailEntity) {
        return false;
    }

    @Override
    public boolean deleteById(Long id) {
        return customerInfoDao.deleteById(id);
    }

    @Override
    public CustomerInfoEntity selectById(Long customerId) {
        return customerInfoDao.selectById(customerId);
    }

    @Override
    public CustomerDetailEntity selectDetail(Long customerId) {
        return customerInfoDao.selectDetail(customerId);
    }
}
