package com.glxp.udi.admin.service.info.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.udi.admin.dao.info.IdentifierDao;
import com.glxp.udi.admin.entity.info.IdentifierEntity;
import com.glxp.udi.admin.req.info.DeleteIdentifierRequest;
import com.glxp.udi.admin.req.info.FilterIdentifierRequest;
import com.glxp.udi.admin.service.info.IdentifierService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@Service
public class IdentifierServiceImpl implements IdentifierService {

    @Resource
    IdentifierDao identifierDao;

    @Override
    public boolean insertIdentifier(IdentifierEntity identifierEntity) {
        return identifierDao.insertIdentifier(identifierEntity);
    }

    @Override
    public boolean insertIdentifierList(List<IdentifierEntity> identifierEntities) {
        return identifierDao.insertIdentifierList(identifierEntities);
    }

    @Override
    public boolean deleteIdentifier(DeleteIdentifierRequest deleteIdentifierRequest) {
        return identifierDao.deleteIdentifier(deleteIdentifierRequest);
    }

    @Override
    public boolean deleteIdentifierByFacidentcode(String UDI_FacIdentCode_FK) {
        return identifierDao.deleteIdentifierByFacidentcode(UDI_FacIdentCode_FK);
    }

    @Override
    public boolean modifyIdentifier(IdentifierEntity identifierEntity) {
        return identifierDao.modifyIdentifier(identifierEntity);
    }

    @Override
    public List<IdentifierEntity> selectByfacident(FilterIdentifierRequest filterIdentifierRequest) {
        if (filterIdentifierRequest == null) {
            return Collections.emptyList();
        }
        int offset = (filterIdentifierRequest.getPage() - 1) * filterIdentifierRequest.getLimit();
        PageHelper.offsetPage(offset, filterIdentifierRequest.getLimit());
        List<IdentifierEntity> identifierEntities = identifierDao.selectByfacident(filterIdentifierRequest);
        return identifierEntities;
    }

    @Override
    public String randomCode(FilterIdentifierRequest filterIdentifierRequest) {
        return identifierDao.randomCode(filterIdentifierRequest);
    }

    @Override
    public boolean isUsed(String facidentcode) {
        if (identifierDao.isUsed(facidentcode) == 0) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isUsedIdentifyCode(String code) {
        if (identifierDao.isUsedIdentifyCode(code) == 0) {
            return false;
        }
        return true;
    }

    @Override
    public boolean updateIsUsed(IdentifierEntity identifierEntity) {
        return identifierDao.updateIsUsed(identifierEntity);
    }

    @Override
    public List<IdentifierEntity> selectIdentifyCode(String code) {
        List<IdentifierEntity> identifierEntities = identifierDao.selectIdentifyCode(code);
        return identifierEntities;
    }

    @Override
    public boolean isSptmUsed(String code) {
        List<IdentifierEntity> identifierEntities = selectIdentifyCode(code);
        if (identifierEntities != null && identifierEntities.size() > 0) {
            for (IdentifierEntity identifierEntity : identifierEntities) {
                if (identifierEntity.getIsSPTM().equals("1")) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public boolean isUdiUsed(String code) {
        List<IdentifierEntity> identifierEntities = selectIdentifyCode(code);
        if (identifierEntities != null && identifierEntities.size() > 0) {
            for (IdentifierEntity identifierEntity : identifierEntities) {
                if (identifierEntity.getIsUDI().equals("1")) {
                    return true;
                }
            }
        }
        return false;
    }
}
