package com.glxp.udi.admin.service.inout.impl;

import com.glxp.udi.admin.dao.udid.*;
import com.glxp.udi.admin.entity.udid.*;
import com.glxp.udi.admin.req.udid.DeviceListRequest;
import com.glxp.udi.admin.req.udid.DeviceSearchRequest;
import com.glxp.udi.admin.service.inout.DeviceService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
public class DeviceServiceImpl implements DeviceService {

    @Resource
    private DeviceMapper deviceMapper;
    @Resource
    private ContactlistMapper contactlistMapper;
    @Resource
    private DeviceclinicalMapper deviceclinicalMapper;
    @Resource
    private DevicepackageMapper devicepackageMapper;
    @Resource
    private DevicestorageMapper devicestorageMapper;

    @Override
    public List<DeviceEntity> listDevices(DeviceListRequest deviceListRequest) {

        if (deviceListRequest == null) {
            return Collections.emptyList();
        }
        if (deviceListRequest.getKey() == null)
            deviceListRequest.setKey("");

        deviceListRequest.setPage(deviceListRequest.getPage() - 1);
        List<Device> devices = deviceMapper.searchBykey(deviceListRequest);
        List<DeviceEntity> deviceEntityList = new ArrayList<>();
        if (devices != null && devices.size() > 0) {
            for (int i = 0; i < devices.size(); i++) {
                Device device = devices.get(i);
                DeviceEntity deviceEntity = new DeviceEntity();
                BeanUtils.copyProperties(device, deviceEntity);
                //
                ContactlistExample contactlistExample = new ContactlistExample();
                ContactlistExample.Criteria contactlistExampleCriteria = contactlistExample.createCriteria();
                contactlistExampleCriteria.andDevicerecordkeyEqualTo(device.getDevicerecordkey());
//                contactlistExampleCriteria.andUuidEqualTo(device.getUuid());
                List<Contactlist> contactlistList = contactlistMapper.selectByExample(contactlistExample);
                deviceEntity.setContactlistList(contactlistList);
                //
                DeviceclinicalExample deviceclinicalExample = new DeviceclinicalExample();
                DeviceclinicalExample.Criteria deviceclinicalExampleCriteria = deviceclinicalExample.createCriteria();
                deviceclinicalExampleCriteria.andUuidEqualTo(device.getUuid());
                List<Deviceclinical> deviceclinicals = deviceclinicalMapper.selectByExample(deviceclinicalExample);
                deviceEntity.setDeviceclinicals(deviceclinicals);
                //
                DevicepackageExample devicepackageExample = new DevicepackageExample();
                DevicepackageExample.Criteria devicepackageExampleCriteria = devicepackageExample.createCriteria();
                devicepackageExampleCriteria.andUuidEqualTo(device.getUuid());
                List<Devicepackage> devicepackages = devicepackageMapper.selectByExample(devicepackageExample);
                deviceEntity.setDevicepackages(devicepackages);
                //
                DevicestorageExample devicestorageExample = new DevicestorageExample();
                DevicestorageExample.Criteria devicestorageExampleCriteria = devicestorageExample.createCriteria();
                devicestorageExampleCriteria.andUuidEqualTo(device.getUuid());
                List<Devicestorage> devicestorages = devicestorageMapper.selectByExample(devicestorageExample);
                deviceEntity.setDevicestorages(devicestorages);
                List<Device> historys = deviceMapper.searchHistory(device.getDevicerecordkey());
                if(historys!=null && historys.size()>0){
                    List<DeviceHistory> deviceHistories = new ArrayList<>();
                    for(Device device1:historys){
                        if(device1.getDevicehistoryrecordkey()!=null){
                            DeviceHistory deviceHistory = new DeviceHistory();
                            deviceHistory.setCpbs(device1.getZxxsdycpbs());
                            deviceHistory.setDeviceHistoryRecordKey(device1.getDevicehistoryrecordkey());
                            deviceHistory.setDeviceRecordKey(device1.getDevicerecordkey());
                            deviceHistories.add(deviceHistory);
                        }
                    }
                    deviceEntity.setDeviceHistories(deviceHistories);
                }
                deviceEntityList.add(deviceEntity);
            }
        }
        return deviceEntityList;
    }


    @Override
    public List<DeviceEntity> search(DeviceSearchRequest deviceSearchRequest) {
        if (deviceSearchRequest == null) {
            return Collections.emptyList();
        }

        deviceSearchRequest.setPage(deviceSearchRequest.getPage() - 1);
        List<Device> devices = deviceMapper.search(deviceSearchRequest);
        List<DeviceEntity> deviceEntityList = new ArrayList<>();
        if (devices != null && devices.size() > 0) {
            for (int i = 0; i < devices.size(); i++) {
                Device device = devices.get(i);
                DeviceEntity deviceEntity = new DeviceEntity();
                BeanUtils.copyProperties(device, deviceEntity);
                //
                ContactlistExample contactlistExample = new ContactlistExample();
                ContactlistExample.Criteria contactlistExampleCriteria = contactlistExample.createCriteria();
                contactlistExampleCriteria.andUuidEqualTo(device.getUuid());
                List<Contactlist> contactlistList = contactlistMapper.selectByExample(contactlistExample);
                deviceEntity.setContactlistList(contactlistList);
                //
                DeviceclinicalExample deviceclinicalExample = new DeviceclinicalExample();
                DeviceclinicalExample.Criteria deviceclinicalExampleCriteria = deviceclinicalExample.createCriteria();
                deviceclinicalExampleCriteria.andUuidEqualTo(device.getUuid());
                List<Deviceclinical> deviceclinicals = deviceclinicalMapper.selectByExample(deviceclinicalExample);
                deviceEntity.setDeviceclinicals(deviceclinicals);
                //
                DevicepackageExample devicepackageExample = new DevicepackageExample();
                DevicepackageExample.Criteria devicepackageExampleCriteria = devicepackageExample.createCriteria();
                devicepackageExampleCriteria.andUuidEqualTo(device.getUuid());
                List<Devicepackage> devicepackages = devicepackageMapper.selectByExample(devicepackageExample);
                deviceEntity.setDevicepackages(devicepackages);
                //
                DevicestorageExample devicestorageExample = new DevicestorageExample();
                DevicestorageExample.Criteria devicestorageExampleCriteria = devicestorageExample.createCriteria();
                devicestorageExampleCriteria.andUuidEqualTo(device.getUuid());
                List<Devicestorage> devicestorages = devicestorageMapper.selectByExample(devicestorageExample);
                deviceEntity.setDevicestorages(devicestorages);

                deviceEntityList.add(deviceEntity);
            }
        }
        return deviceEntityList;
    }

    @Override
    public long seatchTotal(DeviceSearchRequest deviceSearchRequest) {
        return deviceMapper.getTotal(deviceSearchRequest);
    }

    @Override
    public DeviceEntity searchById(String id) {
        DeviceEntity deviceEntity = new DeviceEntity();
        Device device = deviceMapper.selectByPrimaryKey(id);
        BeanUtils.copyProperties(device, deviceEntity);
        ContactlistExample contactlistExample = new ContactlistExample();
        ContactlistExample.Criteria contactlistExampleCriteria = contactlistExample.createCriteria();
        contactlistExampleCriteria.andUuidEqualTo(device.getUuid());
        List<Contactlist> contactlistList = contactlistMapper.selectByExample(contactlistExample);
        deviceEntity.setContactlistList(contactlistList);
        //
        DeviceclinicalExample deviceclinicalExample = new DeviceclinicalExample();
        DeviceclinicalExample.Criteria deviceclinicalExampleCriteria = deviceclinicalExample.createCriteria();
        deviceclinicalExampleCriteria.andUuidEqualTo(device.getUuid());
        List<Deviceclinical> deviceclinicals = deviceclinicalMapper.selectByExample(deviceclinicalExample);
        deviceEntity.setDeviceclinicals(deviceclinicals);
        //
        DevicepackageExample devicepackageExample = new DevicepackageExample();
        DevicepackageExample.Criteria devicepackageExampleCriteria = devicepackageExample.createCriteria();
        devicepackageExampleCriteria.andUuidEqualTo(device.getUuid());
        List<Devicepackage> devicepackages = devicepackageMapper.selectByExample(devicepackageExample);
        deviceEntity.setDevicepackages(devicepackages);
        //
        DevicestorageExample devicestorageExample = new DevicestorageExample();
        DevicestorageExample.Criteria devicestorageExampleCriteria = devicestorageExample.createCriteria();
        devicestorageExampleCriteria.andUuidEqualTo(device.getUuid());
        List<Devicestorage> devicestorages = devicestorageMapper.selectByExample(devicestorageExample);
        deviceEntity.setDevicestorages(devicestorages);
        return deviceEntity;
    }

    @Override
    public Device searchByDeviceRecordKey(String deviceRecordKey) {
        return deviceMapper.searchByDeviceRecordKey(deviceRecordKey);
    }

    @Override
    public Device searchByZxxscpbs(String zxxscpbs) {
        return deviceMapper.searchByZxxscpbs(zxxscpbs);
    }

    @Override
    public boolean deleteHistory(String deviceRecordKey) {
        return deviceMapper.deleteHistory(deviceRecordKey);
    }

    @Override
    public String selectisSame(String deviceRecordKey, String versionNumber) {
        return deviceMapper.selectisSame(deviceRecordKey,versionNumber);
    }

    @Override
    public String selectKey(String zxxsdycpbs) {
        return deviceMapper.selectKey(zxxsdycpbs);
    }

    @Override
    public boolean insertDevice(Device device) {
        deviceMapper.insert(device);
        return true;
    }

    @Override
    public boolean insertDeviceClinical(List<Deviceclinical> deviceclinicals) {
        if (deviceclinicals != null && deviceclinicals.size() > 0) {
            for (int i = 0; i < deviceclinicals.size(); i++) {
                deviceclinicalMapper.insert(deviceclinicals.get(i));
            }
        }
        return true;
    }

    @Override
    public boolean insertDevicestorage(List<Devicestorage> devicestorages) {
        if (devicestorages != null && devicestorages.size() > 0) {
            for (int i = 0; i < devicestorages.size(); i++) {
                devicestorageMapper.insert(devicestorages.get(i));
            }
        }
        return true;
    }

    @Override
    public boolean insertDevicepackage(List<Devicepackage> devicepackages) {
        if (devicepackages != null && devicepackages.size() > 0) {
            for (int i = 0; i < devicepackages.size(); i++) {
                devicepackageMapper.insert(devicepackages.get(i));
            }
        }
        return true;
    }

    @Override
    public boolean insertContactlist(List<Contactlist> contactlists) {
        if (contactlists != null && contactlists.size() > 0) {
            for (int i = 0; i < contactlists.size(); i++) {
                contactlistMapper.insert(contactlists.get(i));
            }
        }
        return true;
    }

    @Override
    public Device searchByBs(String zzxsbs) {
        return deviceMapper.searchByBs(zzxsbs);
    }


}
