package com.glxp.udi.admin.service.inout.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.udi.admin.dao.inout.DruginfoDao;
import com.glxp.udi.admin.dao.inout.DruginfoMapper;
import com.glxp.udi.admin.entity.inout.Druginfo;
import com.glxp.udi.admin.req.inout.DrugFilterRequest;
import com.glxp.udi.admin.req.inout.DrugInfoRequest;
import com.glxp.udi.admin.req.inout.DrugsSaveRequest;
import com.glxp.udi.admin.service.inout.DruginfoService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
public class DruginfoServiceImpl implements DruginfoService {

    @Resource
    private DruginfoDao druginfoDao;

    @Resource
    private DruginfoMapper druginfoMapper;

    @Override
    public List<Druginfo> listDrugInfo(DrugInfoRequest drugInfoRequest) {
        if (drugInfoRequest == null) {
            return Collections.emptyList();
        }
        int offset = (drugInfoRequest.getPage() - 1) * drugInfoRequest.getLimit();
        PageHelper.offsetPage(offset, drugInfoRequest.getLimit());
        List<Druginfo> druginfoList = druginfoDao.listDrugInfo(drugInfoRequest);
        return druginfoList;
    }

    @Override
    public int saveDruginfo(DrugsSaveRequest drugsSaveRequest) {
        List<Druginfo> druginfos = drugsSaveRequest.getDruginfos();
        List<Druginfo> inserts = new ArrayList<>();
        for (Druginfo drug : druginfos) {
            if (drug.getNamecode() != null) {
                inserts.add(drug);
            }
        }
        return druginfoMapper.insertDrugs(inserts);
    }

    /**
     *  List<Druginfo> druginfos = drugsSaveRequest.getDruginfos();
     *         List<Druginfo> inserts = new ArrayList<>();
     *         for (Druginfo drug : druginfos) {
     *             if (drug.getNamecode() != null) {
     *                 inserts.add(drug);
     *             }
     *         }
     *         druginfoMapper.deleteAll();
     *         return druginfoMapper.insertDrugs(inserts);
     */

    @Override
    public List<Druginfo> filterDrugInfo(DrugFilterRequest drugFilterRequest) {
        if (drugFilterRequest == null) {
            return Collections.emptyList();
        }
        int offset = (drugFilterRequest.getPage() - 1) * drugFilterRequest.getLimit();
        PageHelper.offsetPage(offset, drugFilterRequest.getLimit());
        List<Druginfo> druginfoList = druginfoDao.filterDrugInfo(drugFilterRequest);
        return druginfoList;
    }

    @Override
    public List<Druginfo> getSimplyDrugs(String coName, String packRatio, String packSpec) {
        List<Druginfo> druginfos = druginfoDao.getSimplyDrugs(coName, packRatio, packSpec);
        return druginfos;
    }

    @Override
    public List<Druginfo> getDrugsByUuid(String uuid) {
        List<Druginfo> druginfos = druginfoDao.getDrugsByUuid(uuid);
        return druginfos;
    }

    @Override
    public boolean isExit(String nameCode) {
        String data = druginfoDao.isExit(nameCode);
        if (data != null)
            return true;
        else
            return false;
    }

    @Override
    public Druginfo selectByNamecode(String nameCode) {
        return druginfoDao.selectByNamecode(nameCode);
    }

}
