package com.glxp.udi.admin.thread;

import com.glxp.udi.admin.constant.TypeConstant;
import com.glxp.udi.admin.entity.inout.*;
import com.glxp.udi.admin.mongo.service.*;
import com.glxp.udi.admin.req.inout.FilterErpOrderRequest;
import com.glxp.udi.admin.req.inout.OrderFilterRequest;
import com.glxp.udi.admin.req.inout.WarehouseQueryRequest;
import com.glxp.udi.admin.service.inout.UdiInfoService;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
public class InvProductsTrService {


    @Resource
    private OrderMdService orderService;
    @Resource
    private OrderDetailMdService myErpOrderService;
    @Resource
    private WarehouseMdService codesService;
    @Resource
    private UdiInfoService udiInfoService;

    @Resource
    private InvProductMdService invProductService;
    @Resource
    private InvProductDetailMdService invProductDetailService;


    @Async
    public void genInvProducts(String orderId) {
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(orderId);
        List<OrderMdEntity> orderMdEntities = orderService.findAll(orderFilterRequest);
        OrderMdEntity orderEntity = orderMdEntities.get(0);
        if (orderEntity.getStatus().equals("success")) {
            FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
            filterErpOrderRequest.setOrderId(orderId);
            List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterOrderDetails(filterErpOrderRequest);
            if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
                for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {

                    String nameCode = erpOrderEntity.getNameCode();
                    UdiInfoEntity udiInfoEntity = udiInfoService.findByNameCode(nameCode);
                    InvProductEntity invProductEntity = invProductService.selectByUuid(udiInfoEntity.getUuid(), erpOrderEntity.getLotno());
                    if (invProductEntity == null) {
                        invProductEntity = new InvProductEntity();
                        invProductEntity.setRelIdFk(udiInfoEntity.getUuid());
                        invProductEntity.setBatchNo(erpOrderEntity.getLotno());
                        invProductEntity.setCustomerId(orderEntity.getCustomerId() + "");
                        invProductEntity.setExpireDate(erpOrderEntity.getExpireDate());
                        invProductEntity.setProductionDate(erpOrderEntity.getProduceDate());
                        invProductEntity.setGgxh(erpOrderEntity.getPackSpec());
                        invProductEntity.setNameCode(erpOrderEntity.getNameCode());
                        invProductEntity.setProductsName(erpOrderEntity.getCoName());
                        invProductEntity.setYlqxzcrbarmc(erpOrderEntity.getProductCompany());
                        invProductEntity.setZczbhhzbapzbh(erpOrderEntity.getAuthCode());
                        if (orderEntity.getMainAction().equals(TypeConstant.TYPE_OUT)) {
                            invProductEntity.setOutCount(erpOrderEntity.getCount());
                        } else if (orderEntity.getMainAction().equals(TypeConstant.TYPE_PUT)) {
                            invProductEntity.setInCount(erpOrderEntity.getCount());
                        }
                        invProductService.insertInvProduct(invProductEntity);
                        invProductEntity = invProductService.selectByUuid(invProductEntity.getRelIdFk(), invProductEntity.getBatchNo());
                    } else {
                        if (orderEntity.getMainAction().equals(TypeConstant.TYPE_OUT)) {
                            invProductEntity.setOutCount(invProductEntity.getOutCount() + erpOrderEntity.getCount());
                        } else if (orderEntity.getMainAction().equals(TypeConstant.TYPE_PUT)) {
                            invProductEntity.setInCount(invProductEntity.getInCount() + erpOrderEntity.getCount());
                        }
                        invProductService.updateInvProduct(invProductEntity);
                    }
                    WarehouseQueryRequest warehouseQueryRequest = new WarehouseQueryRequest();
                    warehouseQueryRequest.setOrderId(orderId);
                    List<WarehouseEntity> warehouseEntities = codesService.filterWarehouse(warehouseQueryRequest);
                    if (warehouseEntities != null && warehouseEntities.size() > 0) {

                        for (WarehouseEntity warehouseEntity : warehouseEntities) {
                            UdiInfoEntity tempUdi = udiInfoService.findByNameCode(warehouseEntity.getUdi());
                            if (tempUdi.getUuid().equals(erpOrderEntity.getUuidFk())) {
                                if ((warehouseEntity.getBatchNo() == null && erpOrderEntity.getLotno() == null) ||
                                        warehouseEntity.getBatchNo().equals(erpOrderEntity.getLotno())) {
                                    InvProductDetailEntity invProductDetailEntity = new InvProductDetailEntity();
                                    invProductDetailEntity.setProductIdFk(invProductEntity.getId() + "");
                                    invProductDetailEntity.setCode(warehouseEntity.getCode());
                                    invProductDetailEntity.setOrderIdFk(warehouseEntity.getOrderId());
                                    invProductDetailEntity.setCount(warehouseEntity.getCount());
                                    invProductDetailEntity.setAction(warehouseEntity.getAction());
                                    invProductDetailEntity.setMainAction(warehouseEntity.getMainAction());
                                    invProductDetailEntity.setUpdateTime(new Date());
                                    invProductDetailEntity.setCustomerId(orderEntity.getCustomerId() + "");
                                    invProductDetailService.insertInvProduct(invProductDetailEntity);
                                }
                            }
                        }

                    }
                }
            }
        }

    }
}