package com.glxp.udi.admin.thread;

import com.glxp.udi.admin.constant.Constant;
import com.glxp.udi.admin.controller.inout.InoutUtils;
import com.glxp.udi.admin.entity.inout.*;
import com.glxp.udi.admin.entity.udid.UdiEntity;
import com.glxp.udi.admin.mongo.service.OrderDetailMdService;
import com.glxp.udi.admin.mongo.service.OrderMdService;
import com.glxp.udi.admin.mongo.service.WarehouseMdService;
import com.glxp.udi.admin.mongo.service.WarehouseTempMdService;
import com.glxp.udi.admin.req.inout.CommitRequest;
import com.glxp.udi.admin.req.inout.DeleteRequest;
import com.glxp.udi.admin.req.inout.PostOrderRequest;
import com.glxp.udi.admin.req.inout.WarehouseQueryRequest;
import com.glxp.udi.admin.service.info.SetupService;
import com.glxp.udi.admin.service.inout.CodeNeedRlService;
import com.glxp.udi.admin.service.inout.UdiInfoService;
import com.glxp.udi.admin.service.inout.UdiRelevanceService;
import com.glxp.udi.admin.util.DateUtil;
import com.glxp.udi.admin.util.FilterUdiUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class TransInoutService {

    private static final Logger logger = LoggerFactory.getLogger(TransInoutService.class);

    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    SetupService setupService;
    @Resource
    private UdiInfoService udiInfoService;
    @Value("${UCLOD_SERVER_URL}")
    private String ucloudUrl;
    @Value("${UDI_SERVER_URL}")
    private String udiUrl;


    @Resource
    private OrderMdService orderService;
    @Resource
    private WarehouseTempMdService warehouseTempMdService;
    @Resource
    private WarehouseMdService warehouseMdService;
    @Resource
    private OrderDetailMdService orderDetailMdService;
    @Resource
    private CodeNeedRlService codeNeedRlService;
    @Resource
    private InoutUtils inoutUtilService;
    @Resource
    private InvProductsTrService invProductsTrService;

    @Async
    public void transInout(PostOrderRequest postOrderRequest, long customerId) {

        List<PostOrderRequest.PostOrder> postOrders = postOrderRequest.getPostOrder();
        if (postOrders != null && postOrders.size() > 0) {
            for (int i = 0; i < postOrders.size(); i++) {
                PostOrderRequest.PostOrder postOrder = postOrders.get(i);
                List<WarehousetempEntity> warehousetempList = postOrder.getCodes();
                if (warehousetempList != null && warehousetempList.size() > 0) {
                    WarehousetempEntity warehousetempEntity = warehousetempList.get(0);
                    String orderId = System.currentTimeMillis() + i + "";
                    OrderMdEntity orderSaveRequest = new OrderMdEntity();
                    orderSaveRequest.setOrderId(orderId);
                    Date actDate = DateUtil.parseDate(warehousetempEntity.getActDate());
                    orderSaveRequest.setActDate(DateUtil.dateToISODate(actDate));
                    orderSaveRequest.setCustomerId(customerId);
                    orderSaveRequest.setAction(warehousetempEntity.getAction());
                    orderSaveRequest.setActor(warehousetempEntity.getActor());
                    orderSaveRequest.setCorpOrderId(warehousetempEntity.getCorpOrderId());
                    orderSaveRequest.setFromCorpId(warehousetempEntity.getFromCorpId());
                    orderSaveRequest.setMainAction(warehousetempEntity.getMainAction());
                    orderSaveRequest.setFromCorp(warehousetempEntity.getFromCorp());
                    orderSaveRequest.setFromType("手持终端");
                    orderSaveRequest.setStatus(Constant.ORDER_STATUS_PROCESS);
                    orderSaveRequest.setExportStatus("未上传");
                    orderService.insertOrder(orderSaveRequest);
                    for (int index = 0; index < warehousetempList.size(); index++) {
                        warehousetempList.get(index).setOrderId(orderId);
                        warehousetempList.get(index).setCustomerId(customerId);
                    }
                    warehouseTempMdService.insertTempWarehouses(warehousetempList);
                    List<WarehousetempEntity> filterList = warehousetempList;

                    //更新状态，正在处理
                    OrderMdEntity tempEntity = new OrderMdEntity();
                    tempEntity.setOrderId(filterList.get(0).getOrderId());
                    tempEntity.setRemark("正在处理");
                    orderService.updateOrder(tempEntity);

                }
            }

        }
    }


    @Async
    public void repeatCommit(CommitRequest commitRequest) {
        String orderId = commitRequest.getOrderId();
        List<WarehousetempEntity> warehousetempList = warehouseTempMdService.findTempByOrderId(commitRequest.getOrderId());
        if (warehousetempList != null && warehousetempList.size() > 0) {
            String result;
            List<WarehousetempEntity> insertList = new ArrayList<>();
            for (int j = 0; j < warehousetempList.size(); j++) {
                WarehousetempEntity warehousetemp = warehousetempList.get(j);
                result = new InoutUtils().filterCode(warehouseMdService, warehousetemp, setupService);

                if (result == null) {
                    result = inoutUtilService.vailIsUseDy(warehousetemp);
                    if (result == null)
                        insertList.add(warehousetemp);
                    else {
                        OrderMdEntity tempEntity = new OrderMdEntity();
                        tempEntity.setOrderId(orderId);
                        tempEntity.setRemark(result);
                        orderService.updateOrder(tempEntity);
                        insertList.clear();
                        return;
                    }
                } else {
                    OrderMdEntity tempEntity = new OrderMdEntity();
                    tempEntity.setOrderId(orderId);
                    tempEntity.setRemark(result);
                    orderService.updateOrder(tempEntity);
                    insertList.clear();
                    return;
                }
            }
            if (insertList.size() > 0) {
                List<WarehouseEntity> warehouseEntities = insertList.stream().map(e -> {
                    WarehouseEntity warehouseEntity = new WarehouseEntity();
                    BeanUtils.copyProperties(e, warehouseEntity);
                    filterNeedRlCode(warehouseEntity);
                    return warehouseEntity;
                }).collect(Collectors.toList());
                warehouseMdService.insertWarehouses(warehouseEntities);


                //更新状态，成功
                OrderMdEntity tempEntity2 = new OrderMdEntity();
                tempEntity2.setOrderId(orderId);
                tempEntity2.setStatus("success");
                orderService.updateOrder(tempEntity2);


                //删除零时表数据
                DeleteRequest deleteRequest = new DeleteRequest();
                deleteRequest.setOrderId(orderId);
                warehouseTempMdService.deleteCode(deleteRequest);

                WarehouseQueryRequest warehouseQueryRequest = new WarehouseQueryRequest();
                warehouseQueryRequest.setOrderId(orderId);
                List<WarehouseEntity> transData = warehouseMdService.filterWarehouse(warehouseQueryRequest);
                new InoutUtils(udiUrl).generateConstrastDetail(udiInfoService, udiRelevanceService, orderDetailMdService, transData);
                invProductsTrService.genInvProducts(orderId);
            }
        } else {
            return;
        }
    }


    @Async
    public void filterNeedRlCode(WarehouseEntity warehouseEntity) {
        UdiEntity udiEntity = FilterUdiUtils.getUdi(warehouseEntity.getCode());
        if (udiEntity != null) {
            String udi = udiEntity.getUdi();
            UdiInfoEntity udiInfoEntity = udiInfoService.findByNameCode(udi);
            if (udiInfoEntity != null && (udiInfoEntity.getDiType() == 4
                    || (udiInfoEntity.getDiType() == 1 && udiInfoEntity.getIsUseDy() == 1))) {
                CodeNeedRlEntity codeNeedRlEntity = new CodeNeedRlEntity();
                codeNeedRlEntity.setCode(warehouseEntity.getCode());
                codeNeedRlEntity.setOrderIdFk(warehouseEntity.getOrderId());
                BeanUtils.copyProperties(udiInfoEntity, codeNeedRlEntity);
                BeanUtils.copyProperties(udiEntity, codeNeedRlEntity);
                codeNeedRlEntity.setCount(warehouseEntity.getCount());
                codeNeedRlEntity.setCode(warehouseEntity.getCode());
                codeNeedRlEntity.setCustomerId(warehouseEntity.getCustomerId());
                codeNeedRlService.insertCodeNeedRl(codeNeedRlEntity);
            } else {
                logger.error(udi + "产品信息未维护！");
            }

        }
    }


}
