package com.glxp.udi.admin.thread;

import com.glxp.udi.admin.controller.inout.InoutUtils;
import com.glxp.udi.admin.dao.inout.ScheduledDao;
import com.glxp.udi.admin.entity.inout.OrderMdEntity;
import com.glxp.udi.admin.entity.inout.ScheduledEntity;
import com.glxp.udi.admin.entity.inout.WarehouseEntity;
import com.glxp.udi.admin.entity.inout.WarehousetempEntity;
import com.glxp.udi.admin.mongo.service.OrderDetailMdService;
import com.glxp.udi.admin.mongo.service.OrderMdService;
import com.glxp.udi.admin.mongo.service.WarehouseMdService;
import com.glxp.udi.admin.mongo.service.WarehouseTempMdService;
import com.glxp.udi.admin.req.inout.*;
import com.glxp.udi.admin.service.info.SetupService;
import com.glxp.udi.admin.service.inout.UdiInfoService;
import com.glxp.udi.admin.service.inout.UdiRelevanceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Component
@EnableScheduling
public class TransInoutTask implements SchedulingConfigurer {

    final Logger logger = LoggerFactory.getLogger(TransInoutTask.class);

    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    SetupService setupService;
    @Resource
    private UdiInfoService udiInfoService;
    @Value("${UCLOD_SERVER_URL}")
    private String ucloudUrl;
    @Value("${UDI_SERVER_URL}")
    private String udiUrl;


    @Resource
    private OrderMdService orderService;
    @Resource
    private WarehouseTempMdService warehouseTempMdService;
    @Resource
    private WarehouseMdService warehouseMdService;
    @Resource
    private OrderDetailMdService orderDetailMdService;
    @Resource
    protected ScheduledDao scheduledDao;
    @Resource
    TransInoutService transInoutService;
    @Resource
    private InoutUtils inoutUtilService;
    @Resource
    private InvProductsTrService invProductsTrService;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.addTriggerTask(() -> process(),
                triggerContext -> {
                    ScheduledRequest scheduledRequest = new ScheduledRequest();
                    scheduledRequest.setCronName("transInout");
                    ScheduledEntity scheduledEntity = scheduledDao.findScheduled(scheduledRequest);
                    String cron = scheduledEntity.getCron();
                    if (cron.isEmpty()) {
                        logger.error("cron is null");
                    }
                    return new CronTrigger(cron).nextExecutionTime(triggerContext);
                });
    }


    private void process() {
        logger.info("定时任务：零时表转正式表");
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setStatus("process");
        List<OrderMdEntity> orderMdEntities = orderService.findAll(orderFilterRequest);
        if (orderMdEntities != null && orderMdEntities.size() > 0) {
            for (OrderMdEntity orderMdEntity : orderMdEntities) {
                //读数据库最新数据，避免异步时处理两次
                OrderFilterRequest temp = new OrderFilterRequest();
                temp.setId(orderMdEntity.getId());
                List<OrderMdEntity> currents = orderService.findAll(orderFilterRequest);
                if (currents != null && currents.size() > 0) {
                    OrderMdEntity currentEntity = currents.get(0);
                    if (currentEntity.getStatus().equals("process")) {
                        List<WarehousetempEntity> insertList = new ArrayList<>();
                        List<WarehousetempEntity> filterList = warehouseTempMdService.findTempByOrderId(currentEntity.getOrderId());
                        String result;
                        for (int j = 0; j < filterList.size(); j++) {
                            WarehousetempEntity warehousetemp = filterList.get(j);
                            result = new InoutUtils().filterCode(warehouseMdService, warehousetemp, setupService);
                            if (result == null) {
                                result = inoutUtilService.vailIsUseDy(warehousetemp);
                                if (result == null)
                                    insertList.add(warehousetemp);
                                else {
                                    currentEntity.setStatus("error");
                                    orderService.updateOrder(currentEntity);
                                    insertList.clear();
                                    return;
                                }
                            } else {
                                //更新状态，失败
                                currentEntity.setStatus("error");
                                orderService.updateOrder(currentEntity);
                                insertList.clear();
                                break;
                            }
                        }
                        if (insertList.size() > 0) {
                            List<WarehouseEntity> warehouseEntities = insertList.stream().map(e -> {
                                WarehouseEntity warehouseEntity = new WarehouseEntity();
                                BeanUtils.copyProperties(e, warehouseEntity);
                                transInoutService.filterNeedRlCode(warehouseEntity);
                                return warehouseEntity;
                            }).collect(Collectors.toList());
                            warehouseMdService.insertWarehouses(warehouseEntities);

                            //更新状态，成功
                            currentEntity.setStatus("success");
                            orderService.updateOrder(currentEntity);

                            //删除零时表数据
                            DeleteRequest deleteRequest = new DeleteRequest();
                            deleteRequest.setOrderId(currentEntity.getOrderId());
                            warehouseTempMdService.deleteCode(deleteRequest);
                            insertList.clear();

                            //生成订单详情
                            new InoutUtils(udiUrl).generateConstrastDetail(udiInfoService, udiRelevanceService, orderDetailMdService, warehouseEntities);
                            invProductsTrService.genInvProducts(currentEntity.getOrderId());
                        }

                    }
                }
            }
        }


    }
}
